package be.business.connector.chapter4;

import be.business.connector.common.AbstractCommandLine;
import be.business.connector.core.utils.PropertyHandler;
import be.ehealth.business.kmehrcommons.HcPartyUtil;
import be.ehealth.businessconnector.chapterIV.builders.BuilderFactory;
import be.ehealth.businessconnector.chapterIV.builders.ConsultationBuilder;
import be.ehealth.businessconnector.chapterIV.builders.ResponseBuilder;
import be.ehealth.businessconnector.chapterIV.domain.ChapterIVBuilderResponse;
import be.ehealth.businessconnector.chapterIV.domain.ChapterIVKmehrResponseWithTimeStampInfo;
import be.ehealth.businessconnector.chapterIV.domain.ChapterIVReferences;
import be.ehealth.businessconnector.chapterIV.session.ChapterIVService;
import be.ehealth.businessconnector.chapterIV.session.ChapterIVSessionServiceFactory;
import be.ehealth.businessconnector.chapterIV.utils.FolderTypeUtils;
import be.ehealth.technicalconnector.exception.TechnicalConnectorException;
import be.ehealth.technicalconnector.utils.ConnectorIOUtils;
import be.ehealth.technicalconnector.utils.MarshallerHelper;
import be.fgov.ehealth.chap4.protocol.v1.ConsultChap4MedicalAdvisorAgreementResponse;
import be.fgov.ehealth.medicalagreement.core.v1.Kmehrresponse;
import be.fgov.ehealth.standards.kmehr.id.v1.IDPATIENT;
import be.fgov.ehealth.standards.kmehr.schema.v1.AuthorType;
import be.fgov.ehealth.standards.kmehr.schema.v1.FolderType;
import be.fgov.ehealth.standards.kmehr.schema.v1.TransactionType;
import org.joda.time.DateTime;

import java.io.InputStream;
import java.io.PrintStream;

public class Chapter4CommandLine extends AbstractCommandLine{
	private static final PrintStream OUT = System.out;
	
	public static void main(String[] args) throws Exception {

		initializeSystem();
		
		try {
			for (int i = 0; i < args.length; i++) {
				if (args[i].equals("null")) {
					args[i] = null;
				}
			}

			if (args.length == 4
					&& args[0].equals("consultChap4MedicalAdvisorAgreement")) {
				
				InputStream xmlResourceInputStream = ConnectorIOUtils
						.getResourceAsStream(args[1]);
				
				ResponseBuilder responseBuilder = BuilderFactory.getBuilderFactoryForSession().getResponseBuilder();
				ChapterIVService service = ChapterIVSessionServiceFactory.getChapterIVService();
				FolderType folder = parseFolderType(xmlResourceInputStream);
				
				
				for (TransactionType transaction : folder.getTransactions()) {
					AuthorType authorType = new AuthorType();
					authorType.getHcparties().add(HcPartyUtil.createProfessionalParty(PropertyHandler.getInstance().getProperty("pharmacy-holder.ssin"), PropertyHandler.getInstance().getProperty("pharmacy-holder.nihii"), PropertyHandler.getInstance().getProperty("main.kmehr.quality")));
					transaction.setAuthor(authorType);
					transaction.setDate(new DateTime());
					transaction.setTime(new DateTime());
				}


				for (IDPATIENT idPatient : folder.getPatient().getIds()) {
					idPatient.setValue(args[3]);
				}
				
				
				DateTime agreementStartDate = FolderTypeUtils.retrieveConsultationStartDateOrAgreementStartDate(folder);
				
				ChapterIVReferences references = new ChapterIVReferences(true);
				ConsultationBuilder builder = BuilderFactory.getBuilderFactoryForSession().getConsultationBuilder();
				
				ChapterIVBuilderResponse builderResponse = builder.build(folder, true, references, agreementStartDate);
				ConsultChap4MedicalAdvisorAgreementResponse response = service.consultChap4MedicalAdvisorAgreement(builderResponse.getConsultChap4MedicalAdvisorAgreementRequest());
				
				//validate and unseal
				ChapterIVKmehrResponseWithTimeStampInfo chapterIVKmehrResponseWithTimeStampInfo = responseBuilder.validateTimestampAndretrieveChapterIVKmehrResponseWithTimeStampInfo(response);
				
				MarshallerHelper<Kmehrresponse, Kmehrresponse> kmehrResponseMarshaller = new MarshallerHelper<Kmehrresponse, Kmehrresponse>(
						Kmehrresponse.class, Kmehrresponse.class);
				System.out.println(kmehrResponseMarshaller.toString(chapterIVKmehrResponseWithTimeStampInfo.getKmehrresponse()));
			} else {
				if (args.length > 0) {
					OUT.println("ERROR : Invalid number of arguments");
				}
				showHelp();
			}

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Show help.
	 */
	private static void showHelp() {
		// TODO: DENNIS implement help
		OUT.println("No help implemented yet");
	}

	protected static FolderType parseFolderType(
			InputStream xmlResourceInputStream)
			throws TechnicalConnectorException {
		MarshallerHelper<FolderType, FolderType> folderTypeMarshaller = new MarshallerHelper<FolderType, FolderType>(
				FolderType.class, FolderType.class);
		return folderTypeMarshaller.toObject(xmlResourceInputStream);
	}

}
