package be.business.connector.ehealth.varia;

import be.business.connector.common.AbstractCommandLine;
import be.ehealth.business.kmehrcommons.CDConsentBuilderUtil;
import be.ehealth.businessconnector.wsconsent.builders.RequestObjectBuilderFactory;
import be.ehealth.businessconnector.wsconsent.exception.WsConsentBusinessConnectorException;
import be.ehealth.businessconnector.wsconsent.session.WsConsentSessionServiceFactory;
import be.ehealth.technicalconnector.exception.TechnicalConnectorException;
import be.ehealth.technicalconnector.utils.MarshallerHelper;
import be.fgov.ehealth.hubservices.core.v2.*;
import be.fgov.ehealth.standards.kmehr.cd.v1.CDCONSENT;
import be.fgov.ehealth.standards.kmehr.cd.v1.CDCONSENTvalues;
import be.fgov.ehealth.standards.kmehr.id.v1.IDPATIENT;
import be.fgov.ehealth.standards.kmehr.id.v1.IDPATIENTschemes;
import org.joda.time.DateTime;

import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;


public class ConsentCommandLine extends AbstractCommandLine {
private static final PrintStream OUT = System.out;
	
	public static void main(String[] args) throws Exception {
		initializeSystem();
		
		try {
			for (int i = 0; i < args.length; i++) {
				if (args[i].equals("null")) {
					args[i] = null;
				}
			}
			if (args.length == 1 && args[0].equals("retrievePatientConsentForCurrentPatient")) {
				System.out.println("retrievePatientConsentForCurrentPatient called");
				
				List<CDCONSENT> consentList = new ArrayList<CDCONSENT>();
				consentList.add(CDConsentBuilderUtil.createCDConsent("1.0", CDCONSENTvalues.RETROSPECTIVE));
				
				SelectGetPatientConsentType consentType = RequestObjectBuilderFactory.getConsentBuilder().createSelectGetPatientConsent(getPatient(), consentList);
				
				GetPatientConsentRequest consentRequest = RequestObjectBuilderFactory.getRequestObjectBuilder().createGetRequest(getAuthor(), consentType);
				System.out.println("GetPatientConsentRequest: " + convertJaxbObjectToString(consentRequest));
				
				be.ehealth.businessconnector.wsconsent.session.WsConsentService service = WsConsentSessionServiceFactory.getWsConsentService();
		        GetPatientConsentResponse response = service.getPatientConsent(consentRequest);
				System.out.println("GetPatientConsentResponse: " + convertJaxbObjectToString(response));
			} else if (args.length == 1 && args[0].equals("putConsentForCurrentPatient")) {
				System.out.println("putConsentForCurrentPatient called");
					
				ConsentType consentType = createNewConsentTypeForCurrentPatientAndAuthor();
					
				PutPatientConsentRequest putPatientConsentRequest = RequestObjectBuilderFactory.getRequestObjectBuilder().createPutRequest(getAuthor(), consentType);
				System.out.println("PutPatientConsentRequest: " + convertJaxbObjectToString(putPatientConsentRequest));
					
				be.ehealth.businessconnector.wsconsent.session.WsConsentService service = WsConsentSessionServiceFactory.getWsConsentService();
				PutPatientConsentResponse putPatientConsentResponse = service.putPatientConsent(putPatientConsentRequest);
				System.out.println("PutPatientConsentResponse: " + convertJaxbObjectToString(putPatientConsentResponse));
			} else if (args.length == 1 && args[0].equals("revokePatientConsentForCurrentPatient")) {
				System.out.println("revokePatientConsentForCurrentPatient called");
				
				ConsentType consentType = createNewConsentTypeForCurrentPatientAndAuthor();
				consentType.setRevokedate(new DateTime());

				RevokePatientConsentRequest revokePatientConsentRequest = RequestObjectBuilderFactory.getRequestObjectBuilder().createRevokeRequest(getAuthor(), consentType);
				System.out.println("RevokePatientConsentRequest: " + convertJaxbObjectToString(revokePatientConsentRequest));
				
				 be.ehealth.businessconnector.wsconsent.session.WsConsentService service = WsConsentSessionServiceFactory.getWsConsentService();
			     RevokePatientConsentResponse revokePatientConsentResponse = service.revokePatientConsent(revokePatientConsentRequest);
			     System.out.println("RevokePatientConsentResponse: " + convertJaxbObjectToString(revokePatientConsentResponse));
			} else if (args.length == 3 && args[0].equals("retrievePatientConsentForPatient")) {
				System.out.println("retrievePatientConsentForPatient called");
				
				SelectGetPatientConsentType consentType = createNewSelectGetPatientConsentTypeForPatientAndAuthor(args[1], args[2]);
				
				GetPatientConsentRequest consentRequest = RequestObjectBuilderFactory.getRequestObjectBuilder().createGetRequest(getAuthor(), consentType);
				System.out.println("GetPatientConsentRequest: " + convertJaxbObjectToString(consentRequest));
				
				be.ehealth.businessconnector.wsconsent.session.WsConsentService service = WsConsentSessionServiceFactory.getWsConsentService();
		        GetPatientConsentResponse response = service.getPatientConsent(consentRequest);
				System.out.println("GetPatientConsentResponse: " + convertJaxbObjectToString(response));
			} else if (args.length == 3 && args[0].equals("putConsentForPatient")) {
				System.out.println("putConsentForPatient called");
					
				ConsentType consentType = createNewConsentTypeForPatientAndAuthor(args[1], args[2]);
					
				PutPatientConsentRequest putPatientConsentRequest = RequestObjectBuilderFactory.getRequestObjectBuilder().createPutRequest(getAuthor(), consentType);
				System.out.println("PutPatientConsentRequest: " + convertJaxbObjectToString(putPatientConsentRequest));
					
				be.ehealth.businessconnector.wsconsent.session.WsConsentService service = WsConsentSessionServiceFactory.getWsConsentService();
				PutPatientConsentResponse putPatientConsentResponse = service.putPatientConsent(putPatientConsentRequest);
				System.out.println("PutPatientConsentResponse: " + convertJaxbObjectToString(putPatientConsentResponse));
			} else if (args.length == 3 && args[0].equals("revokePatientConsentForPatient")) {
				System.out.println("revokePatientConsentForPatient called");
				
				ConsentType consentType = createNewConsentTypeForPatientAndAuthor(args[1], args[2]);
				consentType.setRevokedate(new DateTime());

				RevokePatientConsentRequest revokePatientConsentRequest = RequestObjectBuilderFactory.getRequestObjectBuilder().createRevokeRequest(getAuthor(), consentType);
				System.out.println("RevokePatientConsentRequest: " + convertJaxbObjectToString(revokePatientConsentRequest));
				
				 be.ehealth.businessconnector.wsconsent.session.WsConsentService service = WsConsentSessionServiceFactory.getWsConsentService();
			     RevokePatientConsentResponse revokePatientConsentResponse = service.revokePatientConsent(revokePatientConsentRequest);
			     System.out.println("RevokePatientConsentResponse: " + convertJaxbObjectToString(revokePatientConsentResponse));
			} else {
				if (args.length > 0) {
					OUT.println("ERROR : Invalid number of arguments");
				}
				showHelp();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Show help.
	 */
	private static void showHelp() {
		OUT.println("Usage :  java <classpath> <-Doption1=option2> " + ConsentCommandLine.class.getPackage() + "." + ConsentCommandLine.class.getName() + " [action] <arg1>...");

		OUT.println(" retrievePatientConsentForCurrentPatient");
		OUT.println("");

		OUT.println(" putConsentForCurrentPatient");
		OUT.println("");

		OUT.println(" revokePatientConsentForCurrentPatient");
		OUT.println("");
		
		OUT.println(" retrievePatientConsentForPatient <arg1> <arg2>");
		OUT.println("	  arg1 : inss");
		OUT.println("	  arg2 : eid number");
		OUT.println("  ex : retrievePatientConsentForPatient 89013036000 591155073000");
		OUT.println("");

		OUT.println(" putConsentForPatient <arg1> <arg2> ");
		OUT.println("	  arg1 : inss");
		OUT.println("	  arg2 : eid number");
		OUT.println("  ex : putConsentForPatient 89013036000 591155073000");
		OUT.println("");

		OUT.println(" revokePatientConsentForPatient <arg1> <arg2>");
		OUT.println("	  arg1 : inss");
		OUT.println("	  arg2 : eid number");
		OUT.println("  ex : revokePatientConsentForPatient 89013036000 591155073000");
		OUT.println("");
		
		OUT.println("");
		OUT.println("-------------- OPTIONS ------------");
		OUT.println(" -Dconfig=<path to the config file> : specified a config file");
		OUT.println(" -Dsession=<path to Xml session file> : specify a session");
	}
	
	private static AuthorWithPatientAndPersonType getAuthor() throws TechnicalConnectorException, WsConsentBusinessConnectorException, InstantiationException {
        AuthorWithPatientAndPersonType author = RequestObjectBuilderFactory.getAuthorBuilder().createAuthor();
        return author;
    }
	
	
	 private static ConsentType createNewConsentTypeForCurrentPatientAndAuthor() throws TechnicalConnectorException, WsConsentBusinessConnectorException, InstantiationException {
	 	List<CDCONSENT> consentList = new ArrayList<CDCONSENT>();
	    consentList.add(CDConsentBuilderUtil.createCDConsent("1.0", CDCONSENTvalues.RETROSPECTIVE));
	    ConsentType consentType = RequestObjectBuilderFactory.getConsentBuilder().createNewConsent(getPatient(), consentList, new DateTime(), getAuthor());
	    return consentType;
	 }
	 
	 private static SelectGetPatientConsentType createNewSelectGetPatientConsentTypeForPatientAndAuthor(String inss, String eidCardNumber) throws TechnicalConnectorException, WsConsentBusinessConnectorException, InstantiationException {
		List<CDCONSENT> consentList = new ArrayList<CDCONSENT>();
		consentList.add(CDConsentBuilderUtil.createCDConsent("1.0", CDCONSENTvalues.RETROSPECTIVE));
		SelectGetPatientConsentType consentType = RequestObjectBuilderFactory.getConsentBuilder().createSelectGetPatientConsent(getDummyPatient(inss, eidCardNumber), consentList);
		return consentType;
	}
	 
	 private static PatientIdType getPatient() throws TechnicalConnectorException {
		 PatientIdType patient = RequestObjectBuilderFactory.getPatientInfoBuilder().readFromEidCard();
	     return patient;
	 }
	 
	 private static ConsentType createNewConsentTypeForPatientAndAuthor(String inss, String eidCardNumber) throws TechnicalConnectorException, WsConsentBusinessConnectorException, InstantiationException {
         List<CDCONSENT> consentList = new ArrayList<CDCONSENT>();
         consentList.add(CDConsentBuilderUtil.createCDConsent("1.0", CDCONSENTvalues.RETROSPECTIVE));
         PatientIdType patient = getDummyPatient(inss, eidCardNumber);
         ConsentType consentType = RequestObjectBuilderFactory.getConsentBuilder().createNewConsent(patient, consentList, new org.joda.time.DateTime(), getAuthor());
         return consentType;
     }
	 
	 private static PatientIdType getDummyPatient(String inss, String eidCardNumber) {
         PatientIdType patient = new PatientIdType();
         // Ask Matching

         //patient.setName("dummyPatient");
         patient.setFamilyname("dummyFamilyName");
         patient.setFirstname("john");

         be.fgov.ehealth.standards.kmehr.id.v1.ObjectFactory objectFactoryIdPatient = new be.fgov.ehealth.standards.kmehr.id.v1.ObjectFactory();
         IDPATIENT idPatient = objectFactoryIdPatient.createIDPATIENT();
         idPatient.setSV("1.0");
         idPatient.setS(IDPATIENTschemes.INSS);
         idPatient.setValue(inss);


         IDPATIENT idPatient2 = objectFactoryIdPatient.createIDPATIENT();
         idPatient2.setSV("1.0");
         idPatient2.setS(IDPATIENTschemes.EID_CARDNO);
         idPatient2.setValue(eidCardNumber);

         patient.getIds().add(idPatient);
         patient.getIds().add(idPatient2);

         return patient;
     }
	 
	 private static <T> String convertJaxbObjectToString(T jaxbObject) {
	        @SuppressWarnings("unchecked")
	        Class<T> clazz = (Class<T>) jaxbObject.getClass();
	        MarshallerHelper<T, T> marshaller = new MarshallerHelper<T, T>(clazz, clazz);
	        String xmlString = marshaller.toString(jaxbObject);
	        return xmlString;
	    }

}
