package be.business.connector.gfddpp.pcdh;

import be.apb.gfddpp.common.exceptions.GFDDPPException;
import be.business.connector.common.AbstractCommandLine;
import be.business.connector.core.exceptions.IntegrationModuleException;
import be.business.connector.gfddpp.domain.ThreeStateBoolean;
import be.business.connector.gfddpp.domain.medicationscheme.protocol.FetchDataEntriesRequest;
import be.business.connector.gfddpp.domain.medicationscheme.protocol.FetchDataEntriesResponse;
import be.business.connector.gfddpp.domain.medicationscheme.protocol.RetrieveTimestampsRequest;
import be.business.connector.gfddpp.domain.medicationscheme.protocol.RetrieveTimestampsResponse;
import be.business.connector.gfddpp.pcdh.mock.PCDHIntegrationModuleMock;
import org.apache.commons.lang.StringUtils;

import javax.xml.bind.JAXBException;
import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class PCDHIntegrationModuleCommandLine extends AbstractCommandLine {

    private static PCDHIntegrationModule module = null;
    /**
     * The Constant OUT.
     */
    private static final PrintStream OUT = System.out;

    public static void main(String[] args) throws Exception {
        for (int i = 0; i < args.length; i++) {
            if (args[i].equals("null")) {
                args[i] = null;
            }
        }

        if (args.length == 9 && args[0].equals("getData")) {
            OUT.println("getData called - for more information look into the log file. ");
            boolean args5 = Boolean.parseBoolean(args[5]);
            boolean args7 = Boolean.parseBoolean(args[7]);

            String xml = null;
            if (!StringUtils.isEmpty(args[8])) {
                ThreeStateBoolean args8 = null;
                if (args[8].equals("true")) {
                    args8 = ThreeStateBoolean.TRUE;
                } else {
                    args8 = ThreeStateBoolean.FALSE;
                }
                xml = getPCDHModule().getData(args[1], args[2], args[3], args[4], args5, loadFile(args[6]), args7, args8);
            } else {
                xml = getPCDHModule().getData(args[1], args[2], args[3], args[4], args5, loadFile(args[6]), args7, ThreeStateBoolean.NULL);
            }

            OUT.println("Result in XML format:");
            OUT.println(xml);
            OUT.println("getData call ended - you can close the window");
        } else if (args.length == 8 && args[0].equals("getPharmacyDetails")) {
            OUT.println("getPharmacyDetails called - for more information look into the log file. ");
            String nihii = null;
            boolean args6 = Boolean.parseBoolean(args[6]);
            if (!StringUtils.isEmpty(args[7])) {
                ThreeStateBoolean args7 = null;
                if (args[7].equals("true")) {
                    args7 = ThreeStateBoolean.TRUE;
                } else {
                    args7 = ThreeStateBoolean.FALSE;
                }
                nihii = getPCDHModule().getPharmacyDetails(args[1], args[2], args[3], args[4], args[5], args6, args7);
            } else {
                nihii = getPCDHModule().getPharmacyDetails(args[1], args[2], args[3], args[4], args[5], args6, ThreeStateBoolean.NULL);
            }

            OUT.println("Result (NIHII):");
            OUT.println(nihii);
            OUT.println("getPharmacyDetails call ended - you can close the window");
        } else if (args.length == 5 && args[0].equals("getDataTypes")) {
            // 1: patientidentifierType
            // 2: patientIdentifier
            // 3: requestPatientSignature
            // 4: therapeuticalRelationShip
            OUT.println("getDataTypes called - for more information look into the log file. ");
            String xml = null;
            boolean args3 = Boolean.parseBoolean(args[3]);
            if (!StringUtils.isEmpty(args[4])) {
                ThreeStateBoolean args4 = null;
                if (args[4].equals("true")) {
                    args4 = ThreeStateBoolean.TRUE;
                } else {
                    args4 = ThreeStateBoolean.FALSE;
                }
                xml = getPCDHModule().getDataTypes(args[1], args[2], args3, args4);
            } else {
                xml = getPCDHModule().getDataTypes(args[1], args[2], args3, ThreeStateBoolean.NULL);
            }

            OUT.println("Result in XML format:");
            OUT.println(xml);
            OUT.println("getDataTypes call ended - you can close the window");
        } else if (args.length == 3 && args[0].equals("getMedicationSchemeTimestamps")) {
            OUT.println("getMedicationSchemeTimestamps called - for more information look into the log file. ");
            getMedicationSchemeTimestamps(args[1], args[2]);
        } else if (args.length == 3 && args[0].equals("getMedicationSchemeEntries")) {
            OUT.println("getMedicationSchemeEntries called - for more information look into the log file. ");
            getMedicationSchemeEntries(args[1], args[2]);
        } else {
            if (args.length > 0) {
                OUT.println("ERROR : Invalid number of arguments");
            }
            showHelp();
        }
    }


    private static void getMedicationSchemeTimestamps(String inputFileLocation, String resultLocation) throws Exception{

        final RetrieveTimestampsRequest request = readXmlFromFile(RetrieveTimestampsRequest.class, inputFileLocation);
        final RetrieveTimestampsResponse response = getPCDHModule().getMedicationSchemeTimestamps(request);
        String xml = writeXmlToFile(RetrieveTimestampsResponse.class, response, resultLocation);
        OUT.println(String.format("Connector responded with xml: [%s]",xml));
        OUT.println(String.format("XML has been written to file [%s]", resultLocation));
    }

    private static void getMedicationSchemeEntries(String inputFileLocation, String resultLocation) throws Exception {
        final FetchDataEntriesRequest request = readXmlFromFile(FetchDataEntriesRequest.class, inputFileLocation);
        final FetchDataEntriesResponse response = getPCDHModule().getMedicationSchemeEntries(request);
        String xml = writeXmlToFile(FetchDataEntriesResponse.class, response, resultLocation);
        OUT.println(String.format("Connector responded with xml: [%s]",xml));
        OUT.println(String.format("XML has been written to file [%s]", resultLocation));
    }



    private static PCDHIntegrationModule initModule() throws Exception {
        initializeSystem();
        if ("true".equals(System.getProperty("mock"))) {
            OUT.println("Using a mock module");
            return new PCDHIntegrationModuleMock();
        }
        return new PCDHIntegrationModuleImpl();
    }


    private static PCDHIntegrationModule getPCDHModule() throws Exception {
        if (module == null) {
            module = initModule();
        }
        return module;
    }

    private static void showHelp() {
        StringBuffer sb = new StringBuffer();

        OUT.println("Usage :  java <classpath> <-Doption1=option2> " + PCDHIntegrationModuleCommandLine.class.getPackage() + "." + PCDHIntegrationModuleCommandLine.class.getName() + " [action] <arg1>...");
        OUT.println(" getData <arg1> <arg2> <arg3> <arg4> <arg5> <arg6> <arg7> <arg8>");
        OUT.println("	  arg1 : patient identifier type");
        OUT.println("	  arg2 : patient identifier ");
        OUT.println("	  arg3 : data type");
        OUT.println("	  arg4 : date range");
        OUT.println("	  arg5 : exclude own data");
        OUT.println("	  arg6 : local data");
        OUT.println("	  arg7 : request patient signature");
        OUT.println("	  arg8 : therapeutical relationship");
        OUT.println("  ex : getData INSZ 81112623980 MEDICALHISTORY DEFAULT false c:/localdata.xml false false");
        OUT.println("");

        OUT.println(" getPharmacyDetails <arg1> <arg2> <arg3> <arg4> <arg5> <arg6> <arg7>");
        OUT.println("	  arg1 : patient identifier type");
        OUT.println("	  arg2 : patient identifier");
        OUT.println("	  arg3 : D-GUID");
        OUT.println("	  arg4 : motivation type");
        OUT.println("	  arg5 : motivation");
        OUT.println("	  arg6 : request patient signature");
        OUT.println("	  arg7 : therapeutical relationship");
        OUT.println("  ex : getPharmacyDetails INSZ 81112623980 7868768768-8796587758-768678-6565674654 WRONG Test false false ");
        OUT.println("");

        OUT.println(" getDataTypes <arg1> <arg2> <arg3> <arg4>");
        OUT.println("	  arg1 : patient identifier type");
        OUT.println("	  arg2 : patient identifier ");
        OUT.println("	  arg3 : request patient signature");
        OUT.println("	  arg4 : therapeutical relationship");
        OUT.println("  ex : getDataTypes INSZ 85020125559  false false");
        OUT.println("");

        OUT.println(" getMedicationSchemeTimestamps <arg1> <arg2>");
        OUT.println("	  arg1 : The location of the RetrieveTimestampsRequest xml");
        OUT.println("	  arg2 : The location to which the RetrieveTimestampsResponse xml file will be stored");
        OUT.println("  ex : getMedicationSchemeTimestamps C:/retrieveTimestampsRequest.xml C:/retrieveTimestampsRespone.xml");
        OUT.println("");

        OUT.println(" getMedicationSchemeEntries <arg1> <arg2>");
        OUT.println("	  arg1 : The location of the FetchDataEntriesRequest xml");
        OUT.println("	  arg2 : The location to which the FetchDataEntriesResponse xml file will be stored");
        OUT.println("  ex : getMedicationSchemeEntries C:/fetchDataEntriesRequest.xml C:/fetchDataEntriesResponse.xml");
        OUT.println("");

        OUT.println("");
        OUT.println("-------------- OPTIONS ------------");
        OUT.println(" -Dconfig=<path to the config file> : specified a config file");
        OUT.println(" -Dsession=<path to Xml session file> : specify a session");

        OUT.println(sb.toString());
    }
}
