package be.business.connector.gfddpp.tipsystem;

import be.business.connector.common.AbstractCommandLine;
import be.business.connector.gfddpp.domain.ThreeStateBoolean;
import be.business.connector.gfddpp.domain.medicationscheme.protocol.StoreDataEntriesRequest;
import be.business.connector.gfddpp.domain.medicationscheme.protocol.StoreDataEntriesResponse;
import be.business.connector.gfddpp.tipsystem.mock.TIPSystemIntegrationModuleMock;
import org.apache.commons.lang.StringUtils;

import java.io.PrintStream;

public class TIPSystemIntegrationModuleCommandLine extends AbstractCommandLine {

    private static TIPSystemIntegrationModule module = null;
    private static final PrintStream OUT = System.out;

    public static void main(String[] args) throws Exception {
        for (int i = 0; i < args.length; i++) {
            OUT.println(args[i]);
            if (args[i].equals("null")) {
                args[i] = null;
            }
        }

        if (args.length == 6 && args[0].equals("registerData")) {

            OUT.println("RegisterData called - for more information look into the log file. ");
            boolean args3 = Boolean.parseBoolean(args[3]);

            String xml;
            ThreeStateBoolean args4 = null;
            if (StringUtils.isEmpty(args[4])) {
                args4 = ThreeStateBoolean.NULL;
            } else if (args[4].equals("true")) {
                args4 = ThreeStateBoolean.TRUE;
            } else if (args[4].equals("false")) {
                args4 = ThreeStateBoolean.FALSE;
            }

            xml = getModule().registerData(loadFile(args[1]), args[2], args3, args4);
            writeFile(args[5], xml);
            OUT.println("Result in XML format:");
            OUT.println(xml);
            OUT.println("RegisterData call ended - you can close the window");

        } else if (args.length == 3 && args[0].equals("archivePrescription")) {

            OUT.println("ArchivePrescription called - for more information look into the log file. ");
            String xml = getModule().archivePrescription(args[1]);
            writeFile(args[2], xml);
            OUT.println("Result in XML format:");
            OUT.println(xml);
            OUT.println("ArchivePrescription call ended - you can close the window");

        } else if (args.length == 4 && args[0].equals("archivePrescriptionComment")) {

            OUT.println("ArchivePrescriptionComment called - for more information look into the log file. ");
            String xml = getModule().archivePrescriptionComment(args[1], new String(loadFile(args[2])));
            writeFile(args[3], xml);
            OUT.println("Result in XML format:");
            OUT.println(xml);
            OUT.println("ArchivePrescriptionComment call ended - you can close the window");

        } else if (args.length == 6 && args[0].equals("updateData")) {
            OUT.println("UpdateData called - for more information look into the log file. ");
            boolean args3 = Boolean.parseBoolean(args[3]);

            String xml;
            if (!StringUtils.isEmpty(args[4])) {
                ThreeStateBoolean args4;
                if (args[4].equals("true")) {
                    args4 = ThreeStateBoolean.TRUE;
                } else {
                    args4 = ThreeStateBoolean.FALSE;
                }
                xml = getModule().updateData(loadFile(args[1]), args[2], args3, args4);
            } else {
                xml = getModule().updateData(loadFile(args[1]), args[2], args3, ThreeStateBoolean.NULL);
            }

            OUT.println("Result in XML format:");
            OUT.println(xml);
            writeFile(args[5], xml);
            OUT.println("UpdateData call ended - you can close the window");

        } else if (args.length == 5 && args[0].equals("revokeData")) {
            OUT.println("RevokeData called - for more information look into the log file. ");
            boolean args3 = Boolean.parseBoolean(args[3]);
            if (!StringUtils.isEmpty(args[4])) {
                ThreeStateBoolean args4;
                if (args[4].equals("true")) {
                    args4 = ThreeStateBoolean.TRUE;
                } else {
                    args4 = ThreeStateBoolean.FALSE;
                }
                getModule().revokeData(loadFile(args[1]), args[2], args3, args4);
            } else {
                getModule().revokeData(loadFile(args[1]), args[2], args3, ThreeStateBoolean.NULL);
            }

            OUT.println("The operation revokeData does not return any data.");
            OUT.println("RevokeData called ended - you can close the window");

        } else if (args.length >= 1 && args.length <= 3 && args[0].equals("getStatusMessages")) {
            OUT.println("GetStatusMessage called ended - for more information look into the log file.");
            String xml;
            switch (args.length) {
                case 3:
                    xml = getModule().getStatusMessages(args[1], args[2]);
                    break;
                case 2:
                    xml = getModule().getStatusMessages(args[1], null);
                    break;
                default:
                    xml = getModule().getStatusMessages(null, null);
                    break;
            }

            OUT.println("Result in XML format:");
            OUT.println(xml);
            OUT.println("GetStatusMessage called ended - you can close the window");
        } else if (args.length == 3 && args[0].equals("registerMedicationSchemeEntries")) {
            registerMedicationSchemeEntries(args[1], args[2]);
        } else {
            if (args.length > 0) {
                OUT.println("ERROR : Invalid number of arguments");
            }
            showHelp();
        }
        /*Thread.sleep(10*1000L);
		getModule().getControle().shutdown();*/
        Thread.sleep(15 * 60 * 1000L);
        System.exit(0);

    }

    private static void registerMedicationSchemeEntries(final String requestFileLocation, final String resultFileLocation) throws Exception {

        final StoreDataEntriesRequest storeDataEntriesRequest = readXmlFromFile(StoreDataEntriesRequest.class, requestFileLocation);
        final StoreDataEntriesResponse response = getModule().registerMedicationSchemeEntries(storeDataEntriesRequest);
        String xml = writeXmlToFile(StoreDataEntriesResponse.class, response, resultFileLocation);
        OUT.println(String.format("Connector responded with xml: [%s]",xml));
        OUT.println(String.format("XML has been written to file [%s]", resultFileLocation));

    }

    private static TIPSystemIntegrationModule initModule() throws Exception {
        initializeSystem();
        if ("true".equals(System.getProperty("mock"))) {
            OUT.println("Using a mock module");
            return new TIPSystemIntegrationModuleMock();
        }
        return new TIPSystemIntegrationModuleImpl();
    }

    private static TIPSystemIntegrationModule getModule() throws Exception {
        if (module == null) {
            module = initModule();
        }
        return module;
    }

    private static void showHelp() {
        StringBuilder sb = new StringBuilder();

        OUT.println("Usage :  java <classpath> <-Doption1=option2> " + TIPSystemIntegrationModuleCommandLine.class.getPackage() + "." + TIPSystemIntegrationModuleCommandLine.class.getName() + " [action] <arg1>...");

        OUT.println(" registerData <arg1> <arg2> <arg3> <arg4> <arg5>");
        OUT.println("	  arg1 : single message");
        OUT.println("	  arg2 : tip id");
        OUT.println("	  arg3 : request patient signature");
        OUT.println("	  arg4 : patient consent");
        OUT.println("	  arg5 : output file name");
        OUT.println("  ex : registerData c:\\sample-smoaMessage-deliveryHuman.xml 0406753266 false false c:\\output\\file.xml");
        OUT.println("");

        OUT.println(" archivePrescription <arg1> <arg2>");
        OUT.println("	  arg1 : rid");
        OUT.println("	  arg2 : output file name");
        OUT.println("  ex : archivePrescription c:\\sample-archive-prescription-event.xml BEPPKGLC1HLF c:\\output\\file.xml");
        OUT.println("");

        OUT.println(" archivePrescriptionComment <arg1> <arg2> <arg3>");
        OUT.println("	  arg1 : rid");
        OUT.println("	  arg1 : comments");
        OUT.println("	  arg2 : output file name");
        OUT.println("  ex : archivePrescriptionComment c:\\sample-archive-prescription-event.xml BEPPKGLC1HLF c:\\output\\file.xml c:\\input\\comment.txt");
        OUT.println("");

        OUT.println(" updateData <arg1> <arg2> <arg3> <arg4> <arg5>");
        OUT.println("	  arg1 : single message");
        OUT.println("	  arg2 : tip id");
        OUT.println("	  arg3 : request patient signature");
        OUT.println("	  arg4 : patient consent");
        OUT.println("	  arg5 : output file name");
        OUT.println("  ex : updateData c:\\sample-smoaMessage-deliveryHuman.xml 0406753266 false false c:\\output\\file.xml");
        OUT.println("");

        OUT.println(" revokeData <arg1> <arg2> <arg3> <arg4> <arg5>");
        OUT.println("	  arg1 : single message");
        OUT.println("	  arg2 : tip id");
        OUT.println("	  arg3 : request patient signature");
        OUT.println("	  arg4 : patient consent");
        OUT.println("  ex : revokeData c:\\sample-smoaMessage-deliveryHuman.xml 0406753266 false false");
        OUT.println("");

        OUT.println(" getStatusMessages <arg1> <arg2>");
        OUT.println("	  arg1 : S-GUID");
        OUT.println("	  arg2 : D-GUID");
        OUT.println("  ex : getStatusMessages 768768767-6500-0000-0000-57575476745 90809809-87987-7868-0000-908080900");
        OUT.println("");

        OUT.println(" registerMedicationSchemeEntries <arg1> <arg2>");
        OUT.println("	  arg1 : The location of the StoreDataEntriesRequest xml");
        OUT.println("	  arg2 : The location to which the result file will be stored");
        OUT.println("  ex : registerMedicationSchemeEntries C:/storeDataEntriesRequest.xml C:/storeDataEntriesResponse.xml");
        OUT.println("");

        OUT.println("");
        OUT.println("-------------- OPTIONS ------------");
        OUT.println(" -Dconfig=<path to the config file> : specified a config file");
        OUT.println(" -Dsession=<path to Xml session file> : specify a session");

        OUT.println(sb.toString());
    }

}
