package be.business.connector.mycarenet;

import be.business.connector.common.AbstractCommandLine;
import be.business.connector.core.utils.PropertyHandler;
import be.business.connector.mycarenet.domain.Insurability;
import be.business.connector.mycarenet.mock.MyCareNetIntegrationModuleMock;

import java.io.PrintStream;

public class MyCareNetIntegrationModuleCommandLine extends AbstractCommandLine {

    /**
     * The Constant OUT.
     */
    private static final PrintStream OUT = System.out;
    private static final String PACKAGE_NAME = "Recipe Connector";


    public static void main(String[] args) throws Exception {
        System.out.println("lengte argumenten: " + args.length);
        for (int i = 0; i < args.length; i++) {
            if (args[i].equals("null")) {
                args[i] = null;
            }
        }
        if (args.length == 9 && args[0].equals("getInsurabilityForPharmacist")) {
            MyCareNetIntegrationModule module = getMyCareNetModule(args[8]);
            final String pharmacyNihii = PropertyHandler.getInstance().getProperty("pharmacy.nihii");
            final String pharmacyHolderNihii = PropertyHandler.getInstance().getProperty("pharmacy-holder.nihii");
            final String pharmacyHolderSsin = PropertyHandler.getInstance().getProperty("pharmacy-holder.ssin");
            Insurability i = module.getInsurabilityForPharmacist(args[1], args[2], pharmacyHolderNihii, pharmacyHolderSsin, pharmacyNihii, args[3], args[4], args[5], args[6], args[7], PACKAGE_NAME, "12345123451234", "userReference");
            printInsurabilty(i);
        } else {
            if (args.length > 0) {
                OUT.println("ERROR : Invalid number of arguments");
            }
            showHelp();
        }
    }

    private static void printInsurabilty(Insurability i) {
        OUT.println("Messageid :");
        OUT.println(i.getMessageId());
        OUT.println("Insurability :");
        OUT.println(i.getXml());
    }

    private static MyCareNetIntegrationModule getMyCareNetModule(String version) throws Exception {
        MyCareNetIntegrationModule module = initMyCareNetModule(version);
        return module;
    }

    private static MyCareNetIntegrationModule initMyCareNetModule(String version) throws Exception {
        initializeSystem();
        if ("true".equals(System.getProperty("mock"))) {
            return new MyCareNetIntegrationModuleMock();
        } else {
            return "V2".equalsIgnoreCase(version) ? new MyCareNetIntegrationModuleV2Impl() : new MyCareNetIntegrationModuleImpl();
        }
    }

    private static void showHelp() {
        StringBuffer sb = new StringBuffer();

        OUT.println("Usage :  java <classpath> <-Doption1=option2> " + MyCareNetIntegrationModuleCommandLine.class.getPackage() + "." + MyCareNetIntegrationModuleCommandLine.class.getName() + " [action] <arg1>...");
        OUT.println(" getInsurabilityForPharmacist <arg1> <arg2> <arg3> <arg4> <arg5> <arg6> <arg7> <arg8>");
        OUT.println("	  arg1 : username");
        OUT.println("	  arg2 : password");
        OUT.println("	  arg3 : date (yyyy-MM-dd)");
        OUT.println("	  arg4 : type");
        OUT.println("	  arg5 : careReceiverSSIN");
        OUT.println("	  arg6 : careReceiverMutuality");
        OUT.println("	  arg7 : careReceiverRegNrWithMut ");
        OUT.println("	  arg8 : version (V1 or V2) ");
        OUT.println("  ex : getInsurabilityForPharmacist phaapb apbfortest 2015-07-01 F 81112623980 null null V1");
        OUT.println("");
        OUT.println("-------------- OPTIONS ------------");
        OUT.println(" -Dconfig=<path to the config file> : specified a config file");
        OUT.println(" -Dsession=<path to Xml session file> : specify a session");

        OUT.println(sb.toString());
    }
}
