/**
 * Copyright (C) 2010 Recip-e
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package be.business.connector.recipe.executor;

import java.io.File;
import java.io.IOException;
import java.io.PrintStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.bouncycastle.util.encoders.Base64;

import be.business.connector.common.AbstractCommandLine;
import be.business.connector.recipe.domain.PrescriptionStatus;
import be.business.connector.recipe.executor.domain.GetPrescriptionForExecutorResult;
import be.business.connector.recipe.executor.mock.ExecutorIntegrationModuleMock;
import be.recipe.services.core.Page;
import be.recipe.services.executor.GetOpenPrescriptionForExecutor;
import be.recipe.services.executor.GetPrescriptionStatusParam;
import be.recipe.services.executor.GetPrescriptionStatusResult;
import be.recipe.services.executor.ListNotificationsItem;
import be.recipe.services.executor.ListOpenPrescriptionsParam;
import be.recipe.services.executor.ListOpenPrescriptionsResult;
import be.recipe.services.executor.ListReservationsParam;
import be.recipe.services.executor.ListReservationsResult;
import be.recipe.services.executor.ListReservationsResultItem;
import be.recipe.services.executor.ListRidsHistoryParam;
import be.recipe.services.executor.ListRidsHistoryResult;
import be.recipe.services.executor.ListRidsHistoryResultItem;
import be.recipe.services.executor.ListRidsInProcessParam;
import be.recipe.services.executor.ListRidsInProcessResult;

/**
 * The Class ExecutorIntegrationModuleCommandLine.
 */
public class ExecutorIntegrationModuleCommandLine extends AbstractCommandLine {

	/** The Constant OUT. */
	private static final PrintStream OUT = System.out;

	/**
	 * The main method.
	 *
	 * @param args
	 *            the arguments
	 * @throws Exception
	 *             the exception
	 */
	public static void main(final String[] args) throws Exception {
		initializeSystem();
		for (int i = 0; i < args.length; i++) {
			if (args[i].equals("null")) {
				args[i] = null;
			}
		}

		if (args.length == 3 && args[0].equals("getPrescription")) {
			OUT.println("GetPrescription " + args[2] + " called - for more information look into the log file. ");
			final GetPrescriptionForExecutorResult p = getModule(args[2]).getPrescription(args[1]);
			printPrescriptionV4(p);
			OUT.println("GetPrescription " + args[2] + " call ended. You can close the window.");
		} else if (args.length == 2 && args[0].equals("getPrescription")) {
			OUT.println("GetPrescription V1 called - for more information look into the log file. ");
			final GetPrescriptionForExecutorResult p = getModule("V1").getPrescription(args[1]);
			printPrescription(p);
			OUT.println("GetPrescription V1 call ended. You can close the window.");
		} else if (args.length == 3 && args[0].equals("getPrescriptionsAndMarkAsDelivered")) {

			OUT.println("GetPrescriptionsAndMarkAsDelivered called - for more information look into the log file. ");
			// read the RIDs from the file
			final List<String> rids = readRIDs(args[1]);
			OUT.println(rids.size() + " RIDs read.");
			for (final String rid : rids) {
				OUT.println("#" + rid + "#");
			}

			final ExecutorIntegrationModule module = getModule(args[2]);

			for (final String rid : rids) {
				OUT.println("Getting RID: " + rid);

				final GetPrescriptionForExecutorResult p = module.getPrescription(rid);
				printPrescription(p);

				OUT.println("Marking RID " + rid + " as delivered");
				module.markAsDelivered(rid);
				OUT.println("Prescription " + rid + " successfully marked as delivered");
			}
			OUT.println("GetPrescriptionsAndMarkAsDelivered call ended. You can close the window.");
		} else if (args.length == 3 && args[0].equals("markAsDelivered")) {
			OUT.println("markAsDelivered called - for more information look into the log file. ");
			getModule(args[2]).markAsDelivered(args[1]);
			OUT.println("Prescription successfully marked as delivered");
			OUT.println("markAsDelivered call ended. You can close the window.");
		} else if (args.length == 3 && args[0].equals("markAsUndelivered")) {
			OUT.println("markAsUndelivered called - for more information look into the log file. ");
			getModule(args[2]).markAsUndelivered(args[1]);
			OUT.println("Prescription successfully marked as undelivered");
			OUT.println("markAsUndelivered call ended. You can close the window.");
		} else if (args.length == 3 && args[0].equals("markAsArchived")) {
			OUT.println("markAsArchived called - for more information look into the log file. ");
			getModule(args[2]).markAsArchived(args[1]);
			OUT.println("Prescription successfully marked as archived");
			OUT.println("markAsArchived call ended. You can close the window.");
		} else if (args.length == 5 && args[0].equals("createFeedback")) {
			OUT.println("createFeedback called - for more information look into the log file. ");
			getModule(args[4]).createFeedback(args[1], args[2], loadFile(args[3]));
			OUT.println("Feedback sent successfully");
			OUT.println("createFeedback call ended. You can close the window.");
		} else if (args.length == 3 && args[0].equals("listNotifications")) {
			OUT.println("listNotifications called - for more information look into the log file. ");
			final List<ListNotificationsItem> notifications = getModule(args[2]).listNotifications(Boolean.valueOf(args[1]));
			if (notifications == null || notifications.isEmpty()) {
				OUT.println("No pending notification.");
			} else {
				int i = 0;
				for (final ListNotificationsItem it : notifications) {
					OUT.println("--------------------------------------------------");
					OUT.println("Notification : " + (++i));
					OUT.println("Sent by : " + it.getSentBy());
					OUT.println("Sent Date : " + it.getSentDate());
					try {
						OUT.println("Content :\n" + new String(it.getContent()));
					} catch (final Throwable t) {
						t.printStackTrace();
					}

				}
			}
			OUT.println("listNotifications call ended. You can close the window.");
		} else if (args.length == 4 && args[0].equals("revokePrescription")) {
			OUT.println("revokePrescription called - for more information look into the log file. ");
			getModule(args[3]).revokePrescription(args[1], args[2]);
			OUT.println("Prescription successfully revoked");
			OUT.println("revokePrescription call ended. You can close the window.");
		} else if (args.length == 2 && args[0].equals("getPrescriptionStatus")) {
			OUT.println("GetPrescriptionStatus called - for more information look into the log file. ");
			final GetPrescriptionStatusParam statusParam = new GetPrescriptionStatusParam();
			statusParam.setRid(args[1]);
			final GetPrescriptionStatusResult p = new ExecutorIntegrationModuleV4Impl().getData(statusParam);
			printPrescriptionStatus(args[1], p);
			OUT.println("GetPrescriptionStatus call ended. You can close the window.");
		} else if (args.length == 3 && args[0].equals("getOpenPrescriptionList")) {
			OUT.println("ListOpenPrescription called - for more information look into the log file. ");
			final ListOpenPrescriptionsParam param = new ListOpenPrescriptionsParam();
			param.setPatientId(args[1]);
			final Page page = new Page();
			page.setPageNumber(java.math.BigInteger.valueOf(Integer.parseInt(args[2])));
			param.setPage(page);
			final ListOpenPrescriptionsResult p = new ExecutorIntegrationModuleV4Impl().getData(param);
			printOpenPrescriptionList(args[1], p);
			OUT.println("ListOpenPrescription call ended. You can close the window.");
		} else if (args.length == 1 && args[0].equals("getAllRidsInProcess")) {
			OUT.println("GetAllRidInProcess called - for more information look into the log file. ");
			final ListRidsInProcessParam param = new ListRidsInProcessParam();
			final ListRidsInProcessResult p = new ExecutorIntegrationModuleV4Impl().getData(param);
			printListRidsInProcessResult(p);
			OUT.println("ListOpenPrescription call ended. You can close the window.");
		} else if (args.length == 3 && args[0].equals("listPrescriptionHistory")) {
			OUT.println("ListRidsHistory called - for more information look into the log file. ");
			final ListRidsHistoryParam param = new ListRidsHistoryParam();
			param.setPatientId(args[1]);
			final Page page = new Page();
			page.setPageNumber(java.math.BigInteger.valueOf(Integer.parseInt(args[2])));
			param.setPage(page);
			final ListRidsHistoryResult p = new ExecutorIntegrationModuleV4Impl().getData(param);
			printListRidsHistoryResult(args[1], p);
			OUT.println("ListRidsHistory call ended. You can close the window.");
		} else if (args.length == 1 && args[0].equals("getAllReservations")) {
			OUT.println("getAllReservations called - for more information look into the log file. ");
			final ListReservationsParam param = new ListReservationsParam();
			final ListReservationsResult p = new ExecutorIntegrationModuleV4Impl().getData(param);
			printListReservationsResult(p);
			OUT.println("getAllReservations call ended. You can close the window.");
		} else if (args.length == 1 && args[0].equals("getPrescription")) {
			OUT.println("getPrescription called - for more information look into the log file. ");
			final ListReservationsParam param = new ListReservationsParam();
			final ListReservationsResult p = new ExecutorIntegrationModuleV4Impl().getData(param);
			printListReservationsResult(p);
			OUT.println("getAllReservations call ended. You can close the window.");
		}else {
			if (args.length > 0) {
				OUT.println("ERROR : Invalid number of arguments");
			}
			showHelp();
		}

	}

	/**
	 * Read ri ds.
	 *
	 * @param fileName
	 *            the file name
	 * @return the list
	 */
	private static List<String> readRIDs(final String fileName) throws IOException {
		return FileUtils.readLines(new File(fileName));
	}

	/**
	 * This method will print a prescription.
	 *
	 * @param p
	 *            the p
	 */
	private static void printPrescription(final GetPrescriptionForExecutorResult p) {
		OUT.println("Prescription successfully retrieved : " + p.getRid());
		OUT.println("KeyID : " + p.getEncryptionKeyId());
		OUT.println("PrescriptionType : " + p.getPrescriptionType());
		OUT.println("Timestamp (RFC3161) : " + p.getTimestampingId());
		OUT.println("CreationDate : " + (p.getCreationDate() != null ? p.getCreationDate().getTime() : null));
		OUT.println("Prescriber :" + p.getPrescriberId());
		OUT.println("Patient :" + p.getPatientId());
		OUT.println("Feedback Allowed :" + p.isFeedbackAllowed());
		OUT.println("Key :" + new String(Base64.encode(p.getEncryptionKey())));
		OUT.println("Sealed Content :" + new String(Base64.encode(p.getSealedContent())));
		OUT.println("Clear Content :");
		OUT.println(new String(p.getPrescription()));
		OUT.println("Insurability :");
		OUT.println(p.getInsurabilityResponse());

	}
	
	private static void printPrescriptionV4(final GetPrescriptionForExecutorResult p) {
		OUT.println("Prescription successfully retrieved : " + p.getRid());
		OUT.println("KeyID : " + p.getEncryptionKeyId());
		OUT.println("PrescriptionType : " + p.getPrescriptionType());
		OUT.println("Timestamp (RFC3161) : " + p.getTimestampingId());
		OUT.println("CreationDate : " + (p.getCreationDate() != null ? p.getCreationDate().getTime() : null));
		OUT.println("Prescriber :" + p.getPrescriberId());
		OUT.println("Patient :" + p.getPatientId());
		OUT.println("Feedback Allowed :" + p.isFeedbackAllowed());
		OUT.println("Key :" + new String(Base64.encode(p.getEncryptionKey())));
		OUT.println("Sealed Content :" + new String(Base64.encode(p.getSealedContent())));
		OUT.println("Clear Content :");
		OUT.println(new String(p.getPrescription()));
		OUT.println("Insurability :");
		OUT.println(p.getInsurabilityResponse());
		OUT.println("Expiration date :");
		final DateFormat dateTimeFormatter = new SimpleDateFormat("dd-MM-yyyy");
		if (p.getExpirationDate() != null && p.getExpirationDate().getTime() != null) {
			OUT.println(dateTimeFormatter.format(p.getExpirationDate().getTime()));
		}
	}

	private static void printPrescriptionStatus(final String rid, final GetPrescriptionStatusResult p) {
		OUT.println("Prescription status for " + rid + " : " + p.getPrescriptionStatus());
	}

	private static void printListRidsHistoryResult(final String patientId, final ListRidsHistoryResult p) {
		OUT.println("ListOpenPrescription for " + patientId);
		for (final ListRidsHistoryResultItem item : p.getItems()) {
			OUT.println("Prescription: rid : " + item.getRid() + " Status: " + item.getPrescriptionStatus());
		}
	}

	private static void printListReservationsResult(final ListReservationsResult p) {
		for (final ListReservationsResultItem item : p.getItems()) {
			OUT.println("Reservation: " + item.getRid());
		}
	}

	private static void printOpenPrescriptionList(final String patient, final ListOpenPrescriptionsResult p) {
		final DateFormat dateTimeFormatter = new SimpleDateFormat("dd-MM-yyyy");
		OUT.println("ListOpenPrescription for " + patient);
		for (final GetOpenPrescriptionForExecutor openPrescription : p.getPrescriptions()) {
			String expirationDateText = "unknown";
			if (openPrescription.getExpirationDate() != null && openPrescription.getExpirationDate().getTime() != null) {
				expirationDateText = dateTimeFormatter.format(openPrescription.getExpirationDate().getTime());
			}
			OUT.println("Prescription: rid : " + openPrescription.getRid() + " ExpirationDate : " + expirationDateText + " : Prescriber: "
					+ openPrescription.getPrescriberId() + " Status: " + PrescriptionStatus.NotDelivered);
		}
	}

	private static void printListRidsInProcessResult(final ListRidsInProcessResult result) {
		OUT.println("ListRidsInProcessResult");
		for (final String rid : result.getRids()) {
			OUT.println(rid + " status: " + PrescriptionStatus.InProcess);
		}
	}

	private static ExecutorIntegrationModule initModule(final String version) throws Exception {
		initializeSystem();
		if ("true".equals(System.getProperty("dev"))) {
			OUT.println("Using a V4Dev module");
			return new ExecutorIntegrationModuleDevV4Impl();
		} else if ("true".equals(System.getProperty("mock"))) {
			return new ExecutorIntegrationModuleMock();
		} else if ("V3".equalsIgnoreCase(version)) {
			return new ExecutorIntegrationModuleV3Impl();
		} else if ("V4".equalsIgnoreCase(version)) {
			return new ExecutorIntegrationModuleV4Impl();
		} else {
			return new ExecutorIntegrationModuleImpl();
		}
	}

	/**
	 * Gets the module.
	 * 
	 * @return the module
	 */
	private static ExecutorIntegrationModule getModule(final String version) throws Exception {
		return initModule(version);
	}

	/**
	 * Show help.
	 */
	private static void showHelp() {
		final StringBuffer sb = new StringBuffer();

		OUT.println("Usage :  java <classpath> <-Doption1=option2> " + ExecutorIntegrationModuleCommandLine.class.getName()
				+ " [action] <arg1>...");

		OUT.println("");
		OUT.println("------------ Action available ------------");
		OUT.println(" getPrescription <arg1> <arg2>");
		OUT.println("	  arg1 : RID");
		OUT.println("	  arg2 : VERSION");
		OUT.println("  ex : getPrescription BEPPabcdefgh V3");
		OUT.println("");

		OUT.println(" markAsDelivered <arg1> <arg2>");
		OUT.println("	  arg1 : RID");
		OUT.println("	  arg2 : VERSION");
		OUT.println("  ex : markAsDelivered BEPPabcdefgh V3");
		OUT.println("");

		OUT.println(" markAsUndelivered <arg1> <arg2>");
		OUT.println("	  arg1 : RID");
		OUT.println("	  arg2 : VERSION");
		OUT.println("  ex : markAsUndelivered BEPPabcdefgh V3");
		OUT.println("");

		OUT.println(" markAsArchived <arg1> <arg2>");
		OUT.println("	  arg1 : RID");
		OUT.println("	  arg2 : VERSION");
		OUT.println("  ex : markAsArchived BEPPabcdefgh V3");
		OUT.println("");

		OUT.println(" createFeedback <arg1> <arg2> <arg3> <arg4>");
		OUT.println("	  arg1 : PrescriberId");
		OUT.println("	  arg2 : RID");
		OUT.println("	  arg3 : path to the feedback XML file");
		OUT.println("	  arg4 : VERSION");
		OUT.println("  ex : createFeedback 12659389004 BEPPabcdefgh feedback.xml V3");
		OUT.println("");

		OUT.println(" listNotifications <arg1> <arg2>");
		OUT.println("	  arg1 : read flag");
		OUT.println("	  arg2 : VERSION");
		OUT.println("  ex : listNotifications true V3");
		OUT.println("");

		OUT.println(" revokePrescription <arg1> <arg2> <arg3>");
		OUT.println("	  arg1 : RID");
		OUT.println("	  arg2 : reason");
		OUT.println("	  arg3 : VERSION");
		OUT.println("  ex : revokePrescription BEPPaabbccdd no_reason V3");
		OUT.println("");

		OUT.println(" getPrescriptionStatus <arg1>");
		OUT.println("	  arg1 : RID");
		OUT.println("  ex : getPrescriptionStatus BEPPabcdefgh");
		OUT.println("");

		OUT.println(" getOpenPrescriptionList <arg1> <arg2> <arg3> ");
		OUT.println("	  arg1 : NISS");
		OUT.println("	  arg2 : page");
		OUT.println("  ex : getOpenPrescriptionList 76020727360  2019 2");
		OUT.println("");

		OUT.println(" getAllRidsInProcess");
		OUT.println("  ex : getAllRidsInProcess");
		OUT.println("");

		OUT.println(" ListRidsHistory <arg1> <arg2> <arg3>");
		OUT.println("	  arg1 : NISS");
		OUT.println("	  arg2 : page");
		OUT.println("  ex : listPrescriptionHistory 76020727360 2");
		OUT.println("");

		OUT.println(" ListReservations");
		OUT.println("  ex : getAllReservations");
		OUT.println("");

		OUT.println("");
		OUT.println("-------------- OPTIONS ------------");
		OUT.println(" -Dconfig=<path to the config file> : specified a config file");
		OUT.println(" -Dsession=<path to Xml session file> : specify a session");

		OUT.println(sb.toString());
	}

}
