﻿Imports System
Imports System.Diagnostics
Imports System.Drawing
Imports System.Globalization
Imports System.Linq
Imports System.Windows.Forms

Namespace JR.Utils.GUI.Forms

    Public Class FlexibleMessageBox

        Public Shared MAX_WIDTH_FACTOR As Double = 0.7

        ''' <summary>
        ''' Defines the maximum height for all FlexibleMessageBox instances in percent of the working area.
        ''' 
        ''' Allowed values are 0.2 - 1.0 where: 
        ''' 0.2 means:  The FlexibleMessageBox can be at most half as high as the working area.
        ''' 1.0 means:  The FlexibleMessageBox can be as high as the working area.
        ''' 
        ''' Default is: 90% of the working area height.
        ''' </summary>
        Public Shared MAX_HEIGHT_FACTOR As Double = 0.9

        ''' <summary>
        ''' Defines the font for all FlexibleMessageBox instances.
        ''' 
        ''' Default is: SystemFonts.MessageBoxFont
        ''' </summary>
        Public Shared FONT As Font = SystemFonts.MessageBoxFont

        Public Overloads Shared Function Show(ByVal text As String) As DialogResult
            Return FlexibleMessageBoxForm.Show(Nothing, text, String.Empty, MessageBoxButtons.OK, MessageBoxIcon.None, MessageBoxDefaultButton.Button1)
        End Function

        ''' <summary>
        ''' Shows the specified message box.
        ''' </summary>
        ''' <param name="owner">The owner.</param>
        ''' <param name="text">The text.</param>
        ''' <returns>The dialog result.</returns>
        Public Overloads Shared Function Show(ByVal owner As IWin32Window, ByVal text As String) As DialogResult
            Return FlexibleMessageBoxForm.Show(owner, text, String.Empty, MessageBoxButtons.OK, MessageBoxIcon.None, MessageBoxDefaultButton.Button1)
        End Function

        ''' <summary>
        ''' Shows the specified message box.
        ''' </summary>
        ''' <param name="text">The text.</param>
        ''' <param name="caption">The caption.</param>
        ''' <returns>The dialog result.</returns>
        Public Overloads Shared Function Show(ByVal text As String, ByVal caption As String) As DialogResult
            Return FlexibleMessageBoxForm.Show(Nothing, text, caption, MessageBoxButtons.OK, MessageBoxIcon.None, MessageBoxDefaultButton.Button1)
        End Function

        ''' <summary>
        ''' Shows the specified message box.
        ''' </summary>
        ''' <param name="owner">The owner.</param>
        ''' <param name="text">The text.</param>
        ''' <param name="caption">The caption.</param>
        ''' <returns>The dialog result.</returns>
        Public Overloads Shared Function Show(ByVal owner As IWin32Window, ByVal text As String, ByVal caption As String) As DialogResult
            Return FlexibleMessageBoxForm.Show(owner, text, caption, MessageBoxButtons.OK, MessageBoxIcon.None, MessageBoxDefaultButton.Button1)
        End Function

        ''' <summary>
        ''' Shows the specified message box.
        ''' </summary>
        ''' <param name="text">The text.</param>
        ''' <param name="caption">The caption.</param>
        ''' <param name="buttons">The buttons.</param>
        ''' <returns>The dialog result.</returns>
        Public Overloads Shared Function Show(ByVal text As String, ByVal caption As String, ByVal buttons As MessageBoxButtons) As DialogResult
            Return FlexibleMessageBoxForm.Show(Nothing, text, caption, buttons, MessageBoxIcon.None, MessageBoxDefaultButton.Button1)
        End Function

        ''' <summary>
        ''' Shows the specified message box.
        ''' </summary>
        ''' <param name="owner">The owner.</param>
        ''' <param name="text">The text.</param>
        ''' <param name="caption">The caption.</param>
        ''' <param name="buttons">The buttons.</param>
        ''' <returns>The dialog result.</returns>
        Public Overloads Shared Function Show(ByVal owner As IWin32Window, ByVal text As String, ByVal caption As String, ByVal buttons As MessageBoxButtons) As DialogResult
            Return FlexibleMessageBoxForm.Show(owner, text, caption, buttons, MessageBoxIcon.None, MessageBoxDefaultButton.Button1)
        End Function

        ''' <summary>
        ''' Shows the specified message box.
        ''' </summary>
        ''' <param name="text">The text.</param>
        ''' <param name="caption">The caption.</param>
        ''' <param name="buttons">The buttons.</param>
        ''' <param name="icon">The icon.</param>
        ''' <returns></returns>
        Public Overloads Shared Function Show(ByVal text As String, ByVal caption As String, ByVal buttons As MessageBoxButtons, ByVal icon As MessageBoxIcon) As DialogResult
            Return FlexibleMessageBoxForm.Show(Nothing, text, caption, buttons, icon, MessageBoxDefaultButton.Button1)
        End Function

        ''' <summary>
        ''' Shows the specified message box.
        ''' </summary>
        ''' <param name="owner">The owner.</param>
        ''' <param name="text">The text.</param>
        ''' <param name="caption">The caption.</param>
        ''' <param name="buttons">The buttons.</param>
        ''' <param name="icon">The icon.</param>
        ''' <returns>The dialog result.</returns>
        Public Overloads Shared Function Show(ByVal owner As IWin32Window, ByVal text As String, ByVal caption As String, ByVal buttons As MessageBoxButtons, ByVal icon As MessageBoxIcon) As DialogResult
            Return FlexibleMessageBoxForm.Show(owner, text, caption, buttons, icon, MessageBoxDefaultButton.Button1)
        End Function

        ''' <summary>
        ''' Shows the specified message box.
        ''' </summary>
        ''' <param name="text">The text.</param>
        ''' <param name="caption">The caption.</param>
        ''' <param name="buttons">The buttons.</param>
        ''' <param name="icon">The icon.</param>
        ''' <param name="defaultButton">The default button.</param>
        ''' <returns>The dialog result.</returns>
        Public Overloads Shared Function Show(ByVal text As String, ByVal caption As String, ByVal buttons As MessageBoxButtons, ByVal icon As MessageBoxIcon, ByVal defaultButton As MessageBoxDefaultButton) As DialogResult
            Return FlexibleMessageBoxForm.Show(Nothing, text, caption, buttons, icon, defaultButton)
        End Function

        ''' <summary>
        ''' Shows the specified message box.
        ''' </summary>
        ''' <param name="owner">The owner.</param>
        ''' <param name="text">The text.</param>
        ''' <param name="caption">The caption.</param>
        ''' <param name="buttons">The buttons.</param>
        ''' <param name="icon">The icon.</param>
        ''' <param name="defaultButton">The default button.</param>
        ''' <returns>The dialog result.</returns>
        Public Overloads Shared Function Show(ByVal owner As IWin32Window, ByVal text As String, ByVal caption As String, ByVal buttons As MessageBoxButtons, ByVal icon As MessageBoxIcon, ByVal defaultButton As MessageBoxDefaultButton) As DialogResult
            Return FlexibleMessageBoxForm.Show(owner, text, caption, buttons, icon, defaultButton)
        End Function

        Class FlexibleMessageBoxForm
            Inherits Form

            Private components As System.ComponentModel.IContainer = Nothing

            ''' <summary>
            ''' Verwendete Ressourcen bereinigen.
            ''' </summary>
            ''' <param name="disposing">True, wenn verwaltete Ressourcen gel�scht werden sollen; andernfalls False.</param>
            Protected Overrides Sub Dispose(ByVal disposing As Boolean)
                If (disposing _
                            AndAlso (Not (Me.components) Is Nothing)) Then
                    Me.components.Dispose()
                End If

                MyBase.Dispose(disposing)
            End Sub

            ''' <summary>
            ''' Erforderliche Methode f�r die Designerunterst�tzung.
            ''' Der Inhalt der Methode darf nicht mit dem Code-Editor ge�ndert werden.
            ''' </summary>
            Private Sub InitializeComponent()
                Me.components = New System.ComponentModel.Container
                Me.button1 = New System.Windows.Forms.Button
                Me.richTextBoxMessage = New System.Windows.Forms.RichTextBox
                Me.FlexibleMessageBoxFormBindingSource = New System.Windows.Forms.BindingSource(Me.components)
                Me.panel1 = New System.Windows.Forms.Panel
                Me.pictureBoxForIcon = New System.Windows.Forms.PictureBox
                Me.button2 = New System.Windows.Forms.Button
                Me.button3 = New System.Windows.Forms.Button
                CType(Me.FlexibleMessageBoxFormBindingSource, System.ComponentModel.ISupportInitialize).BeginInit()
                Me.panel1.SuspendLayout
                CType(Me.pictureBoxForIcon, System.ComponentModel.ISupportInitialize).BeginInit()
                Me.SuspendLayout
                ' 
                ' button1
                ' 
                Me.button1.Anchor = CType((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
                Me.button1.AutoSize = True
                Me.button1.DialogResult = System.Windows.Forms.DialogResult.OK
                Me.button1.Location = New System.Drawing.Point(11, 67)
                Me.button1.MinimumSize = New System.Drawing.Size(0, 24)
                Me.button1.Name = "button1"
                Me.button1.Size = New System.Drawing.Size(75, 24)
                Me.button1.TabIndex = 2
                Me.button1.Text = "OK"
                Me.button1.UseVisualStyleBackColor = True
                Me.button1.Visible = False
                ' 
                ' richTextBoxMessage
                ' 
                Me.richTextBoxMessage.Anchor = CType((((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Bottom) _
                            Or System.Windows.Forms.AnchorStyles.Left) _
                            Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
                Me.richTextBoxMessage.BackColor = System.Drawing.Color.White
                Me.richTextBoxMessage.BorderStyle = System.Windows.Forms.BorderStyle.None
                Me.richTextBoxMessage.DataBindings.Add(New System.Windows.Forms.Binding("Text", Me.FlexibleMessageBoxFormBindingSource, "MessageText", True, System.Windows.Forms.DataSourceUpdateMode.OnPropertyChanged))
                Me.richTextBoxMessage.Font = New System.Drawing.Font("Microsoft Sans Serif", 9.0!, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, CType(0, Byte))
                Me.richTextBoxMessage.Location = New System.Drawing.Point(50, 26)
                Me.richTextBoxMessage.Margin = New System.Windows.Forms.Padding(0)
                Me.richTextBoxMessage.Name = "richTextBoxMessage"
                Me.richTextBoxMessage.ReadOnly = True
                Me.richTextBoxMessage.ScrollBars = System.Windows.Forms.RichTextBoxScrollBars.Vertical
                Me.richTextBoxMessage.Size = New System.Drawing.Size(200, 20)
                Me.richTextBoxMessage.TabIndex = 0
                Me.richTextBoxMessage.TabStop = False
                Me.richTextBoxMessage.Text = "<Message>"
                AddHandler Me.richTextBoxMessage.LinkClicked, AddressOf Me.richTextBoxMessage_LinkClicked
                ' 
                ' panel1
                ' 
                Me.panel1.Anchor = CType((((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Bottom) _
                            Or System.Windows.Forms.AnchorStyles.Left) _
                            Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
                Me.panel1.BackColor = System.Drawing.Color.White
                Me.panel1.Controls.Add(Me.pictureBoxForIcon)
                Me.panel1.Controls.Add(Me.richTextBoxMessage)
                Me.panel1.Location = New System.Drawing.Point(-3, -4)
                Me.panel1.Name = "panel1"
                Me.panel1.Size = New System.Drawing.Size(268, 59)
                Me.panel1.TabIndex = 1
                ' 
                ' pictureBoxForIcon
                ' 
                Me.pictureBoxForIcon.BackColor = System.Drawing.Color.Transparent
                Me.pictureBoxForIcon.Location = New System.Drawing.Point(15, 19)
                Me.pictureBoxForIcon.Name = "pictureBoxForIcon"
                Me.pictureBoxForIcon.Size = New System.Drawing.Size(32, 32)
                Me.pictureBoxForIcon.TabIndex = 8
                Me.pictureBoxForIcon.TabStop = False
                ' 
                ' button2
                ' 
                Me.button2.Anchor = CType((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
                Me.button2.DialogResult = System.Windows.Forms.DialogResult.OK
                Me.button2.Location = New System.Drawing.Point(92, 67)
                Me.button2.MinimumSize = New System.Drawing.Size(0, 24)
                Me.button2.Name = "button2"
                Me.button2.Size = New System.Drawing.Size(75, 24)
                Me.button2.TabIndex = 3
                Me.button2.Text = "OK"
                Me.button2.UseVisualStyleBackColor = True
                Me.button2.Visible = False
                ' 
                ' button3
                ' 
                Me.button3.Anchor = CType((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
                Me.button3.AutoSize = True
                Me.button3.DialogResult = System.Windows.Forms.DialogResult.OK
                Me.button3.Location = New System.Drawing.Point(173, 67)
                Me.button3.MinimumSize = New System.Drawing.Size(0, 24)
                Me.button3.Name = "button3"
                Me.button3.Size = New System.Drawing.Size(75, 24)
                Me.button3.TabIndex = 0
                Me.button3.Text = "OK"
                Me.button3.UseVisualStyleBackColor = True
                Me.button3.Visible = False
                ' 
                ' FlexibleMessageBoxForm
                ' 
                Me.AutoScaleDimensions = New System.Drawing.SizeF(6.0!, 13.0!)
                Me.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Font
                Me.ClientSize = New System.Drawing.Size(260, 102)
                Me.Controls.Add(Me.button3)
                Me.Controls.Add(Me.button2)
                Me.Controls.Add(Me.panel1)
                Me.Controls.Add(Me.button1)
                Me.DataBindings.Add(New System.Windows.Forms.Binding("Text", Me.FlexibleMessageBoxFormBindingSource, "CaptionText", True))
                Me.MaximizeBox = False
                Me.MinimizeBox = False
                Me.MinimumSize = New System.Drawing.Size(276, 140)
                Me.Name = "FlexibleMessageBoxForm"
                Me.ShowIcon = False
                Me.SizeGripStyle = System.Windows.Forms.SizeGripStyle.Show
                Me.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent
                Me.Text = "<Caption>"
                AddHandler Shown, AddressOf Me.FlexibleMessageBoxForm_Shown
                CType(Me.FlexibleMessageBoxFormBindingSource, System.ComponentModel.ISupportInitialize).EndInit()
                Me.panel1.ResumeLayout(False)
                CType(Me.pictureBoxForIcon, System.ComponentModel.ISupportInitialize).EndInit()
                Me.ResumeLayout(False)
                Me.PerformLayout
            End Sub

            Private button1 As System.Windows.Forms.Button

            Private FlexibleMessageBoxFormBindingSource As System.Windows.Forms.BindingSource

            Private richTextBoxMessage As System.Windows.Forms.RichTextBox

            Private panel1 As System.Windows.Forms.Panel

            Private pictureBoxForIcon As System.Windows.Forms.PictureBox

            Private button2 As System.Windows.Forms.Button

            Private button3 As System.Windows.Forms.Button


            Private Shared STANDARD_MESSAGEBOX_SEPARATOR_LINES As String = "---------------------------" & vbLf

            Private Shared STANDARD_MESSAGEBOX_SEPARATOR_SPACES As String = "   "

            'These are the possible buttons (in a standard MessageBox)
            Private Enum ButtonID

                OK = 0

                CANCEL

                YES

                NO

                ABORT

                RETRY

                IGNORE
            End Enum

            'These are the buttons texts for different languages. 
            'If you want to add a new language, add it here and in the GetButtonText-Function
            Private Enum TwoLetterISOLanguageID

                en

                de

                es

                it
            End Enum

            Private Shared BUTTON_TEXTS_ENGLISH_EN() As String

            Private Shared Cancel() As String

            Private Shared &Yes() As String
            
            Private Shared &No() As String
            
            Private Shared &Abort() As String
            
            Private Shared &Retry() As String
            
            Private Shared &Ignore() As String
            
            'Note: This is also the fallback language
            Private Shared BUTTON_TEXTS_GERMAN_DE() As String

            Private Shared Abbrechen() As String

            Private Shared &Ja() As String
            
            Private Shared &Nein() As String
            
            Private Shared &Abbrechen() As String
            
            Private Shared &Wiederholen() As String
            
            Private Shared &Ignorieren() As String
            
            Private Shared BUTTON_TEXTS_SPANISH_ES() As String

            Private Shared Cancelar() As String

            Private Shared &S�() As String
            
            Private Shared &No() As String
            
            Private Shared &Abortar() As String
            
            Private Shared &Reintentar() As String
            
            Private Shared &Ignorar() As String
            
            Private Shared BUTTON_TEXTS_ITALIAN_IT() As String

            Private Shared Annulla() As String

            Private Shared &S�() As String
            
            Private Shared &No() As String
            
            Private Shared &Interrompi() As String
            
            Private Shared &Riprova() As String
            
            Private Shared &Ignora() As String
            
            Private defaultButton As MessageBoxDefaultButton

            Private visibleButtonsCount As Integer

            Private languageID As TwoLetterISOLanguageID = TwoLetterISOLanguageID.en

            Private Sub New()
                MyBase.New
                Me.InitializeComponent()
                'Try to evaluate the language. If this fails, the fallback language English will be used
                Enum.TryParse(Of TwoLetterISOLanguageID)(CultureInfo.CurrentUICulture.TwoLetterISOLanguageName, Me.languageID)
                Me.KeyPreview = True
                Me.KeyUp = (Me.KeyUp + FlexibleMessageBoxForm_KeyUp)
            End Sub

            Private Shared Function GetStringRows(ByVal message As String) As String()
                If String.IsNullOrEmpty(message) Then
                    Return Nothing
                End If

                Dim messageRows = message.Split(New Char() {vbLf}, StringSplitOptions.None)
                Return messageRows
            End Function

            ''' <summary>
            ''' Gets the button text for the CurrentUICulture language.
            ''' Note: The fallback language is English
            ''' </summary>
            ''' <param name="buttonID">The ID of the button.</param>
            ''' <returns>The button text</returns>
            Private Function GetButtonText(ByVal buttonID As ButtonID) As String
                Dim buttonTextArrayIndex = Convert.ToInt32(buttonID)
                Select Case (Me.languageID)
                    Case TwoLetterISOLanguageID.de
                        Return BUTTON_TEXTS_GERMAN_DE(buttonTextArrayIndex)
                    Case TwoLetterISOLanguageID.es
                        Return BUTTON_TEXTS_SPANISH_ES(buttonTextArrayIndex)
                    Case TwoLetterISOLanguageID.it
                        Return BUTTON_TEXTS_ITALIAN_IT(buttonTextArrayIndex)
                    Case Else
                        Return BUTTON_TEXTS_ENGLISH_EN(buttonTextArrayIndex)
                End Select

            End Function

            ''' <summary>
            ''' Ensure the given working area factor in the range of  0.2 - 1.0 where: 
            ''' 
            ''' 0.2 means:  20 percent of the working area height or width.
            ''' 1.0 means:  100 percent of the working area height or width.
            ''' </summary>
            ''' <param name="workingAreaFactor">The given working area factor.</param>
            ''' <returns>The corrected given working area factor.</returns>
            Private Shared Function GetCorrectedWorkingAreaFactor(ByVal workingAreaFactor As Double) As Double
                Const MIN_FACTOR As Double = 0.2
                Const MAX_FACTOR As Double = 1
                If (workingAreaFactor < MIN_FACTOR) Then
                    Return MIN_FACTOR
                End If

                If (workingAreaFactor > MAX_FACTOR) Then
                    Return MAX_FACTOR
                End If

                Return workingAreaFactor
            End Function

            ''' <summary>
            ''' Set the dialogs start position when given. 
            ''' Otherwise center the dialog on the current screen.
            ''' </summary>
            ''' <param name="flexibleMessageBoxForm">The FlexibleMessageBox dialog.</param>
            ''' <param name="owner">The owner.</param>
            Private Shared Sub SetDialogStartPosition(ByVal flexibleMessageBoxForm As FlexibleMessageBoxForm, ByVal owner As IWin32Window)
                'If no owner given: Center on current screen
                If (owner Is Nothing) Then
                    Dim screen = screen.FromPoint(Cursor.Position)
                    flexibleMessageBoxForm.StartPosition = FormStartPosition.Manual
                    flexibleMessageBoxForm.Left = (screen.Bounds.Left _
                                + ((screen.Bounds.Width / 2) _
                                - (flexibleMessageBoxForm.Width / 2)))
                    flexibleMessageBoxForm.Top = (screen.Bounds.Top _
                                + ((screen.Bounds.Height / 2) _
                                - (flexibleMessageBoxForm.Height / 2)))
                End If

            End Sub

            ''' <summary>
            ''' Calculate the dialogs start size (Try to auto-size width to show longest text row).
            ''' Also set the maximum dialog size. 
            ''' </summary>
            ''' <param name="flexibleMessageBoxForm">The FlexibleMessageBox dialog.</param>
            ''' <param name="text">The text (the longest text row is used to calculate the dialog width).</param>
            ''' <param name="text">The caption (this can also affect the dialog width).</param>
            Private Shared Sub SetDialogSizes(ByVal flexibleMessageBoxForm As FlexibleMessageBoxForm, ByVal text As String, ByVal caption As String)
                'First set the bounds for the maximum dialog size
                flexibleMessageBoxForm.MaximumSize = New Size(Convert.ToInt32((SystemInformation.WorkingArea.Width * FlexibleMessageBoxForm.GetCorrectedWorkingAreaFactor(MAX_WIDTH_FACTOR))), Convert.ToInt32((SystemInformation.WorkingArea.Height * FlexibleMessageBoxForm.GetCorrectedWorkingAreaFactor(MAX_HEIGHT_FACTOR))))
                'Get rows. Exit if there are no rows to render...
                Dim stringRows = FlexibleMessageBoxForm.GetStringRows(text)
                If (stringRows Is Nothing) Then
                    Return
                End If

                'Calculate whole text height
                Dim textHeight = TextRenderer.MeasureText(text, FONT).Height
                'Calculate width for longest text line
                Const SCROLLBAR_WIDTH_OFFSET As Integer = 15
                Dim longestTextRowWidth = stringRows.Max(() >= {}, TextRenderer.MeasureText(textForRow, FONT).Width)
                Dim captionWidth = TextRenderer.MeasureText(caption, SystemFonts.CaptionFont).Width
                Dim textWidth = Math.Max((longestTextRowWidth + SCROLLBAR_WIDTH_OFFSET), captionWidth)
                'Calculate margins
                Dim marginWidth = (flexibleMessageBoxForm.Width - flexibleMessageBoxForm.richTextBoxMessage.Width)
                Dim marginHeight = (flexibleMessageBoxForm.Height - flexibleMessageBoxForm.richTextBoxMessage.Height)
                'Set calculated dialog size (if the calculated values exceed the maximums, they were cut by windows forms automatically)
                flexibleMessageBoxForm.Size = New Size((textWidth + marginWidth), (textHeight + marginHeight))
            End Sub

            ''' <summary>
            ''' Set the dialogs icon. 
            ''' When no icon is used: Correct placement and width of rich text box.
            ''' </summary>
            ''' <param name="flexibleMessageBoxForm">The FlexibleMessageBox dialog.</param>
            ''' <param name="icon">The MessageBoxIcon.</param>
            Private Shared Sub SetDialogIcon(ByVal flexibleMessageBoxForm As FlexibleMessageBoxForm, ByVal icon As MessageBoxIcon)
                Select Case (icon)
                    Case MessageBoxIcon.Information
                        flexibleMessageBoxForm.pictureBoxForIcon.Image = SystemIcons.Information.ToBitmap
                    Case MessageBoxIcon.Warning
                        flexibleMessageBoxForm.pictureBoxForIcon.Image = SystemIcons.Warning.ToBitmap
                    Case MessageBoxIcon.Error
                        flexibleMessageBoxForm.pictureBoxForIcon.Image = SystemIcons.Error.ToBitmap
                    Case MessageBoxIcon.Question
                        flexibleMessageBoxForm.pictureBoxForIcon.Image = SystemIcons.Question.ToBitmap
                    Case Else
                        flexibleMessageBoxForm.pictureBoxForIcon.Visible = False
                        flexibleMessageBoxForm.richTextBoxMessage.Left = (flexibleMessageBoxForm.richTextBoxMessage.Left - flexibleMessageBoxForm.pictureBoxForIcon.Width)
                        flexibleMessageBoxForm.richTextBoxMessage.Width = (flexibleMessageBoxForm.richTextBoxMessage.Width + flexibleMessageBoxForm.pictureBoxForIcon.Width)
                End Select

            End Sub

            ''' <summary>
            ''' Set dialog buttons visibilities and texts. 
            ''' Also set a default button.
            ''' </summary>
            ''' <param name="flexibleMessageBoxForm">The FlexibleMessageBox dialog.</param>
            ''' <param name="buttons">The buttons.</param>
            ''' <param name="defaultButton">The default button.</param>
            Private Shared Sub SetDialogButtons(ByVal flexibleMessageBoxForm As FlexibleMessageBoxForm, ByVal buttons As MessageBoxButtons, ByVal defaultButton As MessageBoxDefaultButton)
                'Set the buttons visibilities and texts
                Select Case (buttons)
                    Case MessageBoxButtons.AbortRetryIgnore
                        flexibleMessageBoxForm.visibleButtonsCount = 3
                        flexibleMessageBoxForm.button1.Visible = True
                        flexibleMessageBoxForm.button1.Text = flexibleMessageBoxForm.GetButtonText(ButtonID.ABORT)
                        flexibleMessageBoxForm.button1.DialogResult = DialogResult.Abort
                        flexibleMessageBoxForm.button2.Visible = True
                        flexibleMessageBoxForm.button2.Text = flexibleMessageBoxForm.GetButtonText(ButtonID.RETRY)
                        flexibleMessageBoxForm.button2.DialogResult = DialogResult.Retry
                        flexibleMessageBoxForm.button3.Visible = True
                        flexibleMessageBoxForm.button3.Text = flexibleMessageBoxForm.GetButtonText(ButtonID.IGNORE)
                        flexibleMessageBoxForm.button3.DialogResult = DialogResult.Ignore
                        flexibleMessageBoxForm.ControlBox = False
                    Case MessageBoxButtons.OKCancel
                        flexibleMessageBoxForm.visibleButtonsCount = 2
                        flexibleMessageBoxForm.button2.Visible = True
                        flexibleMessageBoxForm.button2.Text = flexibleMessageBoxForm.GetButtonText(ButtonID.OK)
                        flexibleMessageBoxForm.button2.DialogResult = DialogResult.OK
                        flexibleMessageBoxForm.button3.Visible = True
                        flexibleMessageBoxForm.button3.Text = flexibleMessageBoxForm.GetButtonText(ButtonID.CANCEL)
                        flexibleMessageBoxForm.button3.DialogResult = DialogResult.Cancel
                        flexibleMessageBoxForm.CancelButton = flexibleMessageBoxForm.button3
                    Case MessageBoxButtons.RetryCancel
                        flexibleMessageBoxForm.visibleButtonsCount = 2
                        flexibleMessageBoxForm.button2.Visible = True
                        flexibleMessageBoxForm.button2.Text = flexibleMessageBoxForm.GetButtonText(ButtonID.RETRY)
                        flexibleMessageBoxForm.button2.DialogResult = DialogResult.Retry
                        flexibleMessageBoxForm.button3.Visible = True
                        flexibleMessageBoxForm.button3.Text = flexibleMessageBoxForm.GetButtonText(ButtonID.CANCEL)
                        flexibleMessageBoxForm.button3.DialogResult = DialogResult.Cancel
                        flexibleMessageBoxForm.CancelButton = flexibleMessageBoxForm.button3
                    Case MessageBoxButtons.YesNo
                        flexibleMessageBoxForm.visibleButtonsCount = 2
                        flexibleMessageBoxForm.button2.Visible = True
                        flexibleMessageBoxForm.button2.Text = flexibleMessageBoxForm.GetButtonText(ButtonID.YES)
                        flexibleMessageBoxForm.button2.DialogResult = DialogResult.Yes
                        flexibleMessageBoxForm.button3.Visible = True
                        flexibleMessageBoxForm.button3.Text = flexibleMessageBoxForm.GetButtonText(ButtonID.NO)
                        flexibleMessageBoxForm.button3.DialogResult = DialogResult.No
                        flexibleMessageBoxForm.ControlBox = False
                    Case MessageBoxButtons.YesNoCancel
                        flexibleMessageBoxForm.visibleButtonsCount = 3
                        flexibleMessageBoxForm.button1.Visible = True
                        flexibleMessageBoxForm.button1.Text = flexibleMessageBoxForm.GetButtonText(ButtonID.YES)
                        flexibleMessageBoxForm.button1.DialogResult = DialogResult.Yes
                        flexibleMessageBoxForm.button2.Visible = True
                        flexibleMessageBoxForm.button2.Text = flexibleMessageBoxForm.GetButtonText(ButtonID.NO)
                        flexibleMessageBoxForm.button2.DialogResult = DialogResult.No
                        flexibleMessageBoxForm.button3.Visible = True
                        flexibleMessageBoxForm.button3.Text = flexibleMessageBoxForm.GetButtonText(ButtonID.CANCEL)
                        flexibleMessageBoxForm.button3.DialogResult = DialogResult.Cancel
                        flexibleMessageBoxForm.CancelButton = flexibleMessageBoxForm.button3
                    Case MessageBoxButtons.OK
                    Case Else
                        flexibleMessageBoxForm.visibleButtonsCount = 1
                        flexibleMessageBoxForm.button3.Visible = True
                        flexibleMessageBoxForm.button3.Text = flexibleMessageBoxForm.GetButtonText(ButtonID.OK)
                        flexibleMessageBoxForm.button3.DialogResult = DialogResult.OK
                        flexibleMessageBoxForm.CancelButton = flexibleMessageBoxForm.button3
                End Select

                'Set default button (used in FlexibleMessageBoxForm_Shown)
                flexibleMessageBoxForm.defaultButton = Me.defaultButton
            End Sub


            Private Sub FlexibleMessageBoxForm_Shown(ByVal sender As Object, ByVal e As EventArgs)
                Dim buttonIndexToFocus As Integer = 1
                Dim buttonToFocus As Button
                'Set the default button...
                Select Case (Me.defaultButton)
                    Case MessageBoxDefaultButton.Button1
                    Case Else
                        buttonIndexToFocus = 1
                        If (Me.defaultButton = MessageBoxDefaultButton.Button2) Then
                            buttonIndexToFocus = 2
                        ElseIf (Me.defaultButton = MessageBoxDefaultButton.Button3) Then
                            buttonIndexToFocus = 3
                        End If

                End Select

                If (buttonIndexToFocus > Me.visibleButtonsCount) Then
                    buttonIndexToFocus = Me.visibleButtonsCount
                End If

                If (buttonIndexToFocus = 3) Then
                    buttonToFocus = Me.button3
                ElseIf (buttonIndexToFocus = 2) Then
                    buttonToFocus = Me.button2
                Else
                    buttonToFocus = Me.button1
                End If

                buttonToFocus.Focus
            End Sub

            ''' <summary>
            ''' Handles the LinkClicked event of the richTextBoxMessage control.
            ''' </summary>
            ''' <param name="sender">The source of the event.</param>
            ''' <param name="e">The <see cref="System.Windows.Forms.LinkClickedEventArgs"/> instance containing the event data.</param>
            Private Sub richTextBoxMessage_LinkClicked(ByVal sender As Object, ByVal e As LinkClickedEventArgs)
                Try
                    Cursor.Current = Cursors.WaitCursor
                    Process.Start(e.LinkText)
                Catch  As Exception
                    'Let the caller of FlexibleMessageBoxForm decide what to do with this exception...
                    Throw
                Finally
                    Cursor.Current = Cursors.Default
                End Try

            End Sub

            ''' <summary>
            ''' Handles the KeyUp event of the richTextBoxMessage control.
            ''' </summary>
            ''' <param name="sender">The source of the event.</param>
            ''' <param name="e">The <see cref="System.Windows.Forms.KeyEventArgs"/> instance containing the event data.</param>
            Private Sub FlexibleMessageBoxForm_KeyUp(ByVal sender As Object, ByVal e As KeyEventArgs)
                'Handle standard key strikes for clipboard copy: "Ctrl + C" and "Ctrl + Insert"
                If (e.Control _
                            AndAlso ((e.KeyCode = Keys.C) _
                            OrElse (e.KeyCode = Keys.Insert))) Then
                    Dim buttonsTextLine = Me.button1.Visible
                    'TODO: Warning!!!, inline IF is not supported ?
                    (Me.button1.Text + 'TODO: Warning!!!! NULL EXPRESSION DETECTED...
                    )
                    'Build same clipboard text like the standard .Net MessageBox
                    Dim textForClipboard = (STANDARD_MESSAGEBOX_SEPARATOR_LINES _
                                + (Me.Text _
                                + (Environment.NewLine _
                                + (STANDARD_MESSAGEBOX_SEPARATOR_LINES _
                                + (Me.richTextBoxMessage.Text _
                                + (Environment.NewLine _
                                + (STANDARD_MESSAGEBOX_SEPARATOR_LINES _
                                + (buttonsTextLine.Replace("&", String.Empty) _
                                + (Environment.NewLine + STANDARD_MESSAGEBOX_SEPARATOR_LINES)))))))))
                    'Set text in clipboard
                    Clipboard.SetText(textForClipboard)
                End If

            End Sub

            Public Property CaptionText As String
                Get
                End Get
                Set
                End Set
            End Property

            ''' <summary>
            ''' The text that is been used in the FlexibleMessageBoxForm.
            ''' </summary>
            Public Property MessageText As String
                Get
                End Get
                Set
                End Set
            End Property


            Public Shared Function Show(ByVal owner As IWin32Window, ByVal text As String, ByVal caption As String, ByVal buttons As MessageBoxButtons, ByVal icon As MessageBoxIcon, ByVal defaultButton As MessageBoxDefaultButton) As DialogResult
                'Create a new instance of the FlexibleMessageBox form
                Dim flexibleMessageBoxForm = New FlexibleMessageBoxForm
                flexibleMessageBoxForm.ShowInTaskbar = False
                'Bind the caption and the message text
                flexibleMessageBoxForm.CaptionText = caption
                flexibleMessageBoxForm.MessageText = text
                flexibleMessageBoxForm.FlexibleMessageBoxFormBindingSource.DataSource = flexibleMessageBoxForm
                'Set the buttons visibilities and texts. Also set a default button.
                flexibleMessageBoxForm.SetDialogButtons(flexibleMessageBoxForm, buttons, Me.defaultButton)
                'Set the dialogs icon. When no icon is used: Correct placement and width of rich text box.
                flexibleMessageBoxForm.SetDialogIcon(flexibleMessageBoxForm, icon)
                'Set the font for all controls
                flexibleMessageBoxForm.Font = FONT
                flexibleMessageBoxForm.richTextBoxMessage.Font = FONT
                'Calculate the dialogs start size (Try to auto-size width to show longest text row). Also set the maximum dialog size. 
                FlexibleMessageBoxForm.SetDialogSizes(flexibleMessageBoxForm, text, caption)
                'Set the dialogs start position when given. Otherwise center the dialog on the current screen.
                flexibleMessageBoxForm.SetDialogStartPosition(flexibleMessageBoxForm, owner)
                'Show the dialog
                Return flexibleMessageBoxForm.ShowDialog(owner)
            End Function
        End Class

    End Class
End Namespace