package be.business.connector.gfddpp.pcdh.mock;

import be.apb.gfddpp.common.utils.SingleMessageWrapper;
import be.apb.gfddpp.validation.exception.SingleMessageValidationException;
import be.apb.standards.smoa.schema.id.v1.InssIdType;
import be.apb.standards.smoa.schema.model.v1.DataLocationType;
import be.apb.standards.smoa.schema.model.v1.MedicationHistoryType;
import be.apb.standards.smoa.schema.model.v1.MinSetPatient;
import be.apb.standards.smoa.schema.model.v1.ObjectFactory;
import be.apb.standards.smoa.schema.v1.MedicationHistoryEvent;
import be.apb.standards.smoa.schema.v1.MedicationSchemeEntriesResponse;
import be.apb.standards.smoa.schema.v1.MedicationSchemeTimestampsResponse;
import be.apb.standards.smoa.schema.v1.SingleMessage;
import be.business.connector.core.exceptions.IntegrationModuleException;
import be.business.connector.core.utils.*;
import be.business.connector.gfddpp.domain.ThreeStateBoolean;
import be.business.connector.gfddpp.domain.medicationscheme.protocol.FetchDataEntriesRequest;
import be.business.connector.gfddpp.domain.medicationscheme.protocol.FetchDataEntriesResponse;
import be.business.connector.gfddpp.domain.medicationscheme.protocol.RetrieveTimestampsRequest;
import be.business.connector.gfddpp.domain.medicationscheme.protocol.RetrieveTimestampsResponse;
import be.business.connector.gfddpp.pcdh.PCDHIntegrationModuleImpl;
import be.business.connector.gfddpp.utils.ConsolidateUtils;
import be.business.connector.gfddpp.utils.MappingUtils;
import be.business.connector.projects.common.utils.ValidationUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;

public class PCDHIntegrationModuleMock extends PCDHIntegrationModuleImpl {
    private final static Logger LOG = Logger.getLogger(PCDHIntegrationModuleMock.class);


    /**
     * Instantiates a new executor integration module mock.
     *
     * @throws IntegrationModuleException the integration module exception
     */
    public PCDHIntegrationModuleMock() throws IntegrationModuleException {
        super();
    }

    @Override
    public String getData(String patientIdType, String patientId, String dataType, String dateRange, boolean excludeOwnData, byte[] localData, boolean requestPatientSignature, ThreeStateBoolean therapeuticRelationShip) throws IntegrationModuleException {
        ValidationUtils.validateIncomingFieldsGetData(patientIdType, patientId, dataType, dateRange);

        byte[] singleMessage = getResponse("MOCK_GETDATA");
        SingleMessageWrapper smw = null;
        SingleMessage sm = null;
        MarshallerHelper<SingleMessage, SingleMessage> helper = new MarshallerHelper<SingleMessage, SingleMessage>(SingleMessage.class, SingleMessage.class);

        if (localData != null && localData.length > 0) {
            LOG.debug("******************** Consolidation **********************");
            try {
                sm = ConsolidateUtils.createConsolidatedGetDataResponse(singleMessage, localData, patientId);
            } catch (SingleMessageValidationException e) {
                LOG.error("", e);
                throw new IntegrationModuleException(I18nHelper.getLabel("error.xml.invalid.singlemessage"));
            }
            smw = new SingleMessageWrapper(sm);
        } else {
            smw = new SingleMessageWrapper(helper.toObject(singleMessage));
        }


        //	convertEncryptedContentToProduct(smw);

        ObjectFactory o = new ObjectFactory();
        MinSetPatient patient = o.createMinSetPatient();
        InssIdType inssIdType = new InssIdType();
        inssIdType.setId(patientId);
        patient.setPersonId(inssIdType);

        for (MedicationHistoryType medicationHistoryType : smw.getAllMedicationHistoryEntries()) {
            medicationHistoryType.setMinPatient(patient);
        }

        return new String(helper.toXMLByteArray(smw.getWrappedMessage()));
    }

    @Override
    public RetrieveTimestampsResponse getMedicationSchemeTimestamps(RetrieveTimestampsRequest retrieveTimestampsRequest) throws IntegrationModuleException {
        byte[] singleMessage = getResponse("MOCK_GET_MEDICATIONSCHEME_TIMESTAMPS");
        try {
            MarshallerHelper<SingleMessage, SingleMessage> helper = new MarshallerHelper<SingleMessage, SingleMessage>(SingleMessage.class, SingleMessage.class);
            final SingleMessageWrapper wrapper = new SingleMessageWrapper(helper.toObject(singleMessage));
            final MedicationSchemeTimestampsResponse pcdhResponse = (MedicationSchemeTimestampsResponse) wrapper.getEvents().get(0);

            return MappingUtils.mapMedicationSchemeTimestampsResponse(pcdhResponse);
        } catch (Throwable t) {
            Exceptionutils.errorHandler(t);
            return null;
        }
    }

    @Override
    public FetchDataEntriesResponse getMedicationSchemeEntries(FetchDataEntriesRequest getMedicationEntriesRequest) throws IntegrationModuleException {
        byte[] singleMessage = getResponse("MOCK_GET_MEDICATIONSCHEME_ENTRIES");
        try {
            MarshallerHelper<SingleMessage, SingleMessage> helper = new MarshallerHelper<SingleMessage, SingleMessage>(SingleMessage.class, SingleMessage.class);
            final SingleMessageWrapper wrapper = new SingleMessageWrapper(helper.toObject(singleMessage));
            final MedicationSchemeEntriesResponse pcdhResponse = (MedicationSchemeEntriesResponse) wrapper.getEvents().get(0);
            return MappingUtils.mapMedicationSchemeEntriesResponse(pcdhResponse, null, 10);
        } catch (Throwable t) {
            Exceptionutils.errorHandler(t);
            return null;
        }
    }


    @Override
    public String getDataTypes(String patientIdType, String patientId, boolean requestPatientSignature, ThreeStateBoolean therapeuticRelationShip) throws IntegrationModuleException {
        ValidationUtils.validateIncomingFieldsGetDataTypes(patientId, patientIdType);

        byte[] singleMessage = getResponse("MOCK_GETDATATYPE");

        MarshallerHelper<SingleMessage, SingleMessage> helper = new MarshallerHelper<SingleMessage, SingleMessage>(SingleMessage.class, SingleMessage.class);
        SingleMessage sM = helper.toObject(singleMessage);
        SingleMessageWrapper smw = new SingleMessageWrapper(sM);
        ObjectFactory o = new ObjectFactory();
        MinSetPatient patient = o.createMinSetPatient();
        InssIdType inssIdType = new InssIdType();
        inssIdType.setId(patientId);
        patient.setPersonId(inssIdType);

        List<MedicationHistoryEvent> medicationHistoryEvents = smw.getAllEventsOfType(MedicationHistoryEvent.class);
        List<DataLocationType> dataLocationTypes = medicationHistoryEvents.get(0).getDataLocation();
        be.apb.standards.smoa.schema.model.v1.ObjectFactory objectFactory = new be.apb.standards.smoa.schema.model.v1.ObjectFactory();

        dataLocationTypes.get(0).setAbstractPatient(objectFactory.createAbstractPatient(patient));

        return new String(helper.toXMLByteArray(sM));
    }

    private byte[] getResponse(String propertyName) {
        try {
            InputStream fis = IOUtils.getResourceAsStream(getPropertyHandler().getProperty(propertyName));
            byte[] bytes = new byte[(int) fis.available()];
            fis.read(bytes);
            return bytes;
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public String getPharmacyDetails(String patientIdType, String patientId, String dGuid, String motivationType, String motivationText, boolean requestPatientSignature, ThreeStateBoolean therapeuticRelationShip) throws IntegrationModuleException {
        return "62599147";
    }
}
