/**
 * Copyright (C) 2010 Recip-e
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * <p>
 * http://www.apache.org/licenses/LICENSE-2.0
 * <p>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package be.business.connector.gfddpp.performance.metric;

import org.apache.log4j.Logger;

import be.apb.gfddpp.common.exceptions.GFDDPPException;
import be.apb.gfddpp.helper.PropertyHandler;
import be.business.connector.common.performance.metric.MetricsUploadAppenderCommon;
import be.business.connector.core.exceptions.IntegrationModuleException;
import be.business.connector.core.utils.STSHelper;
import be.business.connector.projects.common.services.pcdh.PcdhServiceImpl;
import be.ehealth.apb.gfddpp.services.pcdh.SystemError_Exception;
import be.ehealth.apb.gfddpp.services.pcdh.UploadPerformanceMetricRequestType;
import be.ehealth.technicalconnector.exception.SessionManagementException;
import be.ehealth.technicalconnector.session.Session;

/**
 * The Class MetricsUploadAppenderExecutor.
 */
public class MetricsUploadAppenderExecutor extends MetricsUploadAppenderCommon {

    public static final String PROP_WSDL_EHEALTH_PCDH = "wsdl.ehealth.pcdh";
    private static final Logger LOG = Logger.getLogger(MetricsUploadAppenderExecutor.class);

    /**
     * Instantiates a new metrics upload appender executor.
     *
     * @throws be.apb.gfddpp.common.exceptions.GFDDPPException
     */
    public MetricsUploadAppenderExecutor() throws GFDDPPException {
        super();
    }

    /**
     * Upload file.
     *
     * @param compressed the compressed
     * @throws IntegrationModuleException the integration module exception {@inheritDoc}
     * @throws SystemError_Exception
     */
    @Override
    public void uploadFile(byte[] compressed) throws Exception {
        if (Session.getInstance().hasValidSession()) {
            LOG.info("No session  available for the metrics upload");
            return;
        }
        PropertyHandler propertyHandler = PropertyHandler.getInstance();
        String nihii;
        try {
            nihii = STSHelper.getNihii(Session.getInstance().getSession().getSAMLToken().getAssertion());
        } catch (Throwable t) {
            LOG.warn("Nihii in session not found: ", t);
            nihii = propertyHandler.getProperty("metrics.select.nihii");
            LOG.debug("NIHII found in proerperty file is [" + nihii + "]");
        }
        boolean metricsFilteringEnabled = Boolean.parseBoolean(propertyHandler.getProperty("metrics.select.nihii.enable", "false"));
        String groupOfNihiiEnabled = propertyHandler.getProperty("metrics.select.nihii.group");
        if (metricsFilteringEnabled) {
            if (nihii.endsWith(groupOfNihiiEnabled)) {
                uploadeMetrics(compressed);
            } else {
                LOG.debug("User does not blong to the selected group. No metrics will be uploaded.");
            }
        } else {
            uploadeMetrics(compressed);
        }
    }

    private void uploadeMetrics(byte[] compressed) throws SessionManagementException, IntegrationModuleException {
        UploadPerformanceMetricRequestType request = buildPCDHRequest(compressed);
        PcdhServiceImpl.getInstance().uploadPerformanceMetric(request);
        LOG.info("Performance metric send!!!!!");
    }
}