package be.business.connector.gfddpp.tipsystem;

import static be.apb.gfddpp.common.medicationscheme.MSConstants.AUTHOR_ORGANIZATION_ID_KEY;
import static be.apb.gfddpp.common.medicationscheme.MSConstants.AUTHOR_ORGANIZATION_ID_SOURCE_KEY;
import static be.apb.gfddpp.common.medicationscheme.MSConstants.AUTHOR_ROLE_KEY;
import static be.apb.gfddpp.common.medicationscheme.MSConstants.AVAILABILITY_STATUS_KEY;
import static be.apb.gfddpp.common.medicationscheme.MSConstants.DGUID_KEY;
import static be.apb.gfddpp.common.medicationscheme.MSConstants.FORMAT_CODE_KEY;
import static be.apb.gfddpp.common.medicationscheme.MSConstants.SGUID_KEY;
import static be.apb.gfddpp.common.medicationscheme.MSConstants.TGUID_KEY;
import static be.apb.gfddpp.common.medicationscheme.status.MSStatusCode.MEDICATION_SCHEME_SUCCESS;
import static be.apb.gfddpp.common.medicationscheme.status.MSStatusCode.MEDICATION_SCHEME_VALIDATION_ERRORS;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import javax.xml.ws.soap.SOAPFaultException;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.log4j.Logger;
import org.perf4j.aop.Profiled;

import be.apb.gfddpp.common.exceptions.GFDDPPException;
import be.apb.gfddpp.common.log.LogsSMC.IntermediateMessage;
import be.apb.gfddpp.common.medicationscheme.status.MSStatusCode;
import be.apb.gfddpp.common.medicationscheme.status.MSStatusResolver;
import be.apb.gfddpp.common.status.StatusCode;
import be.apb.gfddpp.common.status.StatusResolver;
import be.apb.gfddpp.common.utils.SingleMessageBuilder;
import be.apb.gfddpp.common.utils.SingleMessageWrapper;
import be.apb.gfddpp.common.utils.Utils;
import be.apb.gfddpp.helper.SingleMessageValidationHelper;
import be.apb.gfddpp.tipsys.errors.Errors;
import be.apb.standards.smoa.schema.model.v1.AbstractEntityType;
import be.apb.standards.smoa.schema.model.v1.StatusMessageType;
import be.apb.standards.smoa.schema.model.v1.SubjectReferenceType;
import be.apb.standards.smoa.schema.v1.ArchivePrescriptionCommentEventType;
import be.apb.standards.smoa.schema.v1.ArchivePrescriptionEventType;
import be.apb.standards.smoa.schema.v1.AvailabilityStatus;
import be.apb.standards.smoa.schema.v1.DataEntryRequest;
import be.apb.standards.smoa.schema.v1.EventFolderType;
import be.apb.standards.smoa.schema.v1.FormatCode;
import be.apb.standards.smoa.schema.v1.ID;
import be.apb.standards.smoa.schema.v1.MedicationSchemeEntriesRequest;
import be.apb.standards.smoa.schema.v1.MetaDataListType;
import be.apb.standards.smoa.schema.v1.MetaDataType;
import be.apb.standards.smoa.schema.v1.OrganizationIdType;
import be.apb.standards.smoa.schema.v1.OrganizationRoleType;
import be.apb.standards.smoa.schema.v1.SingleMessage;
import be.apb.standards.smoa.schema.v1.SmoaMessageType;
import be.business.connector.common.ApplicationConfig;
import be.business.connector.common.StandaloneRequestorProvider;
import be.business.connector.common.module.AbstractIntegrationModule;
import be.business.connector.core.exceptions.IntegrationModuleEhealthException;
import be.business.connector.core.exceptions.IntegrationModuleException;
import be.business.connector.core.exceptions.IntegrationModuleRuntimeException;
import be.business.connector.core.technical.connector.utils.Crypto;
import be.business.connector.core.utils.EncryptionUtils;
import be.business.connector.core.utils.Exceptionutils;
import be.business.connector.core.utils.I18nHelper;
import be.business.connector.core.utils.PropertyHandler;
import be.business.connector.core.utils.SealedProcessor;
import be.business.connector.gfddpp.domain.SealedMessageRequestTypeXml;
import be.business.connector.gfddpp.domain.ThreeStateBoolean;
import be.business.connector.gfddpp.domain.medicationscheme.DataEntry;
import be.business.connector.gfddpp.domain.medicationscheme.Error;
import be.business.connector.gfddpp.domain.medicationscheme.Status;
import be.business.connector.gfddpp.domain.medicationscheme.protocol.StoreDataEntriesRequest;
import be.business.connector.gfddpp.domain.medicationscheme.protocol.StoreDataEntriesResponse;
import be.business.connector.gfddpp.tipsystem.offline.queue.EHealthMessageQueueFacade;
import be.business.connector.gfddpp.tipsystem.offline.queue.QueueBehavior;
import be.business.connector.gfddpp.tipsystem.offline.queue.QueueControle;
import be.business.connector.gfddpp.tipsystem.status.offline.queue.StatusMessageQueue;
import be.business.connector.gfddpp.utils.DataEntryURI;
import be.business.connector.gfddpp.utils.GFDDPPValidationUtils;
import be.business.connector.gfddpp.utils.MappingUtils;
import be.business.connector.gfddpp.utils.RequestCreatorUtils;
import be.business.connector.gfddpp.utils.SingleMessageInfo;
import be.business.connector.projects.common.services.tipsystem.TipSystemServiceImpl;
import be.business.connector.projects.common.utils.ProductFilterSingleton;
import be.business.connector.projects.common.utils.ProductFilterUtils;
import be.business.connector.projects.common.utils.ValidationUtils;
import be.ehealth.apb.gfddpp.services.tipsystem.LangageType;
import be.ehealth.apb.gfddpp.services.tipsystem.LocalisedString;
import be.ehealth.apb.gfddpp.services.tipsystem.RoutedSealedRequestType;
import be.ehealth.apb.gfddpp.services.tipsystem.RoutedSealedResponseType;
import be.ehealth.apb.gfddpp.services.tipsystem.SealedMessageRequestType;
import be.ehealth.apb.gfddpp.services.tipsystem.SimpleResponseType;
import be.fgov.ehealth.etee.crypto.encrypt.EncryptionToken;

public class TIPSystemIntegrationModuleImpl extends AbstractIntegrationModule implements TIPSystemIntegrationModule {

    private final static Logger LOG = Logger.getLogger(TIPSystemIntegrationModuleImpl.class);
    private final static String MODULE = "TIPSystemIntegrationModuleImpl";

    private final static String SUCCESS = StatusCode.TIP_200.name();
    private final static String TIP_201 = "TIP_201";
    private static final String INVALID_MEDICATION_SCHEME = StatusCode.TIP_ERROR_INVALID_MEDICATION_SCHEME.name();

    private QueueBehavior behavior;
    private static QueueControle controle;
    private static EHealthMessageQueueFacade facade;
    protected static StatusMessageQueue statusMessageQueue;
    private SingleMessageValidationHelper xmlValidationhelper;

    private ProductFilterUtils productFilterUtils;

    public ProductFilterUtils getProductFilterUtils() {
        return productFilterUtils;
    }

    public void setProductFilterUtils(ProductFilterUtils productFilterUtils) {
        this.productFilterUtils = productFilterUtils;
    }

    public TIPSystemIntegrationModuleImpl() throws IntegrationModuleException {
        this(true);
    }

    protected TIPSystemIntegrationModuleImpl(boolean initMessageQueues) throws IntegrationModuleException{
        super();
        ApplicationConfig.getInstance().checkLatestTIPConfiguration();
        xmlValidationhelper = new SingleMessageValidationHelper();
        setProductFilterUtils(new ProductFilterUtils());
        if(initMessageQueues){
            initMessageQueues();
        }
        LOG.info("*************** TIP System module init correctly *******************");

    }

    private void initMessageQueues() throws IntegrationModuleException{
        if (getPropertyHandler().getProperty("MESSAGE_QUEUE_FOLDER").equals(getPropertyHandler().getProperty("STATUS_MESSAGE_QUEUE_FOLDER"))) {
            throw new IntegrationModuleException(I18nHelper.getLabel("error.path.msg.queue.equal.to.status.msg.queue"));
        }
        try {
            statusMessageQueue = new StatusMessageQueue(PropertyHandler.getInstance(), EncryptionUtils.getInstance());
        } catch (Exception e) {
            throw new IntegrationModuleException(I18nHelper.getLabel("error.creation.statusMessageQueue"), e);
        }

        // Init de queue
        setControle(new QueueControle());

        facade = new EHealthMessageQueueFacade(PropertyHandler.getInstance(), EncryptionUtils.getInstance());

        if (behavior == null) {
            behavior = new QueueBehavior(getFacade().getQueue(), statusMessageQueue);
        }
        getControle().processMessages(behavior);
    }

    //protected void validateEventTypesForUpdateRevoke(SingleMessage singleMessage) throws IntegrationModuleException {
    //	SingleMessageWrapper singleMessageWrapper = new SingleMessageWrapper(singleMessage);
    //	List<AbstractEventType> types = singleMessageWrapper.getEvents();
    //	for (AbstractEventType type : types) {
    //		if (!(type instanceof PharmaceuticalCareEventType)) {
    //			throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.event.type.not.supported"));
    //		}
    //	}
    //}
    public String getNihiiPharmacyId(String info, boolean SyncEventType) throws IntegrationModuleException {
        String procedure = "getNihiiPharmacyId";

        String NihiiPharmacyId = "";
        try {
            String requestorId = StandaloneRequestorProvider.getRequestorIdInformation();
            String requestorType = StandaloneRequestorProvider.getRequestorTypeInformation();
            info += " RequestorId:" + requestorId + " " + "requestorType:" + requestorType + " ";
            LOG.debug(MODULE + "=-----= ----- " + procedure + " =$=" + info);
            NihiiPharmacyId = requestorId;
        } catch (IntegrationModuleException e) {
            LOG.info(MODULE + "=-----= ----- " + procedure + " =$=" + info + "no valid session: offline operation" + e.getMessage());
        }

        if (NihiiPharmacyId == null || NihiiPharmacyId.isEmpty()) {
            if (SyncEventType) {
                LOG.error(MODULE + "=-----= ----- " + procedure + " =$=" + info + "error: error.validation.SyncEventType.nosession");
                throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.SyncEventType.nosession"));
            } else {
                NihiiPharmacyId = getPropertyHandler().getProperty("pharmacy.nihii");
                LOG.error(MODULE + "=-----= ----- " + procedure + " =$=" + info + "no valid session load from properties NihiiPharmacy:" + NihiiPharmacyId);
            }
        }

        if (NihiiPharmacyId == null || NihiiPharmacyId.isEmpty()) {
            LOG.error(MODULE + "=-----= ----- " + procedure + " =$=" + info + "error: error.pharmacy.nihii.not.found");
            throw new IntegrationModuleException(I18nHelper.getLabel("error.pharmacy.nihii.not.found"));
            //TODO JL error.pharmacy.nihii.not.found
        }

        //TODO JL securityUtils.isInSystemKeyStore(NihiiPharmacyId)
        //if (securityUtils.isInSystemKeyStore(NihiiPharmacyId)) {
        //	LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: error.validation.SyncEventType.nosession");
        //	throw new IntegrationModuleException(I18nHelper.getLabel("validate.nihii.keystore.not.matching", new Object[] { NihiiPharmacyId }));
        //}
        return NihiiPharmacyId;
    }

    public String getTipId(String info, String tipId) throws IntegrationModuleException {
        if (StringUtils.isEmpty(tipId)) {
            if (getPropertyHandler().hasProperty("default.tip.id")) {
                tipId = getPropertyHandler().getProperty("default.tip.id");
            }
        }
        ValidationUtils.validateExistingTipId(tipId, getPropertyHandler());
        return tipId;
    }

    @Override
    @Profiled(logFailuresSeparately = true, tag = "{$1}.TIPSystemIntegrationModuleImpl#archivePrescription", logger = "org.perf4j.TimingLogger_Executor")
    public String archivePrescription(String rid) throws IntegrationModuleException {
        String procedure = "archivePrescription";
        String info = " ";
        LOG.info(MODULE + "=-----= start " + procedure + " =$=" + info);
        long startTimeStamp = System.nanoTime();
        String events = "";
        try {
			String archivedFilePath = getArchivedFilePath(rid);
            be.apb.standards.smoa.schema.v1.ObjectFactory sof = new be.apb.standards.smoa.schema.v1.ObjectFactory();
            byte[] archivedPrescription = getArchivedPrescription(archivedFilePath, rid, archivedFilePath.substring(0, archivedFilePath.indexOf(rid)-1));
            ArchivePrescriptionEventType apet = new ArchivePrescriptionEventType();
            apet.setRid(rid);
            apet.setArchiveStandard(archivedPrescription);
            apet.setArchiveStandardVersion("V2");
            apet.setMarkAsDeliverTime(getCurrentXMLGregorianCalendar());
            try {
                apet.setExecutorId(getId(StandaloneRequestorProvider.getRequestorIdInformation(), "PHARMACY", "NIHII"));
            } catch (IntegrationModuleException e) {
                LOG.info(MODULE + "=-----= ----- " + procedure + " =$=" + info + "no valid session: offline operation" + e.getMessage());
                apet.setExecutorId(getId(getPropertyHandler().getProperty("pharmacy.nihii"), "PHARMACY", "NIHII"));
            }
            apet.setDguid(null);
            apet.setSguid(null);

            SingleMessage sm = new SingleMessage();
            SmoaMessageType um = new SmoaMessageType();
            EventFolderType eft = sof.createEventFolderType();
            EventFolderType.Events eventFolderTypeEvents = new EventFolderType.Events();
            eventFolderTypeEvents.getEvent().add(apet);
            eft.setEvents(eventFolderTypeEvents);
            um.setAbstractFolder(sof.createEventFolder(eft));
            sm.setUnsigned(um);
            String tip_id = PropertyHandler.getInstance().getProperty("tipsystem.id");
            LOG.debug("Single Message [" + getJaxContextCentralizer().toXml(SingleMessage.class, sm) + "].");
            final RegisterEventResponse registerEventResponse = registerEvent(getJaxContextCentralizer().toXml(SingleMessage.class, sm).getBytes("UTF-8"), info, tip_id, false, null, procedure, startTimeStamp);
            events = registerEventResponse.getSingleMessageInfo().getEvents();
        } catch (GFDDPPException e) {
            double difftime = (double) (System.nanoTime() - startTimeStamp) / 1000000000.0;
            LOG.info(MODULE + "=-----= einde " + procedure + " =$=" + info + " => " + " (dt:" + difftime + ") " + "error GFDDPPException: " + e.getMessage());
            throw new IntegrationModuleException(StatusResolver.resolveMessage(StatusResolver.getLocalResourceBundle(), e.getStatusCode(), e.getPlaceHolders()), e);
        } catch (Throwable t) {
            double difftime = (double) (System.nanoTime() - startTimeStamp) / 1000000000.0;
            LOG.info(MODULE + "=-----= einde " + procedure + " =$=" + info + " => " + " (dt:" + difftime + ") " + "error Throwable: " + t.getMessage());
            Exceptionutils.errorHandler(t);
        }
        return events;
    }

    private ID getId(String value, String type, String typeId) throws IntegrationModuleException {
        ID id = new ID();
        id.setValue(value);
        id.setType(type);
        id.setIdType(typeId);
        return id;
    }

    @Override
    @Profiled(logFailuresSeparately = true, tag = "{$1}.TIPSystemIntegrationModuleImpl#archivePrescriptionComment", logger = "org.perf4j.TimingLogger_Executor")
    public String archivePrescriptionComment(String rid, String comment) throws IntegrationModuleException {
        String procedure = "archivePrescriptionComment";
        String info = " ";
        LOG.info(MODULE + "=-----= start " + procedure + " =$=" + info);
        long startTimeStamp = System.nanoTime();
        String events = "";
        try {
//            String archivedFilePath = getArchivedFilePath(rid);
            be.apb.standards.smoa.schema.v1.ObjectFactory sof = new be.apb.standards.smoa.schema.v1.ObjectFactory();
//            byte[] archivedPrescription = getArchivedPrescription(archivedFilePath);
            ArchivePrescriptionCommentEventType apet = new ArchivePrescriptionCommentEventType();
            apet.setRid(rid);
            apet.setPrescriptionComment(comment.getBytes("UTF-8"));
            apet.setPrescriptionCommentVersion("V2");
            apet.setPrescriptionCommentTime(getCurrentXMLGregorianCalendar());
            try {
                apet.setExecutorId(getId(StandaloneRequestorProvider.getRequestorIdInformation(), "PHARMACY", "NIHII"));
            } catch (IntegrationModuleException e) {
                LOG.info(MODULE + "=-----= ----- " + procedure + " =$=" + info + "no valid session: offline operation" + e.getMessage());
                apet.setExecutorId(getId(getPropertyHandler().getProperty("pharmacy.nihii"), "PHARMACY", "NIHII"));
            }

            apet.setDguid(null);
            apet.setSguid(null);

            SingleMessage sm = new SingleMessage();
            SmoaMessageType um = new SmoaMessageType();
            EventFolderType eft = sof.createEventFolderType();
            EventFolderType.Events eventFolderTypeEvents = new EventFolderType.Events();
            eventFolderTypeEvents.getEvent().add(apet);
            eft.setEvents(eventFolderTypeEvents);
            um.setAbstractFolder(sof.createEventFolder(eft));
            sm.setUnsigned(um);
            String tip_id = PropertyHandler.getInstance().getProperty("tipsystem.id");
            LOG.debug("Single Message [" + getJaxContextCentralizer().toXml(SingleMessage.class, sm) + "].");
            RegisterEventResponse response = registerEvent(getJaxContextCentralizer().toXml(SingleMessage.class, sm).getBytes("UTF-8"), info, tip_id, false, null, procedure, startTimeStamp);
            events = response.getSingleMessageInfo().getEvents();
        } catch (GFDDPPException e) {
            double difftime = (double) (System.nanoTime() - startTimeStamp) / 1000000000.0;
            LOG.info(MODULE + "=-----= einde " + procedure + " =$=" + info + " => " + " (dt:" + difftime + ") " + "error GFDDPPException: " + e.getMessage());
            throw new IntegrationModuleException(StatusResolver.resolveMessage(StatusResolver.getLocalResourceBundle(), e.getStatusCode(), e.getPlaceHolders()), e);
        } catch (Throwable t) {
            double difftime = (double) (System.nanoTime() - startTimeStamp) / 1000000000.0;
            LOG.info(MODULE + "=-----= einde " + procedure + " =$=" + info + " => " + " (dt:" + difftime + ") " + "error Throwable: " + t.getMessage());
            Exceptionutils.errorHandler(t);
        }
        return events;
    }

    @Override
    @Profiled(logFailuresSeparately = true, tag = "{$1}.TIPSystemIntegrationModuleImpl#registerData", logger = "org.perf4j.TimingLogger_Executor")
    public String registerData(byte[] singleMessage, String tipId, boolean requestPatientSignature, ThreeStateBoolean patientConsent) throws IntegrationModuleException {
        String procedure = "registerData";
        String info = " ";
        LOG.info(MODULE + "=-----= start " + procedure + " =$=" + info);
        long startTimeStamp = System.nanoTime();

        String events = "";
        try {
            final RegisterEventResponse response = registerEvent(singleMessage, info, tipId, requestPatientSignature, patientConsent, procedure, startTimeStamp);
            events = response.getSingleMessageInfo().getEvents();
        } catch (GFDDPPException e) {
            double difftime = (double) (System.nanoTime() - startTimeStamp) / 1000000000.0;
            LOG.info(MODULE + "=-----= einde " + procedure + " =$=" + info + " => " + " (dt:" + difftime + ") " + "error GFDDPPException: " + e.getMessage());
            throw new IntegrationModuleException(StatusResolver.resolveMessage(StatusResolver.getLocalResourceBundle(), e.getStatusCode(), e.getPlaceHolders()), e);
        } catch (Throwable t) {
            double difftime = (double) (System.nanoTime() - startTimeStamp) / 1000000000.0;
            LOG.info(MODULE + "=-----= einde " + procedure + " =$=" + info + " => " + " (dt:" + difftime + ") " + "error Throwable: " + t.getMessage());
            Exceptionutils.errorHandler(t);
        }
        return events;
    }

    protected class RegisterEventResponse {

        private SingleMessageInfo singleMessageInfo;
        private SimpleResponseType tipResponse;

        public RegisterEventResponse() {
        }

        public SingleMessageInfo getSingleMessageInfo() {
            return singleMessageInfo;
        }

        public void setSingleMessageInfo(SingleMessageInfo singleMessageInfo) {
            this.singleMessageInfo = singleMessageInfo;
        }

        public SimpleResponseType getTipResponse() {
            return tipResponse;
        }

        public void setTipResponse(SimpleResponseType tipResponse) {
            this.tipResponse = tipResponse;
        }
    }

    protected RegisterEventResponse registerEvent(byte[] singleMessage, String info, String tipId, boolean requestPatientSignature, ThreeStateBoolean patientConsent, String procedure, long startTimeStamp) throws IntegrationModuleEhealthException, IntegrationModuleException, IntegrationModuleRuntimeException {
        final RegisterEventResponse registerEventResponse = new RegisterEventResponse();
        try {

            LOG.debug("===================== Validate single message UTF-8 bytes ==============================");
            if (getPropertyHandler().hasProperty("TIP_UTF_8_ONLY_INPUT")
                    && getPropertyHandler().getProperty("TIP_UTF_8_ONLY_INPUT").equals("true")) {
                ValidationUtils.validateUTF_8Input(singleMessage);
            }

            //=========================== singleMessage ======================================
            SingleMessageInfo smi = new SingleMessageInfo(singleMessage);
            info += smi.getInfo();

            //=========================== tipId ======================================
            tipId = getTipId(info, tipId);
            info += " tipId:" + tipId + " ";

            //=========================== validate ======================================
            smi.validateRegisterData();

            //=========================== NihiiPharmacyId ======================================
            String NihiiPharmacyId = getNihiiPharmacyId(info, smi.getHasSyncEventType());
            info += " NihiiPharmacyId:" + NihiiPharmacyId + " ";

            smi.validateNihiiPharmacyId(NihiiPharmacyId);

            smi.setHeaderNihiiPharmacyId(NihiiPharmacyId);

            //=========================== ProductFilter ======================================
            ProductFilterSingleton.getInstance(getPropertyHandler());
            smi.setFilterProducts(getProductFilterUtils(), getPropertyHandler());

            //=========================== addGUID ======================================
            smi.setGuids(NihiiPharmacyId);

            //=========================== validateSingleMessageXsd ======================================
            smi.validateSingleMessageXsd();

            //=========================== setCommonData ======================================
            SealedMessageRequestType sealedMessageRequestType = smi.setCommonData(getEncryptionUtils(), tipId, requestPatientSignature, patientConsent);

            registerEventResponse.setSingleMessageInfo(smi);
            if (!smi.getHasSyncEventType()) {
                LOG.info(MODULE + "=-----= ----- " + procedure + " =$=" + info + "offline queue");

                SealedMessageRequestTypeXml sealedMessageRequestTypeXml = new SealedMessageRequestTypeXml();
                sealedMessageRequestTypeXml.setSealedMessageRequestType(sealedMessageRequestType);

                String sealedMsgXML = getJaxContextCentralizer().toXml(SealedMessageRequestTypeXml.class, sealedMessageRequestTypeXml);
                byte[] sealedSMCForQueue = sealedMsgXML.getBytes("UTF-8");

                LOG.debug(MODULE + "=-----= ----- " + procedure + " =$=" + info + "offline queue " + IntermediateMessage.TO_OFFLINE_QUEUE + " " + smi.getLogInfoGuids());

                getFacade().registerData(sealedSMCForQueue, smi.getEvents(), smi.getMessageID(), smi.getHeaderPharmacyId());

                LOG.debug(MODULE + "=-----= ----- " + procedure + " =$=" + info + "Register complete ( sync part )");
            } else {
                ApplicationConfig.getInstance().assertValidSession();

                LOG.info(MODULE + "=-----= ----- " + procedure + " =$=" + info + "SyncEventType");

                String sealedMessageTipId = null;
                if (sealedMessageRequestType.getRoutingParameters() == null || org.apache.commons.lang.StringUtils.isEmpty(sealedMessageRequestType.getRoutingParameters().getTIPId())) {
                    sealedMessageRequestType.getRoutingParameters().setTIPId(getPropertyHandler().getProperty("default.tip.id"));
                } else {
                    sealedMessageTipId = sealedMessageRequestType.getRoutingParameters().getTIPId();
                }

                EncryptionToken etk = null;
                try {
                    if ("false".equals(getPropertyHandler().getProperty("multitenancy.disabled", "false"))) {
                        etk = getEtkHelper().getTIP_ETK(sealedMessageTipId).get(0);
                    } else {
                        etk = getEtkHelper().getTIPSystem_ETK(getPropertyHandler().getProperty("tipsystem.id")).get(0);
                    }
                } catch (IntegrationModuleException e) {
                    LOG.error(MODULE + "=-----= ----- " + procedure + " =$=" + info + "error: Couldn't retrieve ETK", e);
                    throw e;
                }

                byte[] singleMessageBytes = sealedMessageRequestType.getSingleMessageSealed();

                byte[] SingleMessageSealed = SealedProcessor.createSealedSync(etk, singleMessageBytes, "SingleMessage");

                sealedMessageRequestType.setSingleMessageSealed(SingleMessageSealed);

                SimpleResponseType response = TipSystemServiceImpl.getInstance().registerData(sealedMessageRequestType);
                registerEventResponse.setTipResponse(response);

                String eventtype = smi.getEventType();
                if (response != null) {
                    final String statusCode = response.getStatus().getCode();
                    if (SUCCESS.equals(statusCode) || INVALID_MEDICATION_SCHEME.equals(statusCode)) {
                        LOG.info(MODULE + "=-----= ----- " + procedure + " =$=" + info + statusCode + " " + statusCode);
                    } else if (StringUtils.equals(response.getStatus().getCode(), TIP_201)) {
                        LOG.info(MODULE + "=-----= ----- " + procedure + " =$=" + info + TIP_201 + " TIP_201");
                        throw new IntegrationModuleException("Sending " + eventtype + " message to tip failed");
                    } else {
                        LOG.error(MODULE + "=-----= ----- " + procedure + " =$=" + info + response.getStatus().getCode());
                        throw new IntegrationModuleException("Sending " + eventtype + " message to tip failed");
                    }
                } else {
                    LOG.error(MODULE + "=-----= ----- " + procedure + " =$=" + info + "Response from the tip/eHealth is null, something went wrong while trying to send a message to the tip");
                    throw new IntegrationModuleException("error.no.response.tip");
                }
            }
        } catch (GFDDPPException e) {
            double difftime = (double) (System.nanoTime() - startTimeStamp) / 1000000000.0;
            LOG.info(MODULE + "=-----= einde " + procedure + " =$=" + info + " => " + " (dt:" + difftime + ") " + "error GFDDPPException: " + e.getMessage());
            throw new IntegrationModuleException(StatusResolver.resolveMessage(StatusResolver.getLocalResourceBundle(), e.getStatusCode(), e.getPlaceHolders()), e);
        } catch (Throwable t) {
            double difftime = (double) (System.nanoTime() - startTimeStamp) / 1000000000.0;
            LOG.info(MODULE + "=-----= einde " + procedure + " =$=" + info + " => " + " (dt:" + difftime + ") " + "error Throwable: " + t.getMessage());
            Exceptionutils.errorHandler(t);
        }
        return registerEventResponse;
    }

    @Profiled(logFailuresSeparately = true, tag = "{$1}.TIPSystemIntegrationModuleImpl#registerMedicationSchemeEntries", logger = "org.perf4j.TimingLogger_Executor")
    public StoreDataEntriesResponse registerMedicationSchemeEntries(final StoreDataEntriesRequest storeRequest) throws IntegrationModuleException {
        String procedure = "registerMedicationSchemeEntries";
        String info = " ";
        LOG.info(MODULE + "=-----= start " + procedure + " =$=" + info);
        long startTimeStamp = System.nanoTime();

        LOG.debug("******************** REGISTER_MEDICATIONSCHEME_ENTRIES - Validate incoming fields **********************");
        final List<Error> errors = GFDDPPValidationUtils.validate(storeRequest);
        if (CollectionUtils.isNotEmpty(errors)) {
            final StoreDataEntriesResponse response = new StoreDataEntriesResponse();
            final Status status = new Status(MEDICATION_SCHEME_VALIDATION_ERRORS.getCode(), MSStatusResolver.getStatusMessage(MEDICATION_SCHEME_VALIDATION_ERRORS));
            status.getErrors().addAll(errors);
            response.setStatus(status);
            response.setServerMessageID(UUID.randomUUID().toString());
            if (storeRequest != null) {
                response.setClientMessageID(storeRequest.getClientMessageID());
                response.setSubjectID(storeRequest.getSubjectID());
            }
            return response;
        }

        try {
            LOG.debug("******************** REGISTER_MEDICATIONSCHEME_ENTRIES -  Create single message  **********************");
            final MedicationSchemeEntriesRequest medicationSchemeEntriesRequest = new MedicationSchemeEntriesRequest();
            medicationSchemeEntriesRequest.setClientMessageId(storeRequest.getClientMessageID());
            medicationSchemeEntriesRequest.setSubjectId(storeRequest.getSubjectID());
            medicationSchemeEntriesRequest.setVersion(storeRequest.getDataEntries().get(0).getNodeVersion());
            medicationSchemeEntriesRequest.setCurrentDateTime(Utils.transformToXMLGregorianCalendar(new Date()));
            for (final DataEntry dataEntry : storeRequest.getDataEntries()) {
                final DataEntryURI dataEntryURI = new DataEntryURI(dataEntry.getDataEntryURI());
                final DataEntryRequest smcDataEntry = new DataEntryRequest();
                if (dataEntryURI.isCreateURI()) {
                    smcDataEntry.setDataEntryId(UUID.randomUUID().toString());
                } else {
                    smcDataEntry.setDataEntryId(dataEntryURI.getDataEntryId());
                    smcDataEntry.setDataEntryVersion(dataEntryURI.getDataEntryVersion());
                }
                smcDataEntry.setAvailabilityStatus(AvailabilityStatus.valueOf(dataEntry.getMetadata().get(AVAILABILITY_STATUS_KEY).toUpperCase()));
                smcDataEntry.setFormatCode(FormatCode.valueOf(dataEntry.getMetadata().get(FORMAT_CODE_KEY).toUpperCase()));

                final List<String> keysToIgnore = Arrays.asList(AVAILABILITY_STATUS_KEY, FORMAT_CODE_KEY);

                final MetaDataListType smcMetaDataList = new MetaDataListType();
                for (Map.Entry<String, String> metaData : dataEntry.getMetadata().entrySet()) {
                    if (!keysToIgnore.contains(metaData.getKey())) {
                        final MetaDataType smcMetaData = new MetaDataType();
                        smcMetaData.setKey(metaData.getKey());
                        smcMetaData.setValue(metaData.getValue());
                        smcMetaDataList.getMetaData().add(smcMetaData);
                    }
                }
                if (CollectionUtils.isNotEmpty(smcMetaDataList.getMetaData())) {
                    smcDataEntry.setMetaDataList(smcMetaDataList);
                }
                smcDataEntry.setBusinessData(dataEntry.getBusinessData());
                smcDataEntry.setReference(dataEntry.getReference());
                medicationSchemeEntriesRequest.getDataEntry().add(smcDataEntry);
            }

            final SingleMessage singleMessage = new SingleMessageBuilder().createSingleMessageUnsigned();
            singleMessage.getUnsigned().setHeader(null);
            final SingleMessageWrapper wrapper = new SingleMessageWrapper(singleMessage);
            wrapper.getEvents().add(medicationSchemeEntriesRequest);

            final String tip_id = PropertyHandler.getInstance().getProperty("tipsystem.id");
            final String singleMessageString = getJaxContextCentralizer().toXml(SingleMessage.class, singleMessage);
            LOG.debug("Single Message [" + singleMessageString + "].");

            final RegisterEventResponse registerEventResponse = registerEvent(singleMessageString.getBytes("UTF-8"), info, tip_id, false, null, procedure, startTimeStamp);
            final SimpleResponseType tipResponse = registerEventResponse.getTipResponse();
            final String tipStatusCode = tipResponse.getStatus().getCode();
            final SingleMessageInfo finalSingleMessageInfo = registerEventResponse.getSingleMessageInfo();

            final SingleMessageWrapper finalSingleMessageWrapper = finalSingleMessageInfo.getWrapper();
            final List<MedicationSchemeEntriesRequest> finalMedicationSchemeEntryRequests = finalSingleMessageWrapper.getAllMedicationSchemeEntryRequests();
            final MedicationSchemeEntriesRequest finalMedicationSchemeEntriesRequest = finalMedicationSchemeEntryRequests.get(0);

            final StoreDataEntriesResponse storeResponse = new StoreDataEntriesResponse();

            storeResponse.setClientMessageID(storeRequest.getClientMessageID());
            storeResponse.setServerMessageID(finalSingleMessageInfo.getMessageID());
            storeResponse.setSubjectID(finalMedicationSchemeEntriesRequest.getSubjectId());
            storeResponse.setStatus(new Status());
            if (SUCCESS.equals(tipStatusCode)) {
                storeResponse.getStatus().setCode(MEDICATION_SCHEME_SUCCESS.getCode());
                storeResponse.getStatus().setMessage(MSStatusResolver.getStatusMessage(MEDICATION_SCHEME_SUCCESS));
            } else {
                storeResponse.getStatus().setCode(MEDICATION_SCHEME_VALIDATION_ERRORS.getCode());
                storeResponse.getStatus().setMessage(MSStatusResolver.getStatusMessage(MEDICATION_SCHEME_VALIDATION_ERRORS));
                for (LocalisedString message : tipResponse.getStatus().getMessage()) {
                    final byte[] messageBytes = message.getValue().getBytes("UTF-8");
                    if (LangageType.NA.equals(message.getLang()) && Base64.isBase64(messageBytes)) {
                        final byte[] decodedData = Base64.decodeBase64(messageBytes);
                        final Errors detailedErrorList = getJaxContextCentralizer().toObject(Errors.class, decodedData);
                        for (be.apb.gfddpp.tipsys.errors.Error tipSysError : detailedErrorList.getError()) {
                            final MSStatusCode tipErrorCode = MappingUtils.mapStatusCode(tipSysError.getCode());
                            final Error newError = new Error();
                            newError.setReference(tipSysError.getReference());
                            newError.setReferenceType(tipSysError.getReferenceType());
                            newError.setCode(tipErrorCode.getCode());
                            newError.setMessage(MSStatusResolver.getStatusMessage(tipErrorCode));
                            storeResponse.getStatus().getErrors().add(newError);
                        }
                    }
                }
                return storeResponse;
            }

            Integer oldMedicationSchemeVersion = finalMedicationSchemeEntriesRequest.getVersion();
            storeResponse.setVersion(oldMedicationSchemeVersion == null ? 1 : oldMedicationSchemeVersion + 1);
            storeResponse.setLastUpdated(finalMedicationSchemeEntriesRequest.getCurrentDateTime().toGregorianCalendar());
            for (DataEntryRequest dataEntryRequest : finalMedicationSchemeEntriesRequest.getDataEntry()) {
                DataEntry dataEntry = new DataEntry();
                dataEntry.setReference(dataEntryRequest.getReference());
                if (dataEntryRequest.getMetaDataList() != null) {
                    for (MetaDataType metaDataType : dataEntryRequest.getMetaDataList().getMetaData()) {
                        dataEntry.getMetadata().put(metaDataType.getKey(), metaDataType.getValue());
                    }
                }
                dataEntry.getMetadata().put(SGUID_KEY, finalMedicationSchemeEntriesRequest.getId());
                dataEntry.getMetadata().put(DGUID_KEY, dataEntryRequest.getDguid());
                dataEntry.getMetadata().put(TGUID_KEY, dataEntryRequest.getDataEntryId());
                dataEntry.getMetadata().put(FORMAT_CODE_KEY, dataEntryRequest.getFormatCode().name());
                dataEntry.getMetadata().put(AVAILABILITY_STATUS_KEY, dataEntryRequest.getAvailabilityStatus().name());
                dataEntry.getMetadata().put(AUTHOR_ORGANIZATION_ID_KEY, finalSingleMessageInfo.getWrapper().getHeaderPharmacyId());
                dataEntry.getMetadata().put(AUTHOR_ORGANIZATION_ID_SOURCE_KEY, OrganizationIdType.NIHII.name());
                dataEntry.getMetadata().put(AUTHOR_ROLE_KEY, OrganizationRoleType.ORG_PHARMACY.name());

                dataEntry.setBusinessData(dataEntryRequest.getBusinessData());
                Integer oldDataEntryVersion = dataEntryRequest.getDataEntryVersion();
                int newDataEntryVersion = oldDataEntryVersion == null ? 1 : oldDataEntryVersion + 1;
                final DataEntryURI uri = DataEntryURI.readURI(finalMedicationSchemeEntriesRequest.getSubjectId(), dataEntryRequest.getDataEntryId(), newDataEntryVersion);
                dataEntry.setDataEntryURI(uri.toString());
                dataEntry.setNodeVersion(storeResponse.getVersion());

                storeResponse.getDataEntries().add(dataEntry);
                return storeResponse;
            }

        } catch (GFDDPPException e) {
            double difftime = (double) (System.nanoTime() - startTimeStamp) / 1000000000.0;
            LOG.info(MODULE + "=-----= einde " + procedure + " =$=" + info + " => " + " (dt:" + difftime + ") " + "error GFDDPPException: " + e.getMessage());
            throw new IntegrationModuleException(StatusResolver.resolveMessage(StatusResolver.getLocalResourceBundle(), e.getStatusCode(), e.getPlaceHolders()), e);
        } catch (Throwable t) {
            double difftime = (double) (System.nanoTime() - startTimeStamp) / 1000000000.0;
            LOG.info(MODULE + "=-----= einde " + procedure + " =$=" + info + " => " + " (dt:" + difftime + ") " + "error Throwable: " + t.getMessage());
            Exceptionutils.errorHandler(t);
        }
        return null;
    }

    @Override
    @Profiled(logFailuresSeparately = true, tag = "{$1}.TIPSystemIntegrationModuleImpl#revokeData", logger = "org.perf4j.TimingLogger_Executor")
    public void revokeData(byte[] singleMessage, String tipId, boolean requestPatientSignature, ThreeStateBoolean patientConsent) throws IntegrationModuleException {

        String procedure = "revokeData";
        String info = " ";
        LOG.info(MODULE + "=-----= start " + procedure + " =$=" + info);
        long startTimeStamp = System.nanoTime();

        try {
            if (getPropertyHandler().hasProperty("TIP_UTF_8_ONLY_INPUT")
                    && getPropertyHandler().getProperty("TIP_UTF_8_ONLY_INPUT").equals("true")) {
                ValidationUtils.validateUTF_8Input(singleMessage);
            }

            //=========================== singleMessage ======================================
            SingleMessageInfo smi = new SingleMessageInfo(singleMessage);
            info += smi.getInfo();

            //=========================== tipId ======================================
            tipId = getTipId(info, tipId);
            info += " tipId:" + tipId + " ";

            //=========================== validate ======================================
            smi.validateRevokeData();

            //=========================== NihiiPharmacyId ======================================
            String NihiiPharmacyId = getNihiiPharmacyId(info, smi.getHasSyncEventType());
            info += " NihiiPharmacyId:" + NihiiPharmacyId + " ";

            smi.validateNihiiPharmacyId(NihiiPharmacyId);

            smi.setHeaderNihiiPharmacyId(NihiiPharmacyId);

            //=========================== validateSingleMessageXsd ======================================
            smi.validateSingleMessageXsd();

            //=========================== setCommonData ======================================
            SealedMessageRequestType sealedMessageRequestType = smi.setCommonData(getEncryptionUtils(), tipId, requestPatientSignature, patientConsent);

            //=========================== send to queue ======================================
            SealedMessageRequestTypeXml sealedMessageRequestTypeXml = new SealedMessageRequestTypeXml();
            sealedMessageRequestTypeXml.setSealedMessageRequestType(sealedMessageRequestType);

            LOG.debug(MODULE + "=-----= ----- " + procedure + " =$=" + info + "=> Encrypt message for queue **********************");
            String s = getJaxContextCentralizer().toXml(SealedMessageRequestTypeXml.class, sealedMessageRequestTypeXml);

            String ids = smi.getEvents();

            xmlValidationhelper.assertValidSingleMessage(sealedMessageRequestType.getSingleMessageSealed());

            LOG.debug(MODULE + "=-----= ----- " + procedure + " =$=" + info + "=> Add revoke message on queue **********************");
            getFacade().deleteData(s.getBytes("UTF-8"), ids, null, smi.getHeaderPharmacyId());

        } catch (Throwable t) {
            double difftime = (double) (System.nanoTime() - startTimeStamp) / 1000000000.0;
            LOG.info(MODULE + "=-----= einde " + procedure + " =$=" + info + " => " + " (dt:" + difftime + ") " + "error Throwable: " + t.getMessage());
            Exceptionutils.errorHandler(t);
        }
    }

    @Override
    @Profiled(logFailuresSeparately = true, tag = "{$1}.TIPSystemIntegrationModuleImpl#updateData", logger = "org.perf4j.TimingLogger_Executor")
    public String updateData(byte[] singleMessage, String tipId, boolean requestPatientSignature, ThreeStateBoolean patientConsent) throws IntegrationModuleException {

        String procedure = "updateData";
        String info = " ";
        LOG.info(MODULE + "=-----= start " + procedure + " =$=" + info);
        long startTimeStamp = System.nanoTime();

        String ids = null;
        try {

            LOG.debug("===================== Validate single message UTF-8 bytes ==============================");
            if (getPropertyHandler().hasProperty("TIP_UTF_8_ONLY_INPUT")
                    && getPropertyHandler().getProperty("TIP_UTF_8_ONLY_INPUT").equals("true")) {
                ValidationUtils.validateUTF_8Input(singleMessage);
            }

            //=========================== singleMessage ======================================
            SingleMessageInfo smi = new SingleMessageInfo(singleMessage);
            info += smi.getInfo();

            //=========================== tipId ======================================
            tipId = getTipId(info, tipId);
            info += " tipId:" + tipId + " ";

            //=========================== validate ======================================
            smi.validateUpdateData();

            //=========================== NihiiPharmacyId ======================================
            String NihiiPharmacyId = getNihiiPharmacyId(info, smi.getHasSyncEventType());
            info += " NihiiPharmacyId:" + NihiiPharmacyId + " ";

            smi.validateNihiiPharmacyId(NihiiPharmacyId);

            smi.setHeaderNihiiPharmacyId(NihiiPharmacyId);

            //=========================== ProductFilter ======================================
            ProductFilterSingleton.getInstance(getPropertyHandler());
            smi.setFilterProducts(getProductFilterUtils(), getPropertyHandler());

            //=========================== validateSingleMessageXsd ======================================
            smi.validateSingleMessageXsd();

            //=========================== setCommonData ======================================
            SealedMessageRequestType sealedMessageRequestType = smi.setCommonData(getEncryptionUtils(), tipId, requestPatientSignature, patientConsent);

            //=========================== send to queue ======================================
            SealedMessageRequestTypeXml sealedMessageRequestTypeXml = new SealedMessageRequestTypeXml();
            sealedMessageRequestTypeXml.setSealedMessageRequestType(sealedMessageRequestType);

            LOG.debug(MODULE + "=-----= ----- " + procedure + " =$=" + info + "=> Encrypt message for queue **********************");
            String s = getJaxContextCentralizer().toXml(SealedMessageRequestTypeXml.class, sealedMessageRequestTypeXml);
            byte[] sealedSMCForQueue = s.getBytes();

            ids = smi.getEvents();

            xmlValidationhelper.assertValidSingleMessage(sealedMessageRequestType.getSingleMessageSealed());

            LOG.debug(MODULE + "=-----= ----- " + procedure + " =$=" + info + "=> Add update message on queue **********************");
            getFacade().updateData(sealedSMCForQueue, ids, smi.getMessageID(), smi.getHeaderPharmacyId());

        } catch (Throwable t) {
            double difftime = (double) (System.nanoTime() - startTimeStamp) / 1000000000.0;
            LOG.info(MODULE + "=-----= einde " + procedure + " =$=" + info + " => " + " (dt:" + difftime + ") " + "error Throwable: " + t.getMessage());
            Exceptionutils.errorHandler(t);
        }
        return ids;
    }

    @Override
    @Profiled(logFailuresSeparately = true, tag = "0.TIPSystemIntegrationModuleImpl#getStatusMessages", logger = "org.perf4j.TimingLogger_Executor")
    public String getStatusMessages(String sGuid, String dGuid) throws IntegrationModuleException {
        ApplicationConfig.getInstance().assertValidSession();
        LOG.debug("******************** Create getStatusMessage Request **********************");
        String xml = null;
        String response = null;
        try {
            String tipId = null;
            if (getPropertyHandler().hasProperty("default.tip.id")) {
                tipId = getPropertyHandler().getProperty("default.tip.id");
            }

            LOG.debug("******************** Validate incoming parameters **********************");
            ValidationUtils.validateIncomingFieldsGetStatusMessage(sGuid, dGuid);
            ValidationUtils.validateExistingTipId(tipId, getPropertyHandler());

            LOG.debug("******************** Get requestor information out of SAML **********************");
            String requestorId = StandaloneRequestorProvider.getRequestorIdInformation();
            String requestorType = StandaloneRequestorProvider.getRequestorTypeInformation();

            LOG.debug("******************** Create request getStatus **********************");
            xml = RequestCreatorUtils.createRequestGetStatus(requestorId, requestorType, sGuid, dGuid);

            byte[] RequestParametersSealed;
            if (StringUtils.isNoneBlank(sGuid)) {
            	RequestParametersSealed = SealedProcessor.createSealedSync(getEtkHelper().getTIPSystem_ETK(tipId).get(0), xml, "RequestGetStatus");
            } else {
            	RequestParametersSealed = SealedProcessor.createSealedAsync(getEtkHelper().getTIPSystem_ETK(tipId).get(0), xml, "RequestGetStatus");
            }
           
            RoutedSealedRequestType routedSealedRequest = new RoutedSealedRequestType();
            routedSealedRequest.setRequestParametersSealed(RequestParametersSealed);

            LOG.debug("******************** Retrieve Status **********************");
            RoutedSealedResponseType routedSealedResponse = null;
            try {
                routedSealedResponse = TipSystemServiceImpl.getInstance().retrieveStatusMessages(routedSealedRequest);
            } catch (SOAPFaultException e) {
                throw new IntegrationModuleException(e);
            }
            LOG.debug("******************** Building GetStatus Response **********************");

            byte[] singleMessageResponse = null;
            if (routedSealedResponse.getSingleMessageSealed() != null
                    && routedSealedResponse.getSingleMessageSealed().length > 1) {
                singleMessageResponse = Crypto.unseal(routedSealedResponse.getSingleMessageSealed());
                xmlValidationhelper.assertValidSingleMessage(singleMessageResponse);
            }

            singleMessageResponse = consolidationStatusResponse(sGuid, dGuid, singleMessageResponse);

            response = new String(singleMessageResponse, "UTF-8");
        } catch (IntegrationModuleException e) {
            LOG.error("Integration Module Exception " + e.getMessage());
            throw e;
        } catch (Throwable t) {
            Exceptionutils.errorHandler(t);
        }
        LOG.debug("******************** Return getStatus response **********************");
        LOG.debug(response);
        return response;
    }

    @Override
    public void shutdownGracefully() throws IntegrationModuleException {
        LOG.debug("******************** Shutdown application gracefully **********************");
        if (getControle() != null) {
            getControle().shutdown();
        }
    }

    public static QueueControle getControle() {
        return controle;
    }

    public static void setControle(QueueControle controle) {
        TIPSystemIntegrationModuleImpl.controle = controle;
    }

    public static EHealthMessageQueueFacade getFacade() {
        return facade;
    }

    public static void setFacade(EHealthMessageQueueFacade facade) {
        TIPSystemIntegrationModuleImpl.facade = facade;
    }

    private byte[] consolidationStatusResponse(String sGuid, String dGuid, byte[] response) throws IntegrationModuleException, GFDDPPException {
        LOG.debug("******************** Consolidation status message **********************");
        SingleMessageBuilder singleMessageBuilder = new SingleMessageBuilder();
        SingleMessage singleMessage = null;
        if (response != null) {
            try {
                singleMessage = getJaxContextCentralizer().toObject(SingleMessage.class, response);
                addMessageToLocalQueue(singleMessage);
            } catch (GFDDPPException e) {
                LOG.debug("******************** The response was not unmarshable **********************");
                singleMessage = singleMessageBuilder.createSingleMessageUnsigned();
            }

        } else {
            singleMessage = singleMessageBuilder.createSingleMessageUnsigned();
        }
        LOG.debug("******************** Retrieving status message from queue **********************");

        SingleMessage singleMessageResponse = singleMessageBuilder.createSingleMessageUnsigned();
        singleMessageResponse.getUnsigned().setHeader(singleMessage.getUnsigned().getHeader());

        singleMessageResponse = retrieveStatusMessageFromLocalQueue(dGuid, sGuid, singleMessageBuilder, singleMessageResponse);

        LOG.debug("******************** Transform response to array **********************");
        try {
            return getJaxContextCentralizer().toXml(SingleMessage.class, singleMessageResponse).getBytes("UTF-8");
        } catch (UnsupportedEncodingException e) {
            LOG.error("", e);//TODO deal with invalid utf-8 charactes in status messages
        }
        return null;
    }

    private SingleMessage retrieveStatusMessageFromLocalQueue(String dGuid, String sGuid, SingleMessageBuilder singleMessageBuilder, SingleMessage singleMessageResponse) throws IntegrationModuleException {
        List<StatusMessageType> statusMessageTypes = new ArrayList<StatusMessageType>();

        if (statusMessageQueue != null && !statusMessageQueue.isEmpty()) {
            StatusMessageType localStatus = null;

            try {
                localStatus = statusMessageQueue.poll();
            } catch (Exception e) {
                throw new IntegrationModuleException(I18nHelper.getLabel("error.technical.status.queue"));
            }

            while (localStatus != null) {
                statusMessageTypes.add(localStatus);
                localStatus = statusMessageQueue.poll();
            }

            for (StatusMessageType statusMessageType : statusMessageTypes) {
                if (shouldStatusMessageBeAdded(statusMessageType, sGuid, dGuid)) {
                    singleMessageBuilder.addEntity(statusMessageType, singleMessageResponse);
                } else {
                    statusMessageQueue.add(statusMessageType);
                }
            }
        }

        return singleMessageResponse;
    }

    private boolean shouldStatusMessageBeAdded(StatusMessageType statusMessageType, String sGuid, String dGuid) {
        String s = null;
        String d = null;

        SubjectReferenceType subjectReference = statusMessageType.getSubjectReference();
        if (subjectReference != null) {
            s = statusMessageType.getSubjectReference().getSessionGUID();
            d = statusMessageType.getSubjectReference().getDispensationGUID();
        }

        return (!StringUtils.isEmpty(sGuid) && !StringUtils.isEmpty(dGuid) && StringUtils.equals(sGuid, s) && StringUtils.equals(dGuid, d))
                || (!StringUtils.isEmpty(sGuid) && StringUtils.isEmpty(dGuid) && StringUtils.equals(sGuid, s))
                || (StringUtils.isEmpty(sGuid) && StringUtils.isEmpty(dGuid));
    }

    public StatusMessageQueue getStatusMessageQueue() {
        return statusMessageQueue;
    }

    private void addMessageToLocalQueue(SingleMessage singleMessage) {
        LOG.debug("******************** Copying message receive from tip **********************");

        SingleMessageWrapper smw = new SingleMessageWrapper(singleMessage);
        for (AbstractEntityType entity : smw.getEntities()) {
            if (entity instanceof StatusMessageType) {
                statusMessageQueue.add((StatusMessageType) entity);
            }
        }
        LOG.debug("******************** Copying message receive from tip (DONE) **********************");
    }

}
