/**
 * Copyright (C) 2010 Recip-e
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package be.business.connector.gfddpp.tipsystem.mock;

import be.apb.gfddpp.common.exceptions.GFDDPPException;
import be.apb.gfddpp.common.status.StatusCode;
import be.apb.standards.smoa.schema.id.v1.NihiiIdType;
import be.apb.standards.smoa.schema.id.v1.ObjectFactory;
import be.apb.standards.smoa.schema.model.v1.MessageInformation;
import be.apb.standards.smoa.schema.model.v1.OriginatorType;
import be.apb.standards.smoa.schema.model.v1.StatusMessageType;
import be.apb.standards.smoa.schema.model.v1.SubjectReferenceType;
import be.apb.standards.smoa.schema.v1.*;
import be.apb.standards.smoa.schema.v1.EventFolderType.EntitySpace;
import be.apb.standards.smoa.schema.v1.EventFolderType.Events;
import be.business.connector.core.exceptions.IntegrationModuleEhealthException;
import be.business.connector.core.exceptions.IntegrationModuleException;
import be.business.connector.core.exceptions.IntegrationModuleRuntimeException;
import be.business.connector.core.utils.Exceptionutils;
import be.business.connector.core.utils.I18nHelper;
import be.business.connector.gfddpp.domain.ThreeStateBoolean;
import be.business.connector.gfddpp.tipsystem.TIPSystemIntegrationModuleImpl;
import be.business.connector.gfddpp.tipsystem.status.offline.queue.StatusMessageQueueMock;
import be.business.connector.gfddpp.utils.SingleMessageInfo;
import be.ehealth.apb.gfddpp.services.tipsystem.SimpleResponseType;
import be.ehealth.apb.gfddpp.services.tipsystem.StatusType;
import org.apache.commons.io.Charsets;
import org.apache.log4j.Logger;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import java.util.GregorianCalendar;

public class TIPSystemIntegrationModuleMock extends TIPSystemIntegrationModuleImpl {
	private final static Logger LOG = Logger.getLogger(TIPSystemIntegrationModuleMock.class);
	//private static StatusMessageQueue<StatusMessageType> statusMessageQueue;
	/*
	 * those 2 string has to be the same than in the statusMessageQueue !
	 */
	private static final String STATUS_MESSAGE_QUEUE_FOLDER_PROPERTY_NAME = "STATUS_MESSAGE_QUEUE_FOLDER";
	private static final String FILE_NAME_SEPARATOR = "_";
	
	public TIPSystemIntegrationModuleMock() throws IntegrationModuleException {
		super(false);
//		PropertyHandler.getInstance().getProperties().put(STATUS_MESSAGE_QUEUE_FOLDER_PROPERTY_NAME, "statusMessageQueue");
		statusMessageQueue = new StatusMessageQueueMock();

	}
	
	@Override
	public String registerData(byte[] singleMessage, String tipId, boolean requestPatientSignature, ThreeStateBoolean patientConsent) throws IntegrationModuleException {
		String ids = null;
		try {
			LOG.debug("******************** Marshal byte[] to SMOAMessagetype  **********************");
			String singleMessageString = new String(singleMessage, Charsets.UTF_8);
			SingleMessage lsingleMessageObject = getJaxContextCentralizer().toObject(SingleMessage.class, singleMessageString);
			SingleMessageInfo smi = new SingleMessageInfo();
                        smi.initSingleMessageInfo(lsingleMessageObject);
			//info += smi.info;
			
			LOG.debug("******************** Validate event type **********************");
			smi.validateRegisterData();
			
			LOG.debug("******************** get all ids **********************");
			
			smi.setGuids();
			ids = smi.getEvents();
			LOG.info("Data is Registered!");
		} catch (Throwable t) {
			Exceptionutils.errorHandler(t);
		}
		return ids;
	}

	@Override
	protected TIPSystemIntegrationModuleImpl.RegisterEventResponse registerEvent(byte[] singleMessage, String info, String tipId, boolean requestPatientSignature, ThreeStateBoolean patientConsent, String procedure, long startTimeStamp) throws IntegrationModuleEhealthException, IntegrationModuleException, IntegrationModuleRuntimeException {
		final SingleMessageInfo singleMessageInfo = new SingleMessageInfo(singleMessage);

		if("registerMedicationSchemeEntries".equals(procedure)) {
			try {
				singleMessageInfo.validateRegisterData();
				singleMessageInfo.setGuids();
				singleMessageInfo.setHeaderNihiiPharmacyId("62599147"); // random pharmacy

				final SimpleResponseType tipSuccess = new SimpleResponseType();
				tipSuccess.setStatus(new StatusType());
				tipSuccess.getStatus().setCode(StatusCode.TIP_200.name());
				final RegisterEventResponse response = new RegisterEventResponse();
				response.setSingleMessageInfo(singleMessageInfo);
				response.setTipResponse(tipSuccess);
				return response;
			} catch (Throwable t) {
				Exceptionutils.errorHandler(t);
				return null;
			}
		}else{
			return super.registerEvent(singleMessage, info, tipId, requestPatientSignature, patientConsent, procedure, startTimeStamp);
		}
	}

	@Override
	public void revokeData(byte[] singleMessage, String tipId, boolean requestPatientSignature, ThreeStateBoolean patientConsent) throws IntegrationModuleException {
		try {
			LOG.debug("******************** Marshal byte[] to SMOAMessagetype  **********************");
			String singleMessageString = new String(singleMessage, Charsets.UTF_8);
			SingleMessage lsingleMessageObject = getJaxContextCentralizer().toObject(SingleMessage.class, singleMessageString);
			SingleMessageInfo smi = new SingleMessageInfo();
                        smi.initSingleMessageInfo(lsingleMessageObject);
			LOG.debug("******************** Validate event type **********************");
			smi.validateRevokeData();
			
			LOG.info("Data is Revoked!");
			
		} catch (Throwable t) {
			Exceptionutils.errorHandler(t);
		}
	}
	
	@Override
	public String updateData(byte[] singleMessage, String tipId, boolean requestPatientSignature, ThreeStateBoolean patientConsent) throws IntegrationModuleException {
		String ids = null;
		try {
			
			LOG.debug("******************** Marshal byte[] to SMOAMessagetype  **********************");
			String singleMessageString = new String(singleMessage, Charsets.UTF_8);
			SingleMessage lsingleMessageObject = getJaxContextCentralizer().toObject(SingleMessage.class, singleMessageString);
			SingleMessageInfo smi = new SingleMessageInfo();
                        smi.initSingleMessageInfo(lsingleMessageObject);
			LOG.debug("******************** Validate event type **********************");
			smi.validateUpdateData();
			
			LOG.debug("******************** get all ids **********************");
			
			ids = smi.getEvents();
			LOG.info("Data is Updated!");
		} catch (Throwable t) {
			Exceptionutils.errorHandler(t);
		}
		return ids;
	}
	
	public String getStatusMessages(String sGuid, String dGuid) throws IntegrationModuleException {

		be.apb.standards.smoa.schema.v1.ObjectFactory schemaObjectFactory = new be.apb.standards.smoa.schema.v1.ObjectFactory();
		be.apb.standards.smoa.schema.model.v1.ObjectFactory modelObjectFactory = new be.apb.standards.smoa.schema.model.v1.ObjectFactory();
		be.apb.standards.smoa.schema.id.v1.ObjectFactory idObjectFactory = new ObjectFactory();
		DatatypeFactory dataTypeFactory;
		byte[] xml = null;
		try {
			dataTypeFactory = DatatypeFactory.newInstance();
			
			LOG.debug("********************** getStatusMessage - creating Single Message Structure ********************** ");
			SingleMessage singleMessage = schemaObjectFactory.createSingleMessage();
			
			SmoaMessageType message = schemaObjectFactory.createSmoaMessageType();

			String requestorId = "62599147";
			
			message.setHeader(createHeader(requestorId, dataTypeFactory, schemaObjectFactory));
			
			EventFolderType eventFolderType = schemaObjectFactory.createEventFolderType();
			
			
			Events events = new Events();
			eventFolderType.setEvents(events);
			
			EntitySpace entitySpace = new EntitySpace();
			
			eventFolderType.setEntitySpace(entitySpace);
			
			StatusMessageType statusMessageEntity = 	new StatusMessageType();
			
			OriginatorType originator = modelObjectFactory.createOriginatorType();
			originator.setOriginatorId("62599147");
			originator.setServiceType(ServiceType.CM.toString());
			statusMessageEntity.setOriginator(originator);
			
			MessageInformation messageInformation = modelObjectFactory.createMessageInformation();
			messageInformation.setMessageSubType("401000");
			messageInformation.setMessageType(MessageType.ERROR);
			messageInformation.setMessageDetails("Failed to write to audit log");
			
			GregorianCalendar gregorianCalendar = new GregorianCalendar();
			gregorianCalendar.setTimeInMillis(System.currentTimeMillis());
			messageInformation.setDateTime(dataTypeFactory.newXMLGregorianCalendar(gregorianCalendar));
			
			statusMessageEntity.setMessageInformation(messageInformation);
			SubjectReferenceType subjectReference = modelObjectFactory.createSubjectReferenceType();
			subjectReference.setMessageGUID("12345678-1234-1234-1234-0123456789AB");
			subjectReference.setDispensationGUID("12345678-1234-1234-1234-0123456789AB");
			subjectReference.setSessionGUID("12345678-5234-1244-1634-0123467789DB");
			statusMessageEntity.setSubjectReference(subjectReference);
			
			//DeliveriesType deliveriesType = schemaObjectFactory.createDeliveriesType();
		
			entitySpace.getEntity().add(statusMessageEntity);
			message.setAbstractFolder(schemaObjectFactory.createEventFolder(eventFolderType));
			
			singleMessage.setUnsigned(message);
			
			xml = getJaxContextCentralizer().toXml(SingleMessage.class, singleMessage).getBytes();
		} catch (DatatypeConfigurationException e) {
			throw new IntegrationModuleException(I18nHelper.getLabel("error.technical"));
		} catch (GFDDPPException e) {
			throw new IntegrationModuleException(e);
		}
		return new String(xml);
	}
	
	private HeaderType createHeader(String nihiiPharmacyNumber, DatatypeFactory dataTypeFactory, be.apb.standards.smoa.schema.v1.ObjectFactory schemaObjectFactory) {
		HeaderType header = schemaObjectFactory.createHeaderType();
		GregorianCalendar gregorianCalendar = new GregorianCalendar();
		gregorianCalendar.setTimeInMillis(System.currentTimeMillis());
		header.setMessageCreateDate(dataTypeFactory.newXMLGregorianCalendar(gregorianCalendar));
		SenderType sender = schemaObjectFactory.createSenderType();
		
		be.apb.standards.smoa.schema.model.v1.ReferencePharmacyType referencePharmacyType = new be.apb.standards.smoa.schema.model.v1.ReferencePharmacyType();
		NihiiIdType nihiiIdType = new NihiiIdType();
		nihiiIdType.setNihiiPharmacyNumber(nihiiPharmacyNumber);
		referencePharmacyType.setPharmacyId(nihiiIdType);
		be.apb.standards.smoa.schema.model.v1.ObjectFactory objectFactory = new be.apb.standards.smoa.schema.model.v1.ObjectFactory();
		sender.setAbstractPharmacy(objectFactory.createAbstractPharmacy(referencePharmacyType));

		return header;
	}
	
	@Override
	public void shutdownGracefully() {
	
	}
}
