package be.business.connector.gfddpp.tipsystem.offline.queue;

import be.apb.gfddpp.common.exceptions.GFDDPPException;
import be.apb.gfddpp.common.utils.JaxContextCentralizer;
import be.apb.gfddpp.helper.SingleMessageValidationHelper;
import be.apb.gfddpp.rtrn.registerdata.ArchivePrescriptionCommentEventType;
import be.apb.gfddpp.rtrn.registerdata.ArchivePrescriptionEventType;
import be.apb.gfddpp.rtrn.registerdata.Dispensation;
import be.apb.gfddpp.rtrn.registerdata.Events;
import be.apb.gfddpp.validation.exception.SingleMessageValidationException;
import be.apb.standards.smoa.schema.model.v1.*;
import be.apb.standards.smoa.schema.v1.MessageType;
import be.apb.standards.smoa.schema.v1.ServiceType;
import be.apb.standards.smoa.schema.v1.SingleMessage;
import be.apb.standards.smoa.schema.v1.TextType;
import be.business.connector.common.StandaloneRequestorProvider;
import be.business.connector.core.exceptions.IntegrationModuleException;
import be.business.connector.core.exceptions.IntegrationModuleRuntimeException;
import be.business.connector.core.utils.*;
import be.business.connector.gfddpp.domain.SealedMessageRequestTypeXml;
import be.business.connector.gfddpp.tipsystem.status.offline.queue.StatusMessageQueue;
import be.business.connector.gfddpp.utils.SingleMessageInfo;
import be.business.connector.projects.common.services.tipsystem.TipSystemServiceImpl;
import be.business.connector.projects.common.utils.SystemServicesUtils;
import be.business.connector.projects.common.utils.ValidationUtils;
import be.ehealth.apb.gfddpp.services.tipsystem.*;
import be.ehealth.technicalconnector.exception.TechnicalConnectorException;
import be.ehealth.technicalconnector.session.Session;
import be.fgov.ehealth.etee.crypto.encrypt.EncryptionToken;
import com.sun.xml.ws.client.ClientTransportException;
import org.apache.commons.io.Charsets;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.log4j.Logger;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.ws.soap.SOAPFaultException;
import java.io.File;
import java.io.IOException;
import java.util.*;

public class QueueBehavior implements Runnable {

    private static Logger LOG = Logger.getLogger(QueueBehavior.class);
    private final static String UNENCRYPTED_FOLDER_PROPERTY_NAME = "UNENCRYPTED_FOLDER";
    private final static String INVALID_MESSAGES_FOLDER_PROPERTY_NAME = "INVALID_FOLDER";
    private String unencrypted_folder_name;
    private String invalid_folder_name;
    private final MessageQueue<EHealthMessage> queue;
    private final StatusMessageQueue statusQueue;
    private final ETKHelper etkHelper;
    private final PropertyHandler propertyHandler;
    private final JaxContextCentralizer jaxContextCentralizer = JaxContextCentralizer.getInstance();
    private final Set<String> retransmissionErrors = new HashSet<>(Arrays.asList("SOA-00001", "SOA-01001", "SOA-01002", "SOA-02001"));
    private final SingleMessageValidationHelper xmlValidationhelper = new SingleMessageValidationHelper();
    private final static String SUCCESS = "TIP_200";
    private final static String TIP_201 = "TIP_201";
    private final static String ERROR = "500";
    private int sendRetryCount = 1;
    private final int rescheduledTime;
    public static boolean shouldStopProcessing = false;

    public QueueBehavior(MessageQueue<EHealthMessage> queue, StatusMessageQueue statusQueue) throws IntegrationModuleException {
        this.queue = queue;
        this.etkHelper = new ETKHelper(PropertyHandler.getInstance(), EncryptionUtils.getInstance());
        this.propertyHandler = PropertyHandler.getInstance();
        this.statusQueue = statusQueue;
        jaxContextCentralizer.addContext(Events.class);
        jaxContextCentralizer.addContext(SealedMessageRequestTypeXml.class);
        initFailedMsgFolders(this.propertyHandler);
        rescheduledTime = this.propertyHandler.getIntegerProperty("MESSAGE_QUEUE_TIMER");

    }

    private void proceedResponse(SimpleResponseType response, EHealthMessage eHealthMessage, MessageQueue<EHealthMessage> q) {
        if (response != null) {
            LOG.info("The response code: " + response.getStatus().getCode() + " is returned from TIP System.");
            if (response.getStatus().getCode().equals(SUCCESS)) {
                q.remove();
                LOG.info("Send to tip success, Removed from local queue " + eHealthMessage.getdGuidSGuid()
                        + " for operation " + eHealthMessage.getMethodName());
            } else //response not successfull
            if (StringUtils.equals(response.getStatus().getCode(), TIP_201)) {
                LOG.info("Send to tip success with " + TIP_201 + ". Message is removed an put back on queue for retransmission.");
                q.remove();
                q.add(eHealthMessage);
            } else {
                //Is it the case of failure ?? all but 200 or 2001
                q.remove();
                LOG.info("Send to tip failed, Removed from local queue " + eHealthMessage.getdGuidSGuid());
                createPutStatusMessage(eHealthMessage, response, null);
                LOG.info("Send to tip success, Removed from local queue " + eHealthMessage.getdGuidSGuid());
            }//response not successfull

        } //response == null
        else {
            LOG.info("Response from the tip is null, something went wrong while trying to send a message to the tip");
        }
    }

    @Override
    public void run() {
        LOG.debug(">>> daemon process");
        OfflineQueueAssitant offlineQueueAssitant = new OfflineQueueAssitant();
        EHealthMessage eHealthMessage = null;
        SimpleResponseType response = null;

        while (!queue.isEmpty() && !offlineQueueAssitant.isStopOfflineQueue() && !queue.isFirstItemLocked()) {
            LOG.debug("***** Retrieve message from queue ....****");
            try {
                offlineQueueAssitant = new OfflineQueueAssitant();
                SealedMessageRequestType sealedMessageRequestType = null;
                String methodName;
                SingleMessage singleMessageObject;
                EncryptionToken etk = null;
                try {
                    //we have null in case when message can not be decrypted 
                    eHealthMessage = queue.elementAndLockItem();
                    if (null == eHealthMessage) {

                        offlineQueueAssitant.setMoveToUnencryptable(true);
                        throw new IntegrationModuleException("unable to decrypt message");
                    }

                    methodName = eHealthMessage.getMethodName();
                    LOG.info("Message picked from offline queue for " + methodName + " operation. Contain : " + eHealthMessage.getdGuidSGuid());

                    if (SessionValidator.isValidSession(Session.getInstance().getSession())) {//invalid session STOP processing
                        LOG.info("Has a Valid session to ehealth");

                        try {
                            sealedMessageRequestType = extractSealedMessageRequest(eHealthMessage);
                            singleMessageObject = extractSingleMessage(sealedMessageRequestType);
                            LOG.debug("******************** Validate nihii pharmacy  and set Headers in single message **********************");

                            SingleMessageInfo smi = new SingleMessageInfo();
                            smi.initSingleMessageInfo(singleMessageObject);

                            String NihiiPharmacyId = smi.validateNihiiPharmacyId(StandaloneRequestorProvider.getRequestorIdInformation());
                            smi.validateSingleMessageHeaderFilled(NihiiPharmacyId);

                            sealedMessageRequestType.setSingleMessageSealed(smi.getXmlBytes());

                            xmlValidationhelper.assertValidSingleMessage(sealedMessageRequestType.getSingleMessageSealed());

                        } catch (IntegrationModuleException e) {

                            LOG.error("The message couldn't be extracted. Incorrect message in queue");
                            offlineQueueAssitant.setMoveToInvalid(true);
                            throw e;
                        } catch (GFDDPPException e) {

                            offlineQueueAssitant.setMoveToInvalid(true);
                            LOG.error("Error transforming SingleMessage to XML ");
                            throw new IntegrationModuleException("Error transforming SingleMessage to XML", e);

                        } catch (SingleMessageValidationException e) {

                            LOG.error("single message not valid to XSD");
                            offlineQueueAssitant.setMoveToInvalid(true);//TODO clear text
                            throw new IntegrationModuleException("single message content INVALID TO xsd", e);
                        }

                        // tipID setting && validation
                        if (sealedMessageRequestType.getRoutingParameters() != null
                                || isTipIDEmpty(sealedMessageRequestType)) {

                            sealedMessageRequestType.getRoutingParameters().
                                    setTIPId(propertyHandler.getProperty("default.tip.id"));
                        } else {
                            //What should go here ???
                        }
                        String tipid = sealedMessageRequestType.getRoutingParameters().getTIPId();

                        try {
                            ValidationUtils.validateExistingTipId(
                                    sealedMessageRequestType.getRoutingParameters().getTIPId(), propertyHandler);
                        } catch (IntegrationModuleException e) {
                            LOG.info("TIP ID validation exception :"
                                    + sealedMessageRequestType.getRoutingParameters().getTIPId() + " invalid");
                            offlineQueueAssitant.setMoveToInvalid(true);
                            offlineQueueAssitant.setCreateStatusMessage(true);
                            throw e;
                        }

                        //can encryption token be changed on the fly or become invalid somehow ????
                        try {
                            etk = extractEncryptionToken(tipid);
                        } catch (IntegrationModuleException e) {
                            LOG.info("====================Couldn't retrieve ETK==============================");
                            offlineQueueAssitant.setStopOfflineQueue(true);
                            throw e;
                        }
                        byte[] tmpSingleMsgBytes = sealedMessageRequestType.getSingleMessageSealed();

                        /* As the code has no effect we comment it out 25th September 2015    2015/09/25
					 * The code has no effect here we set the signature and later efectivelly ignore the change*/
                        /*  if ( sealedMessageRequestType.getAuthorizationParameters().getPatientConsent() != null && 
						sealedMessageRequestType.getAuthorizationParameters().getPatientConsent().getPatientSignature() != null &&
						sealedMessageRequestType.getAuthorizationParameters().getPatientConsent().getPatientSignature().getSignature() != null ) {
						try {
							//TODO What is happening here ???? We seal patient signature DO We need encryption here ??????
							//this piece has no effect should there be double encryption ???
							byte[] patientSignUnsealed =sealedMessageRequestType.getAuthorizationParameters().
									getPatientConsent().getPatientSignature().getSignature();
							//encrypt patient signature bytes
							byte[] patientSignSealed = Crypto.seal(etk, patientSignUnsealed );
							
							sealedMessageRequestType.getAuthorizationParameters().getPatientConsent().
									getPatientSignature().setSignature(	patientSignSealed	);
							
						} catch (IntegrationModuleException e) {
							LOG.info("Can not seal the patient signature for the tip for the following dispensation: " + eHealthMessage.getdGuidSGuid());
							offlineQueueAssitant.setStopOfflineQueue(true);
							throw new IntegrationModuleException(I18nHelper.getLabel("error.data.seal"));
						}
					}*//* end not effective code */

                        byte[] sealedMessage = sealedMessageRequestType.getSingleMessageSealed();

                        try {//encryption of whole sealedMessageRequestType
                            sealedMessage = SealedProcessor.createSealedAsync(etk, tmpSingleMsgBytes, "SingleMessage");

                        } catch (IntegrationModuleException e) {

                            LOG.info("Can not seal the request for the tip for the following dispensation: " + eHealthMessage.getdGuidSGuid());
                            offlineQueueAssitant.setStopOfflineQueue(true);
                            throw new IntegrationModuleException(I18nHelper.getLabel("error.data.seal"));
                        }

                        sealedMessageRequestType.setSingleMessageSealed(sealedMessage);

                        LOG.debug("***** Send retrieved message from queue start ....****"); /*Thread.sleep(1000L); When debugging DO NOT REMOVE */

                        try {//================TRY SEND MESSAGE TO TIP ===========================================
                            LOG.info("Try to send message with id: " + eHealthMessage.getdGuidSGuid());
                            response = sendMessage(methodName, sealedMessageRequestType,
                                    sealedMessageRequestType.getRoutingParameters().getTIPId(),
                                    offlineQueueAssitant);

                        } catch (IntegrationModuleException e) {
                            //unwrapping Technical
                            if (e.getCause() instanceof TechnicalConnectorException) {
                                sendRetryCount = sendRetryCount * 2;
                                queue.unlockFirstFile();
                            }
                            throw e;
                        } catch (SystemError_Exception e) {

                            if (e.getFaultInfo() != null && retransmissionErrors.contains(e.getFaultInfo().getCode())) {
                                LOG.info("Send to tip failed with  error: "
                                        + e.getFaultInfo().getCode()
                                        + ". File not removed from queue. Will try to send the message again the next time");
                            } else {
                                LOG.info("Send to tip failed, invalid request, Removed from local queue " + eHealthMessage.getdGuidSGuid());
                                sendRetryCount = sendRetryCount * 2;
                                queue.unlockFirstFile();
                            };
                            throw new IntegrationModuleException(e.getMessage());

                        } catch (SOAPFaultException e) {

                            if (retransmissionErrors.contains(e.getMessage())) {
                                LOG.info("Send to tip failed but was error: " + e.getMessage()
                                        + ". File not removed from queue. Will try to send the message again the next time");
                            } else {
                                LOG.info("Send to tip failed, invalid request, Removed from local queue " + eHealthMessage.getdGuidSGuid());
                                queue.unlockFirstFile();
                                sendRetryCount = sendRetryCount * 2;
                            }
                            throw new IntegrationModuleException(e.getMessage());
                        } catch (ClientTransportException e) {

                            LOG.error("No connection could be made with the TIP System. Retry after: "
                                    + propertyHandler.getProperty("MESSAGE_QUEUE_TIMER")
                                    + " for the following sguid dguid : " + eHealthMessage.getdGuidSGuid());
                            LOG.error(e);

                            queue.unlockFirstFile();
                            //exponential back-off
                            sendRetryCount = sendRetryCount * 2;
                            throw new IntegrationModuleException(I18nHelper.getLabel("error.connection.tip"));
                        }

                        LOG.debug("***** Check response message send from TIP system....****");
                        sendRetryCount = 1;
                        proceedResponse(response, eHealthMessage, queue);
                    } else {
                        LOG.info("No valid session with eHealth. Retry after: " + propertyHandler.getProperty("MESSAGE_QUEUE_TIMER"));
                        offlineQueueAssitant.setStopOfflineQueue(true);// CORRECT
                        queue.unlockFirstFile();
                    }
                } catch (IntegrationModuleException e) {
                    proceedIntegrationModuleException(e, eHealthMessage, offlineQueueAssitant);
                }

            } catch (Exception e) {
                //TODO recreate lack of network connection. Unhandled and unspecified exception is throwsn aka TechnicalConnectionError
                LOG.error("Unexpected exception occured. Will try to unlock first file on queue.", e);
                queue.unlockFirstFile();

            }
        }//while should stop running constantly
        LOG.debug("<<< daemon process\n");
        if (true == shouldStopProcessing) {
            throw new RuntimeException();
        }

    }

    private boolean isTipIDEmpty(
            SealedMessageRequestType sealedMessageRequestType) {

        return org.apache.commons.lang.StringUtils.isEmpty(sealedMessageRequestType.getRoutingParameters().getTIPId());
    }

    private void proceedIntegrationModuleException(IntegrationModuleException e, EHealthMessage eHealthMessage, OfflineQueueAssitant offlineQueueAssitant) {
        if (offlineQueueAssitant.isCreateStatusMessage()) {
            putStatusMessageOnQueue(createStatusMessage(eHealthMessage, null, e));
        }
        if (offlineQueueAssitant.isRemoveFromQueue()) {
            if (eHealthMessage != null) {
                LOG.info("Message containing : " + eHealthMessage.getdGuidSGuid() + " will be removed from queue.");
            }
            queue.remove();
            LOG.info("Message removed");
        }
        if (offlineQueueAssitant.isAddBackToQueue()) {
            queue.remove();
            queue.add(eHealthMessage);
            LOG.info("Message containing : " + eHealthMessage.getdGuidSGuid() + " added back to queue");
        }

        if (offlineQueueAssitant.isMoveToInvalid()) {
            moveQueueMsgPlainText(eHealthMessage);
            queue.moveAbnormal(invalid_folder_name);

        }
        if (offlineQueueAssitant.isMoveToUnencryptable()) {
            queue.moveAbnormal(unencrypted_folder_name);
        }

        try {
            Thread.sleep(/*miliseconds*/1000L * rescheduledTime/*seconds*/ * sendRetryCount);
        } catch (InterruptedException e1) {
            // TODO Auto-generated catch block
            LOG.warn("Error occurs: ", e1);
        }

    }

    private SimpleResponseType sendMessage(String methodName,
            SealedMessageRequestType sealedMessageRequestType,
            String tipId, OfflineQueueAssitant offlineQueueAssitant) throws SystemError_Exception, IntegrationModuleException {

        String endpoint = SystemServicesUtils.getInstance(propertyHandler).getEndpointOutOfSystemConfiguration(propertyHandler.getProperty("tipsystem.id"), "TIP", "TIPService");

        if (endpoint == null) {
            LOG.error("***** No endpoint found in system services configuration for TIP: " + tipId + " - Not going to send message to TIP! ****");
            offlineQueueAssitant.removeAndCreateStatusMessage();
            offlineQueueAssitant.setStopOfflineQueue(true);
            throw new IntegrationModuleException(I18nHelper.getLabel("error.technical.endpoint.not.found"));
        }

        LOG.debug("***** Send message to endpoint: " + endpoint + "****");

        LOG.debug("***** Send message from queue: " + methodName + "****");
        if (methodName.equals(MethodName.REGISTERDATA.toString())) {
            return TipSystemServiceImpl.getInstance().registerData(sealedMessageRequestType);
        } else if (methodName.equals(MethodName.UPDATEDATA.toString())) {
            return TipSystemServiceImpl.getInstance().updateData(sealedMessageRequestType);
        } else if (methodName.equals(MethodName.REVOKEDATA.toString())) {
            return TipSystemServiceImpl.getInstance().deleteData(sealedMessageRequestType);
        }
        return null;         //not register-update-remove
    }

    private void createPutStatusMessage(EHealthMessage eHealthMessage, SimpleResponseType response, IntegrationModuleException e) {
        putStatusMessageOnQueue(createStatusMessage(eHealthMessage, response, e));
        LOG.info("Local status message created locally for : " + eHealthMessage.getdGuidSGuid());
    }

    private List<StatusMessageType> createStatusMessage(
            EHealthMessage messageOnQueue,
            SimpleResponseType response, IntegrationModuleException ex) {
        LOG.debug("***** CREATE STATUS MESSAGE ****");
        be.apb.standards.smoa.schema.model.v1.ObjectFactory modelObjectFactory = new be.apb.standards.smoa.schema.model.v1.ObjectFactory();

        Events events = null;
        try {
            events = jaxContextCentralizer.toObject(Events.class, messageOnQueue.getdGuidSGuid().getBytes());
        } catch (GFDDPPException e) {
            LOG.error("A problem occur while trying to unmarshall DeliveryIds" + e.getMessage());
        }

        List<StatusMessageType> statusMessageTypes = new ArrayList<>();

        StatusMessageType statusMessage;
        Dispensation products;
        if (events != null && events.getPharmaceuticalCareEventType() != null) {
            for (Iterator iterator = events.getPharmaceuticalCareEventType().getDispensation().iterator(); iterator.hasNext();) {
                products = (Dispensation) iterator.next();
                statusMessage = modelObjectFactory.createStatusMessageType();
                statusMessage.setSubjectReference(createSubjectReferenceType(products.getDguid(), events.getPharmaceuticalCareEventType().getSguid(), messageOnQueue.getMessageId()));
                statusMessage.setOriginator(createOriginator(messageOnQueue.getPharmacyId()));
                statusMessage.setTarget(createTargetType(messageOnQueue.getPharmacyId()));
                statusMessage.setMessageInformation(createMessageInformation(response, ex));
                statusMessageTypes.add(statusMessage);

                try {
                    String xml = jaxContextCentralizer.toXml(StatusMessageType.class, statusMessage);
                    LOG.debug("***** STATUS MESSAGE : " + xml + " ****");
                } catch (GFDDPPException e) {
                    LOG.error("A problem occur while trying to marshal PharmaceuticalCareIds" + e.getMessage());
                }
            }
        }

        if (events != null && events.getArchivePrescriptionEventType() != null) {
            ArchivePrescriptionEventType apet = events.getArchivePrescriptionEventType();
            statusMessage = modelObjectFactory.createStatusMessageType();
            statusMessage.setSubjectReference(createSubjectReferenceType(apet.getDguid(), apet.getSguid(), messageOnQueue.getMessageId()));
            statusMessage.setOriginator(createOriginator(messageOnQueue.getPharmacyId()));
            statusMessage.setTarget(createTargetType(messageOnQueue.getPharmacyId()));
            statusMessage.setMessageInformation(createMessageInformation(response, ex));
            statusMessageTypes.add(statusMessage);
            try {
                String xml = jaxContextCentralizer.toXml(StatusMessageType.class, statusMessage);
                LOG.debug("***** STATUS MESSAGE : " + xml + " ****");
            } catch (GFDDPPException e) {
                LOG.error("A problem occur while trying to marshal PharmaceuticalCareIds" + e.getMessage());
            }
        }

        if (events != null && events.getArchivePrescriptionCommentEventType() != null) {
            ArchivePrescriptionCommentEventType apet = events.getArchivePrescriptionCommentEventType();
            statusMessage = modelObjectFactory.createStatusMessageType();
            statusMessage.setSubjectReference(createSubjectReferenceType(apet.getDguid(), apet.getSguid(), messageOnQueue.getMessageId()));
            statusMessage.setOriginator(createOriginator(messageOnQueue.getPharmacyId()));
            statusMessage.setTarget(createTargetType(messageOnQueue.getPharmacyId()));
            statusMessage.setMessageInformation(createMessageInformation(response, ex));
            statusMessageTypes.add(statusMessage);
            try {
                String xml = jaxContextCentralizer.toXml(StatusMessageType.class, statusMessage);
                LOG.debug("***** STATUS MESSAGE : " + xml + " ****");
            } catch (GFDDPPException e) {
                LOG.error("A problem occur while trying to marshal PharmaceuticalCareIds" + e.getMessage());
            }
        }
        return statusMessageTypes;
    }

    private MessageInformation createMessageInformation(SimpleResponseType response, IntegrationModuleException ex) {
        be.apb.standards.smoa.schema.model.v1.ObjectFactory modelObjectFactory = new be.apb.standards.smoa.schema.model.v1.ObjectFactory();
        MessageInformation messageInformation = modelObjectFactory.createMessageInformation();

        if (response != null) {
            LOG.debug("***** RESPONSE FROM TIP SYSTEM NOT NULL ****");
            if (response.getStatus() != null) {
                LOG.debug("***** STATUS IN RESPONSE NOT NULL ****");
                fillMessageInformationResponse(messageInformation, MessageType.INFO, response);
            } else {
                LOG.debug("***** STATUS IN RESPONSE NULL ****");
                fillMessageInformationLabel(messageInformation, MessageType.INFO, I18nHelper.getAllLanguagesLabels("statusmanagement.empty.status.response"));
            }
        } else if (ex != null) {
            LOG.debug("***** RESPONSE is NULL but got an exception: " + ex + "****");
            fillMessageInformationLabelLocal(messageInformation, MessageType.ERROR, ex.getLocalizedMessage());
        } else {
            LOG.debug("***** RESPONSE is NULL but got an exception: " + ex + "****");
            fillMessageInformationLabel(messageInformation, MessageType.ERROR, I18nHelper.getAllLanguagesLabels("statusmanagement.unknown.error"));

        }
        return messageInformation;
    }

    private void fillMessageInformationLabelLocal(MessageInformation messageInformation, MessageType messageType, String localizedMessage) {
        //		String[] splitted = localizedMessage.split(",");
        TextType textType = new TextType();
        textType.setLanguage(IntegrationModuleException.getUserLocale().toUpperCase());
        textType.setValue(localizedMessage);
        messageInformation.getMessageText().add(textType);
        fillMessageInformation(messageInformation, messageType, null);

    }

    private void fillMessageInformationLabel(MessageInformation messageInformation, MessageType messageType, Map<String, String> labels) {

        String nlLabel = labels.get("NL");
        String frLabel = labels.get("FR");
        String engLabel = labels.get("ENG");

        String[] splittedNL = nlLabel.split(",");
        String[] splittedFR = frLabel.split(",");
        String[] splittedENG = engLabel.split(",");

        TextType textTypeNL = new TextType();
        textTypeNL.setLanguage("NL");
        textTypeNL.setValue(splittedNL[1]);
        messageInformation.getMessageText().add(textTypeNL);

        TextType textTypeFR = new TextType();
        textTypeFR.setLanguage(Locale.FRENCH.toString().toUpperCase());
        textTypeFR.setValue(splittedFR[1]);
        messageInformation.getMessageText().add(textTypeFR);

        TextType textTypeENG = new TextType();
        textTypeENG.setLanguage(Locale.ENGLISH.toString().toUpperCase());
        textTypeENG.setValue(splittedENG[1]);
        messageInformation.getMessageText().add(textTypeENG);

        fillMessageInformation(messageInformation, messageType, splittedENG[0]);

    }

    private void fillMessageInformation(MessageInformation messageInformation, MessageType messageType, String messageSubType) {
        DatatypeFactory dataTypeFactory;
        try {
            dataTypeFactory = DatatypeFactory.newInstance();
        } catch (DatatypeConfigurationException e) {
            throw new IntegrationModuleRuntimeException(e.getCause());
        }
        GregorianCalendar gregorianCalendar = new GregorianCalendar();
        gregorianCalendar.setTimeInMillis(System.currentTimeMillis());
        messageInformation.setDateTime(dataTypeFactory.newXMLGregorianCalendar(gregorianCalendar));
        messageInformation.setMessageType(messageType);
        messageInformation.setMessageSubType(messageSubType);
    }

    private void fillMessageInformationResponse(MessageInformation messageInformation, MessageType messageType, SimpleResponseType response) {
        for (LocalisedString message : response.getStatus().getMessage()) {
            TextType textType = new TextType();
            textType.setLanguage(message.getLang().toString());
            textType.setValue(message.getValue());
            messageInformation.getMessageText().add(textType);
        }
        DatatypeFactory dataTypeFactory;
        try {
            dataTypeFactory = DatatypeFactory.newInstance();
        } catch (DatatypeConfigurationException e) {
            throw new IntegrationModuleRuntimeException(e.getCause());
        }
        GregorianCalendar gregorianCalendar = new GregorianCalendar();
        gregorianCalendar.setTimeInMillis(System.currentTimeMillis());
        messageInformation.setDateTime(dataTypeFactory.newXMLGregorianCalendar(gregorianCalendar));
        messageInformation.setMessageType(messageType);
        messageInformation.setMessageSubType(response.getStatus().getCode());

    }

    private TargetType createTargetType(String pharmacyId) {
        be.apb.standards.smoa.schema.model.v1.ObjectFactory modelObjectFactory = new be.apb.standards.smoa.schema.model.v1.ObjectFactory();
        TargetType target = modelObjectFactory.createTargetType();
        target.setTargetId(pharmacyId);
        target.setTargetType("NIHII");
        return target;
    }

    private OriginatorType createOriginator(String pharmacyId) {
        be.apb.standards.smoa.schema.model.v1.ObjectFactory modelObjectFactory = new be.apb.standards.smoa.schema.model.v1.ObjectFactory();
        OriginatorType originator = modelObjectFactory.createOriginatorType();
        originator.setServiceType(ServiceType.CM.toString());
        originator.setOriginatorType("NIHII");
        originator.setOriginatorId(pharmacyId);
        return originator;
    }

    private SubjectReferenceType createSubjectReferenceType(String dguid, String sguid, String mguid) {
        be.apb.standards.smoa.schema.model.v1.ObjectFactory modelObjectFactory = new be.apb.standards.smoa.schema.model.v1.ObjectFactory();
        SubjectReferenceType subjectReferenceType = modelObjectFactory.createSubjectReferenceType();
        subjectReferenceType.setDispensationGUID(dguid);
        subjectReferenceType.setSessionGUID(sguid);
        subjectReferenceType.setMessageGUID(mguid);
        return subjectReferenceType;
    }

    private void putStatusMessageOnQueue(List<StatusMessageType> statusMessages) {
        LOG.debug("***** ADDING NEW STATUS MESSAGE ON QUEUE****");
        for (StatusMessageType statusMessageType : statusMessages) {
            statusQueue.add(statusMessageType);
            LOG.debug("***** NEW STATUS MESSAGE ON QUEUE****");
        }

    }

    /*public ETKHelper getEtkHelper() {
		return etkHelper;
	}*/

    /*public void setEtkHelper(ETKHelper etkHelper) {
		this.etkHelper = etkHelper;
	}*/
    private SealedMessageRequestType extractSealedMessageRequest(EHealthMessage eHealthMessage) throws IntegrationModuleException {
        //TODO SealedMessageRequestTypeXML is it necessary wrapping layer ????????????????????
        LOG.info("extracting SealedMessagerRequestType from EHealthMessage");
        byte[] queueMessage = eHealthMessage.getMessage();
        SealedMessageRequestTypeXml sealXml = null;
        try {
            sealXml = jaxContextCentralizer.toObject(SealedMessageRequestTypeXml.class, queueMessage);
        } catch (GFDDPPException e) {
            throw new IntegrationModuleException("Couldn't unmarshal Object SealedMessageRequestTypeXml", e.getCause());
        }

        return sealXml.getSealedMessageRequestType();
    }

    private SingleMessage extractSingleMessage(SealedMessageRequestType sealedMessageRequestType) throws IntegrationModuleException {
        //jaxContextCentralizer.toObject(SingleMessage.class, sealedMessageRequestType.getSingleMessageSealed());
        LOG.info("extracting SingleMessage from SealedMessagerRequestType");
        byte[] sealedMsgReqTypBytes = sealedMessageRequestType.getSingleMessageSealed();
        SingleMessage singleMessage = null;
        try {
            singleMessage = jaxContextCentralizer.toObject(SingleMessage.class, sealedMsgReqTypBytes);
        } catch (GFDDPPException e) {
            throw new IntegrationModuleException("Couldn't unmarshal Object SingleMessage ", e.getCause());
        }

        return singleMessage;
    }

    protected void initFailedMsgFolders(PropertyHandler propertyHandler) throws IntegrationModuleException {
        if (propertyHandler.hasProperty(UNENCRYPTED_FOLDER_PROPERTY_NAME)) {
            unencrypted_folder_name = propertyHandler.getProperty(UNENCRYPTED_FOLDER_PROPERTY_NAME);
        } else {
            throw new RuntimeException("UNENCRYPTED_FOLDER not found in properties file !");
        }

        if (propertyHandler.hasProperty(INVALID_MESSAGES_FOLDER_PROPERTY_NAME)) {
            invalid_folder_name = propertyHandler.getProperty(INVALID_MESSAGES_FOLDER_PROPERTY_NAME);
        } else {
            throw new RuntimeException("INVALID FOLDER not found in properties file !");
        }
        File unencryptedMsgFolder = new File(unencrypted_folder_name);
        File invalidMsgFolder = new File(invalid_folder_name);

        // create folders if it doesn't exist
        if (false == unencryptedMsgFolder.exists()) {

            if (false == unencryptedMsgFolder.mkdirs()) {
                throw new RuntimeException("Not able to create folder " + unencrypted_folder_name);
            }
        }

        if (false == invalidMsgFolder.exists()) {

            if (false == invalidMsgFolder.mkdirs()) {
                throw new RuntimeException("Not able to create folder " + invalid_folder_name);
            }
        }
    }

    private EncryptionToken extractEncryptionToken(String tipid) throws IntegrationModuleException {
        EncryptionToken etk;
        if (propertyHandler.hasProperty("multitenancy.disabled")) {
            if ("false".equals(propertyHandler.getProperty("multitenancy.disabled"))) {
                etk = etkHelper.getTIP_ETK(tipid).get(0);
            } else {
                etk = etkHelper.getTIPSystem_ETK(propertyHandler.getProperty("tipsystem.id")).get(0);
            }
        } else {
            etk = etkHelper.getTIP_ETK(tipid).get(0);
        }
        return etk;
    }

    /**
     * We write SingleMessage xml to file with the same naming convention as corresponding eHealtMessage in the queue
     *
     * @param ehealthMsg
     */
    private void moveQueueMsgPlainText(EHealthMessage ehealthMsg) {

        SealedMessageRequestType sealedMessageRequestType;
        File dumpFile = null;
        try {
            sealedMessageRequestType = extractSealedMessageRequest(ehealthMsg);
            SingleMessage singleMsgObj = extractSingleMessage(sealedMessageRequestType);
            byte[] xmlSingleMsg = jaxContextCentralizer.toXml(SingleMessage.class, singleMsgObj).getBytes(Charsets.UTF_8);
            String currentFileName = queue.getFirstFileName();
            String dumpFileName = invalid_folder_name
                    + File.separator
                    + currentFileName.substring(0, currentFileName.lastIndexOf(MessageQueue.FILE_LOCK_SUFFIX))
                    + ".xml";
            dumpFile = new File(dumpFileName);
            FileUtils.writeByteArrayToFile(dumpFile, xmlSingleMsg);

        } catch (IntegrationModuleException | GFDDPPException | IOException e) {
            LOG.error(e);
        }
        // IOExceptions are taken care of in FileUtils - stream always closed
        

    }

    public void shouldStopProcessing() {
        this.shouldStopProcessing = true;
    }
}
