package be.business.connector.gfddpp.tipsystem.offline.queue;

import static java.util.concurrent.TimeUnit.SECONDS;

import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;

import org.apache.log4j.Logger;


import be.business.connector.core.utils.PropertyHandler;

public class QueueControle {
	private static Logger LOG = Logger.getLogger(QueueControle.class);

	private static final String MESSAGE_QUEUE_TIMER_KEY = "MESSAGE_QUEUE_TIMER";

	private final int period;
	private final ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(1);
	private ScheduledFuture<?> messageHandle;

	public QueueControle() {
		final PropertyHandler propertyHandler = PropertyHandler.getInstance();
		if (propertyHandler.hasProperty(MESSAGE_QUEUE_TIMER_KEY)) {
			period = Integer.decode(propertyHandler.getProperty(MESSAGE_QUEUE_TIMER_KEY));
			LOG.info("Interval for between runs of queuebehavior in seconds: " + period);
		} else {
			LOG.info("Property missinng:" + MESSAGE_QUEUE_TIMER_KEY);
			throw new RuntimeException(MESSAGE_QUEUE_TIMER_KEY + " not found in properties file !");
		}
	}

	public void processMessages(Runnable processBehavior) {
		try {
			messageHandle = scheduler.scheduleWithFixedDelay(processBehavior, period, period, SECONDS);
			LOG.info("remaining time(in seconds before next execution of the offline queue: " + messageHandle.getDelay(SECONDS));
		} catch (Exception e) {
			LOG.info("Something wrong happened in offline queue"+ e.getMessage());
					
		}
	}

	public ScheduledFuture<?> getMessageHandle() {
		return messageHandle;
	}

	public void shutdown() {
		LOG.info("Offline queue shutting down");
		scheduler.shutdown();
	}

	public int getPeriod() {
		return period;
	}
}
