package be.business.connector.gfddpp.tipsystem.threading;

import be.business.connector.core.exceptions.IntegrationModuleException;
import be.business.connector.core.technical.connector.utils.Crypto;
import be.ehealth.technicalconnector.service.kgss.domain.KeyResult;
import net.sf.ehcache.Cache;
import net.sf.ehcache.Element;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import java.util.concurrent.Semaphore;

/**
 * @author Liesje Demuynck.
 */
public class GeneralDecryptorThread extends Thread {

    private final static Logger LOG = Logger.getLogger(GeneralDecryptorThread.class);

    private byte[] encoded;
    private byte[] decoded;
    private String encryptionKeyId;
    private Semaphore available;
    private Cache kgssCache;
    private IntegrationModuleException error;


    public GeneralDecryptorThread(Semaphore available, byte[] encoded, String encryptionKeyId, Cache kgssCache) {
        this.available = available;
        this.encoded = encoded;
        this.encryptionKeyId = encryptionKeyId;
        this.kgssCache = kgssCache;
        this.decoded = null;
        this.error = null;
    }

    @Override
    public void run() {
        if (encoded.length > 0 && StringUtils.isNotBlank(encryptionKeyId)) {
            try {
                KeyResult key = getKey(encryptionKeyId);
                decoded = Crypto.unsealForUnknown(key, encoded);
            } catch (IntegrationModuleException e) {
                this.error = e;
            }
        }

        available.release();
    }

    private KeyResult getKey(String encryptionKeyId) {
        LOG.info("GET KEY FROM CACHE FOR ID: " + encryptionKeyId);
        Element tmpKey = kgssCache.get(encryptionKeyId);
        if (tmpKey != null) {
            return (KeyResult) tmpKey.getObjectValue();
        }
        return null;
    }

    public byte[] getDecoded() throws IntegrationModuleException{
        if(error != null){
            throw error;
        }
        return decoded;
    }
}
