package be.business.connector.gfddpp.tipsystem.threading;

import java.util.concurrent.Semaphore;

import net.sf.ehcache.Cache;
import net.sf.ehcache.Element;

import org.apache.log4j.Logger;

import be.business.connector.core.exceptions.IntegrationModuleException;
import be.business.connector.gfddpp.pcdh.PCDHIntegrationModuleImpl;
import be.ehealth.technicalconnector.service.kgss.domain.KeyResult;

public class KGSSThread extends Thread {

    private final static Logger LOG = Logger.getLogger(KGSSThread.class);

    private final byte[] etkKey;
    private final PCDHIntegrationModuleImpl pcdhModule;
    private final Semaphore available;
    private final Cache kgssCache;
    private final String encryptionId;
    private boolean errorOccured = false;

    public KGSSThread(Semaphore available, Cache kgssCache, String encryptionId, byte[] etkKey, PCDHIntegrationModuleImpl pcdhIntegrationModuleImpl) {
        this.available = available;
        this.kgssCache = kgssCache;
        this.encryptionId = encryptionId;
        this.etkKey = etkKey;
        this.pcdhModule = pcdhIntegrationModuleImpl;
    }

    @Override
    public void run() {

        long start = System.currentTimeMillis();
        LOG.info("GET KEY START");
        try {
            KeyResult key = pcdhModule.getKeyFromKgss(encryptionId, etkKey);
            setErrorOccured(true);
            LOG.info("PUT KGSS ID IN ELEMENT: " + encryptionId);
            Element element = new Element(encryptionId, key);
            LOG.info("PUT KGSS ELEMENT IN CACHE: " + encryptionId);
            kgssCache.put(element);
        } catch (IntegrationModuleException e) {
            LOG.error("Exception - Call KGSS - encryptionId: " + encryptionId);
            LOG.error("Exception occured in KGSSTHREAD", e);
        } catch (Throwable e) {
            LOG.error("Exception occured in KGSSTHREAD:", e);
        }
        long stop = System.currentTimeMillis();
        LOG.info("GET KEY END: " + (stop - start));

        available.release();
    }

    public boolean isErrorOccured() {
        return this.errorOccured;
    }

    public void setErrorOccured(boolean errorOccured) {
        this.errorOccured = errorOccured;
    }
}
