package be.business.connector.gfddpp.utils;

import java.io.UnsupportedEncodingException;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

import be.apb.standards.smoa.schema.v1.*;
import org.apache.commons.io.Charsets;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import be.apb.gfddpp.common.exceptions.GFDDPPException;
import be.apb.gfddpp.common.status.StatusCode;
import be.apb.gfddpp.common.status.StatusResolver;
import be.apb.gfddpp.common.utils.JaxContextCentralizer;
import be.apb.gfddpp.common.utils.SingleMessageWrapper;
import be.apb.gfddpp.common.utils.Utils;
import be.apb.gfddpp.domain.PersonIdType;
import be.apb.gfddpp.rtrn.registerdata.BvacDocument;
import be.apb.gfddpp.rtrn.registerdata.Dispensation;
import be.apb.gfddpp.rtrn.registerdata.Events;
import be.apb.gfddpp.validation.exception.SingleMessageValidationException;
import be.apb.standards.smoa.schema.id.v1.AbstractMedicinalProductIdType;
import be.apb.standards.smoa.schema.id.v1.AtcDddSystemIdType;
import be.apb.standards.smoa.schema.id.v1.BeRegistrationIdType;
import be.apb.standards.smoa.schema.id.v1.CnkIdType;
import be.apb.standards.smoa.schema.id.v1.EntityIdType;
import be.apb.standards.smoa.schema.id.v1.NihiiIdType;
import be.apb.standards.smoa.schema.model.v1.DataLocationType;
import be.apb.standards.smoa.schema.model.v1.MaxSetPatient;
import be.apb.standards.smoa.schema.model.v1.MaxSetPersonType;
import be.apb.standards.smoa.schema.model.v1.MaxSetProductType;
import be.apb.standards.smoa.schema.model.v1.MedicationHistoryType;
import be.apb.standards.smoa.schema.model.v1.ReferencePharmacyType;
import be.apb.standards.smoa.schema.model.v1.MinSetProductType.Description;
import be.apb.standards.smoa.schema.v1.PharmaceuticalCareEventType.DispensedForSamePrescription;
import be.apb.standards.smoa.schema.v1.PharmaceuticalCareEventType.DispensedWithoutPrescription;
import be.apb.standards.smoa.schema.v1.PharmaceuticalCareEventType.DispensedForSamePrescription.Product;
import be.apb.standards.smoa.schema.v1.ContinuedPharmaceuticalCareDossierEvent;
import be.business.connector.core.exceptions.IntegrationModuleException;
import be.business.connector.core.utils.EncryptionUtils;
import be.business.connector.core.utils.Exceptionutils;
import be.business.connector.core.utils.I18nHelper;
import be.business.connector.core.utils.PropertyHandler;
import be.business.connector.gfddpp.domain.ThreeStateBoolean;
import be.business.connector.projects.common.utils.ProductFilterUtils;
import be.business.connector.projects.common.utils.SystemServicesUtils;
import be.ehealth.apb.gfddpp.services.tipsystem.RoutingParametersType;
import be.ehealth.apb.gfddpp.services.tipsystem.SealedMessageRequestType;

public class SingleMessageInfo {

    private final static Logger LOG = Logger.getLogger(SingleMessageInfo.class);
    private final static String module = "SingleMessageInfo";

    private JaxContextCentralizer jaxContextCentralizer;
    private final static SingleMessageValidationXsd smvh = new SingleMessageValidationXsd();

    private boolean HasArchivePrescriptionEventType = false;
    private boolean HasArchivePrescriptionCommentEventType = false;
    private boolean HasPharmaceuticalCareEventType = false;
    private boolean HasBvacEventType = false;
    private boolean HasSyncEventType = false;
    private boolean HasMedicationHistoryEvent = false;
    private boolean HasMedicationSchemeEntriesRequest = false;
    private boolean HasContinuedPharmaceuticalCareDossierEvent = false;
    
    private String info = " ";
    private SingleMessageWrapper singleMessageWrapper = null;
    private SingleMessage singleMessageObject = null;

    private String eventtype = "";

    private JaxContextCentralizer getJaxContextCentralizer() {
        if (this.jaxContextCentralizer == null) {
            jaxContextCentralizer = JaxContextCentralizer.getInstance();
            //jaxContextCentralizer.addContext(GetKeyRequestContent.class);
            //jaxContextCentralizer.addContext(GetKeyResponseContent.class);
        }
        return this.jaxContextCentralizer;
    }

    public SingleMessageInfo() {
    }

    public SingleMessageInfo(byte[] singleMessage) throws IntegrationModuleException {
        String procedure = "SingleMessageInfo";
        if (singleMessage == null) {
            LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error.validation.dataContent singleMessage == null");
            throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.dataContent"));
        }

        try {
            SingleMessage lsingleMessageObject = getJaxContextCentralizer().toObject(SingleMessage.class, new String(singleMessage, Charsets.UTF_8));
            initSingleMessageInfo(lsingleMessageObject);
        } catch (GFDDPPException e) {
            LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + e.getMessage());
            throw new IntegrationModuleException(e.getLocalizedMessage(), e);
        }
    }


    public final void initSingleMessageInfo(SingleMessage singleMessageObject) throws IntegrationModuleException {
        String procedure = "InitSingleMessageInfo";
        setSingleMessageObject(singleMessageObject);
        List<AbstractEventType> types = singleMessageWrapper.getEvents();
        long tel = 0;
        for (AbstractEventType type : types) {
            tel++;
            if (type instanceof ArchivePrescriptionEventType) {
                HasArchivePrescriptionEventType = true;
                info += " EventType:ArchivePrescriptionEventType ";
                HasSyncEventType = false;
                eventtype = "ArchivePrescriptionEventType";
            } else if (type instanceof ArchivePrescriptionCommentEventType) {
                HasArchivePrescriptionCommentEventType = true;
                info += " EventType:ArchivePrescriptionCommentEventType ";
                HasSyncEventType = false;
                eventtype = "ArchivePrescriptionCommentEventType";
            } else if (type instanceof PharmaceuticalCareEventType) {
                HasPharmaceuticalCareEventType = true;
                info += " EventType:PharmaceuticalCareEventType ";
                eventtype = "PharmaceuticalCareEventType";
            } else if (type instanceof BvacEventType) {
                HasBvacEventType = true;
                info += " EventType:BvacEventType ";
                HasSyncEventType = true;
                eventtype = "Bvac";
            } else if (type instanceof MedicationHistoryEvent) {
                HasMedicationHistoryEvent = true;
                info += " EventType:MedicationHistoryEvent ";
                eventtype = "MedicationHistoryEvent";
            } else if (type instanceof MedicationSchemeEntriesRequest) {
                HasMedicationSchemeEntriesRequest = true;
                info += " EventType:MedicationSchemeEntriesRequest ";
                eventtype = "MedicationSchemeEntriesRequest";
                HasSyncEventType = true;
            } else if (type instanceof ContinuedPharmaceuticalCareDossierEvent) {
                HasContinuedPharmaceuticalCareDossierEvent = true;
                info += " EventType:ContinuedPharmaceuticalCareDossierEvent ";
                eventtype = "ContinuedPharmaceuticalCareDossierEvent";
                HasSyncEventType = true;
            } else {
                LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error unknown EventType: " + type.getClass().toString());
                throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.event.type"));
            }
        }

        if (tel > 1) {
            LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "more than 1 Event ");
            throw new IntegrationModuleException(I18nHelper.getLabel("EventType.Multiple"));
        }

        LOG.debug(module + "=-----= ----- " + procedure + " =$=" + info);

    }

    private void setSingleMessageObject(SingleMessage singleMessageObject) {
        this.singleMessageObject = singleMessageObject;
        singleMessageWrapper = new SingleMessageWrapper(singleMessageObject);
    }

    public void validateRegisterData() throws IntegrationModuleException, GFDDPPException {
        String procedure = "validateRegisterData";

        if (HasMedicationHistoryEvent) {
            LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: error.validation.event.type");
            throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.event.type"));
        }

        validateKnownEventType();
        validateMultipleEventTypes();
        validateEventTypesForRegisterData();

        validateEmptyHeader();
        validateSGuidEmptyOk();
        validateDGuidEmptyOk();

        validateSessionDateOk();
        validateSingleMessagePatientMaxDataSet();
        validateBvacEventType();
    }

    public void validateRevokeData() throws IntegrationModuleException {
        String procedure = "validateRevokeData";

        if (HasMedicationHistoryEvent) {
            LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: error.validation.event.type");
            throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.event.type"));
        }

        validateKnownEventType();
        validateMultipleEventTypes();
        validateEventTypesForRevokeData();

        validateEmptyHeader();
        validateSGuidFilledOk();
        validateDGuidFilledOk();

        //validateSessionDateOk();
        validateMotivationOk();
        validateBvacEventType();
    }

    public void validateUpdateData() throws IntegrationModuleException {
        String procedure = "validateUpdateData";

        if (HasMedicationHistoryEvent) {
            LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: error.validation.event.type");
            throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.event.type"));
        }

        validateKnownEventType();
        validateMultipleEventTypes();
        validateEventTypesForUpdateData();

        validateEmptyHeader();
        validateSGuidFilledOk();
        validateDGuidFilledOk();

        validateSessionDateOk();
        validateMotivationOk();

        validateBvacEventType();
    }

    public void validateKnownEventType() throws IntegrationModuleException {
        String procedure = "validateKnownEventType";
        if (!HasBvacEventType && !HasPharmaceuticalCareEventType && !HasArchivePrescriptionEventType && !HasArchivePrescriptionCommentEventType && !HasMedicationSchemeEntriesRequest && !HasContinuedPharmaceuticalCareDossierEvent) {
            LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: error.validation.event.type");
            throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.event.type"));
        }
    }

    public void validateEventTypesForRegisterData() throws IntegrationModuleException {
        String procedure = "validateEventTypesForRegisterData";
        if (!HasPharmaceuticalCareEventType && !HasBvacEventType && !HasArchivePrescriptionEventType && !HasArchivePrescriptionCommentEventType && !HasMedicationSchemeEntriesRequest && !HasContinuedPharmaceuticalCareDossierEvent) {
            LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: error.validation.event.type");
            throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.event.type.not.supported"));
        }
    }

    public void validateEventTypesForUpdateData() throws IntegrationModuleException {
        String procedure = "validateEventTypesForUpdateData";
        if (HasBvacEventType || HasArchivePrescriptionEventType || HasArchivePrescriptionCommentEventType || HasMedicationSchemeEntriesRequest || HasContinuedPharmaceuticalCareDossierEvent) {
            LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: error.validation.event.type");
            throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.event.type.not.supported"));
        }
        if (!HasPharmaceuticalCareEventType) {
            LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: error.validation.event.type");
            throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.event.type.not.supported"));
        }
    }

    public void validateEventTypesForRevokeData() throws IntegrationModuleException {
        String procedure = "validateEventTypesForRevokeData";
        if (HasBvacEventType || HasArchivePrescriptionEventType || HasArchivePrescriptionCommentEventType || HasMedicationSchemeEntriesRequest || HasContinuedPharmaceuticalCareDossierEvent) {
            LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: error.validation.event.type");
            throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.event.type.not.supported"));
        }
        if (!HasPharmaceuticalCareEventType) {
            LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: error.validation.event.type");
            throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.event.type.not.supported"));
        }
    }

    public void validateMultipleEventTypes() throws IntegrationModuleException {
        String procedure = "validateMultipleEventTypes";
        if (HasBvacEventType && HasPharmaceuticalCareEventType) {
            LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: bvacEventType.together.with.pharmaceuticalCareEventType");
            throw new IntegrationModuleException(I18nHelper.getLabel("bvacEventType.together.with.pharmaceuticalCareEventType"));
        }
    }

    public void validateSingleMessageXsd() throws IntegrationModuleException, GFDDPPException {
        String procedure = "validateSingleMessageXsd";
        byte[] singleMessage = getXmlBytes();
        try {
            synchronized (smvh) {
                smvh.assertValidSingleMessage(singleMessage);
            }
        } catch (SingleMessageValidationException e) {
            LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: error.validation.event.type");
            //TODO jl exception error.validation.event.type
            Exceptionutils.errorHandler(e);
        }
    }

    public void validateBvacEventType() throws IntegrationModuleException {
        String procedure = "ValidateBvacEventType";
        if (HasBvacEventType) {
            for (BvacEventType bvacEventType : singleMessageWrapper.getAllEventsOfType(BvacEventType.class)) {
                String cbfa = null;
                if (bvacEventType.getPharmacyList() != null
                        && bvacEventType.getPharmacyList().getPharmacy().size() > 0
                        && bvacEventType.getPharmacyList().getPharmacy().get(0).getBvacList() != null
                        && bvacEventType.getPharmacyList().getPharmacy().get(0).getBvacList().getBvacDocument().size() > 0
                        && bvacEventType.getPharmacyList().getPharmacy().get(0).getBvacList().getBvacDocument().get(0).getIdentification() != null
                        && bvacEventType.getPharmacyList().getPharmacy().get(0).getBvacList().getBvacDocument().get(0).getIdentification().getBarCode() != null) {
                    cbfa = bvacEventType.getPharmacyList().getPharmacy().get(0).getBvacList().getBvacDocument().get(0).getIdentification().getBarCode().getInsurer();
                }
                if (!isValidCbfa(cbfa)) {
                    LOG.error(module + "=-----= error " + procedure + " =$=" + " => " + "error: bvacEventType.validate.cbfa");
                    throw new IntegrationModuleException(I18nHelper.getLabel("bvacEventType.validate.cbfa"));
                }
            }
        }
    }

    private boolean isValidCbfa(String cbfa) {
        List<String> cbfas = SystemServicesUtils.getInstance().getAllCbfas();
        if (cbfas.contains(cbfa)) {
            return true;
        } else {
            return false;
        }
    }

    public void validateSingleMessagePatientMaxDataSet() throws GFDDPPException, IntegrationModuleException {
        String procedure = "validateSingleMessagePatientMaxDataSet";

        String patientId = null;
        if (HasPharmaceuticalCareEventType) {
            for (Iterator<PharmaceuticalCareEventType> iterator = singleMessageWrapper.getAllEventsOfType(PharmaceuticalCareEventType.class).iterator(); iterator.hasNext(); ) {
                PharmaceuticalCareEventType pharmaceuticalCareEventType = iterator.next();
                if (pharmaceuticalCareEventType.getMaxPatient() != null) {
                    MaxSetPersonType maxSetPersonType = pharmaceuticalCareEventType.getMaxPatient().getIdentification();
                    PersonIdType personIdType = null;
                    if (maxSetPersonType != null) {
                        personIdType = PersonIdType.valueOf(maxSetPersonType.getPersonId());
                    } else {
                        LOG.error(module + "=-----= error " + procedure + " =$=" + " => " + "error: COMMON_ERROR_PATIENT_MISSING error.validation.patient.id.not.filled.in");
                        throw new GFDDPPException(StatusCode.COMMON_ERROR_PATIENT_MISSING, new String[]{PharmaceuticalCareEventType.class.getName()});
                        //I18nHelper.getLabel("error.validation.patient.id.not.filled.in"));
                    }
                    String pId = personIdType.getIdFrom(maxSetPersonType.getPersonId());

                    if (StringUtils.isEmpty(pId)) {
                        LOG.error(module + "=-----= error " + procedure + " =$=" + " => " + "error: COMMON_ERROR_PATIENT_MISSING error.validation.patient.id.not.filled.in");
                        throw new GFDDPPException(StatusCode.COMMON_ERROR_PATIENT_MISSING);
                        //I18nHelper.getLabel("error.validation.patient.id.not.filled.in"));
                    } else if (patientId == null) {
                        patientId = pId;
                    } else if (!pId.equals(patientId)) {

                        GFDDPPException e = new GFDDPPException(StatusCode.COMMON_ERROR_MULTIPLE_PATIENT, new String[]{patientId, pId});
                        LOG.error(module + "=-----= error " + procedure + " =$=" + " => " + "error: COMMON_ERROR_MULTIPLE_PATIENT");
                        throw e;
                    }
                } else {
                    LOG.error(module + "=-----= error " + procedure + " =$=" + " => " + "error: COMMON_ERROR_PATIENT_MISSING error.validation.patient.id.not.filled.in");
                    throw new GFDDPPException(StatusCode.COMMON_ERROR_PATIENT_MISSING);
                }
            }
        }
    }

    public String validateNihiiPharmacyId(String pharmacyIdOriginal) throws IntegrationModuleException {
        String procedure = "validateNihiiPharmacyId";
        String compareNihiiPharmacy = null;

        if (HasPharmaceuticalCareEventType) {
            for (PharmaceuticalCareEventType pharmaceuticalCareEventType : singleMessageWrapper.getAllEventsOfType(PharmaceuticalCareEventType.class)) {
                if (pharmaceuticalCareEventType.getPharmacyId() != null && !StringUtils.isEmpty(pharmaceuticalCareEventType.getPharmacyId().getNihiiPharmacyNumber())) {
                    compareNihiiPharmacy = pharmaceuticalCareEventType.getPharmacyId().getNihiiPharmacyNumber();
                    if (!pharmacyIdOriginal.equals(compareNihiiPharmacy)) {
                        LOG.error(module + "=-----= error " + procedure + " =$=" + " => " + "error: validate.nihii.pharmaceuticalCareEventType.not.the.same");
                        throw new IntegrationModuleException(I18nHelper.getLabel("validate.nihii.pharmaceuticalCareEventType.not.the.same"));
                    }
                } else {
                    LOG.error(module + "=-----= error " + procedure + " =$=" + " => " + "error: pharmaceuticalCareEventType.pharmacy.nihii.not.found");
                    throw new IntegrationModuleException(I18nHelper.getLabel("pharmaceuticalCareEventType.pharmacy.nihii.not.found"));
                }

            }
        }

        if (HasBvacEventType) {
            String nihiiPharmacy = null;
            for (BvacEventType bvacEventType : singleMessageWrapper.getAllEventsOfType(BvacEventType.class)) {
                if (bvacEventType.getPharmacyList() != null && bvacEventType.getPharmacyList().getPharmacy().size() > 0 && bvacEventType.getPharmacyList().getPharmacy().get(0).getIdentification() != null && StringUtils.isNotBlank(bvacEventType.getPharmacyList().getPharmacy().get(0).getIdentification().getId())) {
                    nihiiPharmacy = bvacEventType.getPharmacyList().getPharmacy().get(0).getIdentification().getId();
                    if (StringUtils.isEmpty(compareNihiiPharmacy)) {
                        compareNihiiPharmacy = nihiiPharmacy;
                    } else if (!compareNihiiPharmacy.equals(nihiiPharmacy)) {
                        LOG.error(module + "=-----= error " + procedure + " =$=" + " => " + "error: validate.nihii.bvacEventType.not.the.same");
                        throw new IntegrationModuleException(I18nHelper.getLabel("validate.nihii.bvacEventType.not.the.same"));
                    }
                } else {
                    LOG.error(module + "=-----= error " + procedure + " =$=" + " => " + "error: bvacEventType.pharmacy.nihii.not.found");
                    throw new IntegrationModuleException(I18nHelper.getLabel("bvacEventType.pharmacy.nihii.not.found"));
                }
            }
        }

        if (HasArchivePrescriptionEventType) {
            for (ArchivePrescriptionEventType archivePrescriptionEventType : singleMessageWrapper.getAllEventsOfType(ArchivePrescriptionEventType.class)) {
                if (archivePrescriptionEventType.getExecutorId() != null && !StringUtils.isEmpty(archivePrescriptionEventType.getExecutorId().getValue())) {
                    compareNihiiPharmacy = archivePrescriptionEventType.getExecutorId().getValue();
                    if (!pharmacyIdOriginal.equals(compareNihiiPharmacy)) {
                        LOG.error(module + "=-----= error " + procedure + " =$=" + " => " + "error: validate.nihii.archivePrescriptionEventType.not.the.same");
                        throw new IntegrationModuleException(I18nHelper.getLabel("validate.nihii.archivePrescriptionEventType.not.the.same"));
                    }
                } else {
                    LOG.error(module + "=-----= error " + procedure + " =$=" + " => " + "error: archivePrescriptionEventType.pharmacy.nihii.not.found");
                    throw new IntegrationModuleException(I18nHelper.getLabel("archivePrescriptionEventType.pharmacy.nihii.not.found"));
                }

            }
        }

        if (HasArchivePrescriptionCommentEventType) {
            for (ArchivePrescriptionCommentEventType apcet : singleMessageWrapper.getAllEventsOfType(ArchivePrescriptionCommentEventType.class)) {
                if (apcet.getExecutorId() != null && !StringUtils.isEmpty(apcet.getExecutorId().getValue())) {
                    compareNihiiPharmacy = apcet.getExecutorId().getValue();
                    if (!pharmacyIdOriginal.equals(compareNihiiPharmacy)) {
                        LOG.error(module + "=-----= error " + procedure + " =$=" + " => " + "error: validate.nihii.archivePrescriptionCommentEventType.not.the.same");
                        throw new IntegrationModuleException(I18nHelper.getLabel("validate.nihii.archivePrescriptionCommentEventType.not.the.same"));
                    }
                } else {
                    LOG.error(module + "=-----= error " + procedure + " =$=" + " => " + "error: archivePrescriptionCommentEventType.pharmacy.nihii.not.found");
                    throw new IntegrationModuleException(I18nHelper.getLabel("archivePrescriptionCommentEventType.pharmacy.nihii.not.found"));
                }

            }
        }
        
        if (HasContinuedPharmaceuticalCareDossierEvent) {
            for (ContinuedPharmaceuticalCareDossierEvent apcet : singleMessageWrapper.getAllEventsOfType(ContinuedPharmaceuticalCareDossierEvent.class)) {
                if (apcet.getPharmacyId() != null && !StringUtils.isEmpty(apcet.getPharmacyId().getNihiiPharmacyNumber())) {
                    compareNihiiPharmacy = apcet.getPharmacyId().getNihiiPharmacyNumber();
                    if (!pharmacyIdOriginal.equals(compareNihiiPharmacy)) {
                        LOG.error(module + "=-----= error " + procedure + " =$=" + " => " + "error: validate.nihii.ContinuedPharmaceuticalCareDossierEvent.not.the.same");
                        throw new IntegrationModuleException(I18nHelper.getLabel("validate.nihii.ContinuedPharmaceuticalCareDossierEvent.not.the.same"));
                    }
                } else {
                    LOG.error(module + "=-----= error " + procedure + " =$=" + " => " + "error: ContinuedPharmaceuticalCareDossierEvent.pharmacy.nihii.not.found");
                    throw new IntegrationModuleException(I18nHelper.getLabel("ContinuedPharmaceuticalCareDossierEvent.pharmacy.nihii.not.found"));
                }

            }
        }
        if (HasMedicationSchemeEntriesRequest || HasArchivePrescriptionEventType || HasArchivePrescriptionCommentEventType) {
            compareNihiiPharmacy = pharmacyIdOriginal; // pharmacy Id not mentioned in request
        }

        //if (isInSystemKeyStore(compareNihiiPharmacy)) {
        // throw new
        // IntegrationModuleException(I18nHelper.getLabel("validate.nihii.keystore.not.matching",
        // new Object[] { compareNihiiPharmacy }));
        //}
        if (compareNihiiPharmacy == null) {
            LOG.error(module + "=-----= error " + procedure + " =$=" + " => " + "error: pharmacy.nihii.not.found");
            throw new IntegrationModuleException(I18nHelper.getLabel("pharmacy.nihii.not.found"));
            //TODO jl pharmacy.nihii.not.found
        }

        return compareNihiiPharmacy;
    }

    public void validateMedicationHistoryPatient(String patientid) throws IntegrationModuleException {
        String procedure = "validateMedicationHistoryPatient";
        SingleMessageWrapper wrapper = singleMessageWrapper;

        for (Iterator<MedicationHistoryType> iterator = wrapper.getAllMedicationHistoryEntries().iterator(); iterator.hasNext(); ) {
            MedicationHistoryType medicationHistoryType = iterator.next();
            if (medicationHistoryType.getMinPatient() != null) {
                PersonIdType personIdType = PersonIdType.valueOf(medicationHistoryType.getMinPatient().getPersonId());
                String pId = personIdType.getIdFrom(medicationHistoryType.getMinPatient().getPersonId());
                if (StringUtils.isEmpty(pId)) {
                    LOG.error(module + "=-----= error " + procedure + " =$=" + " => " + "error: error.validation.patient.id.not.filled.in");
                    throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.patient.id.not.filled.in"));
                } else if (!pId.equals(patientid)) {
                    LOG.error(module + "=-----= error " + procedure + " =$=" + " => " + "error: error.validation.patient.id.not.equal.to.parameters");
                    throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.patient.id.not.equal.to.parameters") + " " + pId + " != " + patientid);
                }
            } else {
                LOG.error(module + "=-----= error " + procedure + " =$=" + " => " + "error: error.validation.patient.not.filled.in");
                throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.patient.not.filled.in"));
            }
        }

    }

    public void validateGetDataTypesResult() throws IntegrationModuleException {
        String procedure = "validateGetDataTypesResult";
        SingleMessageWrapper wrapper = singleMessageWrapper;

        for (Iterator<DataLocationType> iterator = wrapper.getAllEntitiesOfType(DataLocationType.class).iterator(); iterator.hasNext(); ) {
            DataLocationType dlt = iterator.next();
            if (dlt.getLocation() == null) {
                LOG.error(module + "=-----= error " + procedure + " =$=" + " => " + "error: error.validation.empty.datatype.response");
                throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.empty.datatype.response"));
            }
            if (dlt.getLocation().isEmpty()) {
                LOG.error(module + "=-----= error " + procedure + " =$=" + " => " + "error: error.validation.empty.datatype.response");
                throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.empty.datatype.response"));
            }
        }
    }

    public void validateEmptyHeader() throws IntegrationModuleException {
        String procedure = "validateEmptyHeader";
        SingleMessageWrapper wrapper = singleMessageWrapper;

        if (wrapper.getHeader() != null && wrapper.getHeader().getMessageCreateDate() != null) {
            LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error.validation.header.message.create.date.filled.in");
            throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.header.message.create.date.filled.in"));
        }
        if (wrapper.getHeader() != null && !StringUtils.isEmpty(wrapper.getHeader().getMessageID())) {
            LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error.validation.header.message.id.filled.in");
            throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.header.message.id.filled.in"));
        }
        if (wrapper.getHeader() != null && wrapper.getHeader().getSender() != null) {
            LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error.validation.header.sender.filled.in");
            throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.header.sender.filled.in"));
        }
        if (wrapper.getHeader() != null && !StringUtils.isEmpty(wrapper.getHeader().getVersion())) {
            LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error.validation.header.version.filled.in");
            throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.header.version.filled.in"));
        }
    }

    public void validateSingleMessageHeaderFilled(String NihiiPharmacyId) throws IntegrationModuleException {
        String procedure = "validateSingleMessageHeaderFilled";
        SingleMessageWrapper wrapper = singleMessageWrapper;
        HeaderType ht = wrapper.getHeader();
        if (ht == null) {
            //No problem
        } else {
            if (ht.getMessageCreateDate() == null) {
                LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error.validation.header.message.create.date.filled.in");
                throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.header.message.create.date.filled.in"));
            }
            if (StringUtils.isEmpty(ht.getMessageID())) {
                LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error.validation.header.message.id.filled.in");
                throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.header.message.id.filled.in"));

            }
            if (ht.getSender() == null) {
                LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error.validation.header.sender.filled.in");
                throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.header.sender.filled.in"));
            } else if (!this.getHeaderPharmacyId().equals(NihiiPharmacyId)) {
                LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error.validation.header.sender.filled.in" + " " + NihiiPharmacyId);
                throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.header.sender.filled.in"));
            }
            if (StringUtils.isEmpty(ht.getVersion())) {
                LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error.validation.header.version.filled.in");
                throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.header.version.filled.in"));
            }
        }
    }

    public void validateSGuidEmptyOk() throws IntegrationModuleException {
        String procedure = "validateSGuidEmptyOk";

        SingleMessageWrapper wrapper = singleMessageWrapper;
        List<PharmaceuticalCareEventType> pharmaceuticalCareEventTypes = wrapper.getAllEventsOfType(PharmaceuticalCareEventType.class);

        for (Iterator<PharmaceuticalCareEventType> iterator = pharmaceuticalCareEventTypes.iterator(); iterator.hasNext(); ) {
            PharmaceuticalCareEventType pharmaceuticalCareEventType = iterator.next();
            if (!StringUtils.isEmpty(pharmaceuticalCareEventType.getId())) {
                LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: error.validation.sguid.filled.in");
                throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.sguid.filled.in"));
            }
        }
        for (MedicationSchemeEntriesRequest medicationSchemeEntriesRequest : wrapper.getAllMedicationSchemeEntryRequests()) {
            if (StringUtils.isNotBlank(medicationSchemeEntriesRequest.getId())) {
                LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: error.validation.sguid.filled.in");
                throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.sguid.filled.in"));
            }
        }
    }

    public void validateSGuidFilledOk() throws IntegrationModuleException {
        String procedure = "validateSGuidFilledOK";
        SingleMessageWrapper wrapper = singleMessageWrapper;

        List<PharmaceuticalCareEventType> pharmaceuticalCareEventTypes = wrapper.getAllEventsOfType(PharmaceuticalCareEventType.class);
        for (Iterator<PharmaceuticalCareEventType> iterator = pharmaceuticalCareEventTypes.iterator(); iterator.hasNext(); ) {
            PharmaceuticalCareEventType pharmaceuticalCareEventType = iterator.next();
            if (StringUtils.isEmpty(pharmaceuticalCareEventType.getId())) {
                LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error.validation.sguid.not.filled.in");
                throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.sguid.not.filled.in"));
            }
        }

        for (Iterator<MedicationHistoryType> iterator = wrapper.getAllMedicationHistoryEntries().iterator(); iterator.hasNext(); ) {
            MedicationHistoryType medicationHistoryType = iterator.next();
            if (StringUtils.isEmpty(medicationHistoryType.getSessionID())) {
                LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: error.validation.sguid.not.filled.in");
                throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.sguid.not.filled.in"));
            }
        }

        for (MedicationSchemeEntriesRequest medicationSchemeEntryRequest : wrapper.getAllMedicationSchemeEntryRequests()) {
            if (StringUtils.isBlank(medicationSchemeEntryRequest.getId())) {
                LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: error.validation.sguid.not.filled.in");
                throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.sguid.not.filled.in"));
            }
        }
    }

    public void validateDGuidEmptyOk() throws IntegrationModuleException {
        String procedure = "validateDGuidEmptyOk";
        SingleMessageWrapper wrapper = singleMessageWrapper;
        for (Iterator<MaxSetProductType> iterator = wrapper.getAllDispensedProducts().iterator(); iterator.hasNext(); ) {
            MaxSetProductType product = iterator.next();
            if (!StringUtils.isEmpty(product.getDispensationGUID())) {
                LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: error.validation.dguid.filled.in");
                throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.dguid.filled.in"));
            }
        }

        for (MedicationSchemeEntriesRequest medicationSchemeEntryRequest : wrapper.getAllMedicationSchemeEntryRequests()) {
            for (DataEntryRequest dataEntryRequest : medicationSchemeEntryRequest.getDataEntry()) {
                if (StringUtils.isNotBlank(dataEntryRequest.getDguid())) {
                    LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: error.validation.dguid.filled.in");
                    throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.dguid.filled.in"));
                }
            }
        }

    }

    public void validateDGuidFilledOk() throws IntegrationModuleException {
        String procedure = "validateDGuidFilledOk";
        SingleMessageWrapper wrapper = singleMessageWrapper;
        boolean validationFailure = false;

        for (Iterator<MaxSetProductType> iterator = wrapper.getAllDispensedProducts().iterator(); iterator.hasNext(); ) {
            MaxSetProductType product = iterator.next();
            if (StringUtils.isEmpty(product.getDispensationGUID())) {
                validationFailure = true;
            }
        }

        for (Iterator<MedicationHistoryType> iterator = wrapper.getAllMedicationHistoryEntries().iterator(); iterator.hasNext(); ) {
            MedicationHistoryType medicationHistoryType = iterator.next();
            if (medicationHistoryType.getEntityId() != null) {
                String dGuid = ((EntityIdType) medicationHistoryType.getEntityId()).getId();
                if (StringUtils.isEmpty(dGuid)) {
                    validationFailure = true;
                }
            } else {
                validationFailure = true;
            }
        }

        for (MedicationSchemeEntriesRequest medicationSchemeEntryRequest : wrapper.getAllMedicationSchemeEntryRequests()) {
            for (DataEntryRequest dataEntryRequest : medicationSchemeEntryRequest.getDataEntry()) {
                if (StringUtils.isBlank(dataEntryRequest.getDguid())) {
                    validationFailure = true;
                }
            }
        }

        if (validationFailure) {
            LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: error.validation.dguid.not.filled.in");
            throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.dguid.not.filled.in"));
        }

    }

    public void validateSessionDateEmpty() throws IntegrationModuleException {
        String procedure = "validateSessionDateEmpty";
        SingleMessageWrapper wrapper = new SingleMessageWrapper(singleMessageObject);
        List<PharmaceuticalCareEventType> pharmaceuticalCareEventTypes = wrapper.getAllEventsOfType(PharmaceuticalCareEventType.class);
        if (HasPharmaceuticalCareEventType) {
            for (Iterator<PharmaceuticalCareEventType> iterator = pharmaceuticalCareEventTypes.iterator(); iterator.hasNext(); ) {
                PharmaceuticalCareEventType pharmaceuticalCareEventType = iterator.next();
                if (pharmaceuticalCareEventType.getSessionDateTime() != null) {
                    LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: error.validation.session.date.filled.in");
                    throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.session.date.filled.in"));
                }
            }
        }
    }

    public void validateSessionDateOk() throws IntegrationModuleException {
        String procedure = "validateSessionDateOk";
        XMLGregorianCalendar sessionDateTime = null;

        if (HasPharmaceuticalCareEventType) {
            for (PharmaceuticalCareEventType pharmaceuticalCareEventType : singleMessageWrapper.getAllEventsOfType(PharmaceuticalCareEventType.class)) {
                if (pharmaceuticalCareEventType.getSessionDateTime() != null) {
                    sessionDateTime = pharmaceuticalCareEventType.getSessionDateTime();
                    Date sessionDateTimeDate = sessionDateTime.toGregorianCalendar().getTime();
                    Calendar validationDate = Calendar.getInstance();
                    validationDate.setTime(new Date());
                    validationDate.add(Calendar.HOUR_OF_DAY, 2);
                    if (sessionDateTimeDate.after(validationDate.getTime())) {
                        LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: pharmaceuticalCareEventType.session.date.time.in.future");
                        throw new IntegrationModuleException(I18nHelper.getLabel("pharmaceuticalCareEventType.session.date.time.in.future"));
                    }
                } else {
                    LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: pharmaceuticalCareEventType.session.date.time.not.found");
                    throw new IntegrationModuleException(I18nHelper.getLabel("pharmaceuticalCareEventType.session.date.time.not.found"));
                }
            }
        }
    }

    public void validateMedicationHistorySessionDateFilledOk() throws IntegrationModuleException {
        String procedure = "validateMedicationHistorySessionDateFilledOk";
        SingleMessageWrapper wrapper = singleMessageWrapper;

        for (Iterator<MedicationHistoryType> iterator = wrapper.getAllMedicationHistoryEntries().iterator(); iterator.hasNext(); ) {
            MedicationHistoryType medicationHistoryType = iterator.next();
            if (medicationHistoryType.getDeliveryDate() == null) {
                LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: error.validation.delivery.date.not.filled.in");
                throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.delivery.date.not.filled.in"));
            }
        }
    }

    public void validateMotivationOk() throws IntegrationModuleException {
        String procedure = "validateMotivationOk";
        SingleMessageWrapper wrapper = singleMessageWrapper;

        for (Iterator<MaxSetProductType> iterator = wrapper.getAllDispensedProducts().iterator(); iterator.hasNext(); ) {
            MaxSetProductType product = iterator.next();
            if (product.getMotivation() == null) {
                LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: error.validation.motivation.not.filled.in");
                throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.motivation.not.filled.in"));
            }
            if (StringUtils.isEmpty(product.getMotivation().getFreeText())) {
                LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: error.validation.motivation.not.filled.in");
                throw new IntegrationModuleException(I18nHelper.getLabel("error.validation.motivation.not.filled.in"));
            }
        }
    }

    public void setHeaderNihiiPharmacyId(String NihiiPharmacyId) throws IntegrationModuleException {
        singleMessageObject.getUnsigned().setHeader(createHeaderType(NihiiPharmacyId));
        setSingleMessageObject(singleMessageObject);
    }

    private HeaderType createHeaderType(String compareNihiiPharmacy) throws IntegrationModuleException {
        String procedure = "createHeaderType";

        HeaderType header = new HeaderType();
        header.setSender(createSenderType(compareNihiiPharmacy));
        header.setMessageID(Utils.generateGUID(compareNihiiPharmacy));
        header.setMessageCreateDate(getXMLGregorianCalendarNow());

        try {
            header.setVersion(SingleMessageWrapper.getSMCVersion());
        } catch (GFDDPPException e) {
            LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: " + e.getStatusCode());
            throw new IntegrationModuleException(StatusResolver.resolveMessage(StatusResolver.getLocalResourceBundle(), e.getStatusCode(), e.getPlaceHolders()), e);
        }
        return header;
    }

    private XMLGregorianCalendar getXMLGregorianCalendarNow() throws IntegrationModuleException {
        String procedure = "getXMLGregorianCalendarNow";
        try {
            GregorianCalendar gregorianCalendar = new GregorianCalendar();
            DatatypeFactory datatypeFactory = DatatypeFactory.newInstance();
            XMLGregorianCalendar now = datatypeFactory.newXMLGregorianCalendar(gregorianCalendar);
            return now;
        } catch (DatatypeConfigurationException d) {
            LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error: error.technical");
            throw new IntegrationModuleException(I18nHelper.getLabel("error.technical"));
        }
    }

    private SenderType createSenderType(String compareNihiiPharmacy) {
        be.apb.standards.smoa.schema.model.v1.ObjectFactory objectFactory = new be.apb.standards.smoa.schema.model.v1.ObjectFactory();

        SenderType senderType = new SenderType();
        ReferencePharmacyType referencePharmacyType = new ReferencePharmacyType();
        NihiiIdType nihiiIdType = new NihiiIdType();
        nihiiIdType.setNihiiPharmacyNumber(compareNihiiPharmacy);
        referencePharmacyType.setPharmacyId(nihiiIdType);

        senderType.setAbstractPharmacy(objectFactory.createRefPharmacy(referencePharmacyType));
        return senderType;
    }

    public void setFilterProducts(ProductFilterUtils productFilterUtils, PropertyHandler propertyHandler) throws IntegrationModuleException {
        String procedure = "setFilterProducts";
        if (HasPharmaceuticalCareEventType) {

            LOG.debug(module + "=-----= ----- " + procedure + " =$=" + info + "== Filtering products out of single message");
            SingleMessage updatedSingleMessage = singleMessageObject; //only when productfiltering is done, there is an updated single message
            updatedSingleMessage = productFilterUtils.filterProducts(singleMessageObject, propertyHandler);
            setSingleMessageObject(updatedSingleMessage);
        }
    }

    public void setGuids() throws IntegrationModuleException {
        setGuids(null);
    }

    public void setGuids(String nihiiPharmacyNumber) throws IntegrationModuleException {
        String procedure = "setGuids";
        SingleMessageWrapper wrapper = singleMessageWrapper;

        if (HasPharmaceuticalCareEventType) {
            if ((nihiiPharmacyNumber == null) || (nihiiPharmacyNumber.isEmpty())) {
                nihiiPharmacyNumber = wrapper.getPharmaceuticalCareEventTypePharmacyId(wrapper.getAllEventsOfType(PharmaceuticalCareEventType.class).get(0));
            }
            for (PharmaceuticalCareEventType pharmaceuticalCareEventType : wrapper.getAllEventsOfType(PharmaceuticalCareEventType.class)) {
                String sGuid = Utils.generateGUID(String.valueOf(System.currentTimeMillis()) + nihiiPharmacyNumber);
                LOG.debug(module + "=-----= ----- " + procedure + " =$=" + info + "Generated SGUID: " + sGuid);
                pharmaceuticalCareEventType.setId(sGuid);
            }

            for (Product product : wrapper.getAllDispensedForSamePrescriptionProducts()) {
                AbstractMedicinalProductIdType abstractMedicinalProductIdType = (AbstractMedicinalProductIdType) product.getDescription().getProductCode();
                String productKey = getProductId(abstractMedicinalProductIdType, product);
                String dGuid = Utils.generateGUID(productKey + nihiiPharmacyNumber);
                LOG.debug(module + "=-----= ----- " + procedure + " =$=" + info + "Generated DGUID: " + dGuid);
                product.setDispensationGUID(dGuid);
            }

            for (MaxSetProductType product : wrapper.getAllDispensedWithoutPrescriptionProducts()) {
                AbstractMedicinalProductIdType abstractMedicinalProductIdType = (AbstractMedicinalProductIdType) product.getDescription().getProductCode();
                String productKey = getProductMaxSetProductTypeId(abstractMedicinalProductIdType, product);
                String dGuid = Utils.generateGUID(productKey + nihiiPharmacyNumber);
                LOG.debug(module + "=-----= ----- " + procedure + " =$=" + info + "Generated DGUID: " + dGuid);
                product.setDispensationGUID(dGuid);
            }
        }

        if (HasBvacEventType) {
            if ((nihiiPharmacyNumber == null) || (nihiiPharmacyNumber.isEmpty())) {
                nihiiPharmacyNumber = wrapper.getBvacEventTypePharmacyId(wrapper.getAllEventsOfType(BvacEventType.class).get(0));
            }
            for (BvacEventType bvacEventType : wrapper.getAllEventsOfType(BvacEventType.class)) {
                String sGuid = Utils.generateGUID(String.valueOf(System.currentTimeMillis()) + nihiiPharmacyNumber);
                LOG.debug(module + "=-----= ----- " + procedure + " =$=" + info + "Generated SGUID: " + sGuid);
                String dGuid = Utils.generateGUID(String.valueOf(System.currentTimeMillis()) + nihiiPharmacyNumber);
                LOG.debug(module + "=-----= ----- " + procedure + " =$=" + info + "Generated DGUID: " + dGuid);
                bvacEventType.setId(sGuid);
                if (bvacEventType.getPharmacyList() != null && bvacEventType.getPharmacyList().getPharmacy().size() > 0
                        && bvacEventType.getPharmacyList().getPharmacy().get(0).getBvacList() != null
                        && bvacEventType.getPharmacyList().getPharmacy().get(0).getBvacList().getBvacDocument().size() > 0
                        && bvacEventType.getPharmacyList().getPharmacy().get(0).getBvacList().getBvacDocument().get(0).getIdentification() != null) {
                    bvacEventType.getPharmacyList().getPharmacy().get(0).getBvacList().getBvacDocument().get(0).getIdentification().setId(dGuid);
                }
            }
        }

        if (HasArchivePrescriptionEventType) {
            if ((nihiiPharmacyNumber == null) || (nihiiPharmacyNumber.isEmpty())) {
                nihiiPharmacyNumber = wrapper.getArchivePrescriptionEventTypePharmacyId(wrapper.getAllEventsOfType(ArchivePrescriptionEventType.class).get(0));
            }

            for (ArchivePrescriptionEventType apet : wrapper.getAllEventsOfType(ArchivePrescriptionEventType.class)) {
                String sGuid = Utils.generateGUID(String.valueOf(System.currentTimeMillis()) + nihiiPharmacyNumber);
                LOG.debug(module + "=-----= ----- " + procedure + " =$=" + info + "Generated SGUID: " + sGuid);
                String dGuid = Utils.generateGUID(String.valueOf(System.currentTimeMillis()) + nihiiPharmacyNumber);
                LOG.debug(module + "=-----= ----- " + procedure + " =$=" + info + "Generated DGUID: " + dGuid);
                apet.setId(sGuid);
                apet.setSguid(sGuid);
                apet.setDguid(dGuid);
            }
        }

        if (HasArchivePrescriptionCommentEventType) {
            if ((nihiiPharmacyNumber == null) || (nihiiPharmacyNumber.isEmpty())) {
                nihiiPharmacyNumber = wrapper.getArchivePrescriptionCommentEventTypePharmacyId(wrapper.getAllEventsOfType(ArchivePrescriptionCommentEventType.class).get(0));
            }

            for (ArchivePrescriptionCommentEventType apcet : wrapper.getAllEventsOfType(ArchivePrescriptionCommentEventType.class)) {
                String sGuid = Utils.generateGUID(String.valueOf(System.currentTimeMillis()) + nihiiPharmacyNumber);
                LOG.debug(module + "=-----= ----- " + procedure + " =$=" + info + "Generated SGUID: " + sGuid);
                String dGuid = Utils.generateGUID(String.valueOf(System.currentTimeMillis()) + nihiiPharmacyNumber);
                LOG.debug(module + "=-----= ----- " + procedure + " =$=" + info + "Generated DGUID: " + dGuid);
                apcet.setId(sGuid);
                apcet.setSguid(sGuid);
                apcet.setDguid(dGuid);
            }
        }

        if (HasMedicationSchemeEntriesRequest) {
            nihiiPharmacyNumber = StringUtils.defaultString(nihiiPharmacyNumber);
            for (MedicationSchemeEntriesRequest medicationSchemeEntriesRequest : wrapper.getAllMedicationSchemeEntryRequests()) {
                String sGuid = Utils.generateGUID(String.valueOf(System.currentTimeMillis()) + nihiiPharmacyNumber);
                LOG.debug(module + "=-----= ----- " + procedure + " =$=" + info + "Generated SGUID: " + sGuid);
                medicationSchemeEntriesRequest.setId(sGuid);
                for (DataEntryRequest dataEntryRequest : medicationSchemeEntriesRequest.getDataEntry()) {
                    String dGuid = Utils.generateGUID(String.valueOf(System.currentTimeMillis()) + nihiiPharmacyNumber);
                    LOG.debug(module + "=-----= ----- " + procedure + " =$=" + info + "Generated DGUID: " + dGuid);
                    dataEntryRequest.setDguid(dGuid);
                }
            }
        }

        if (HasContinuedPharmaceuticalCareDossierEvent) {
            nihiiPharmacyNumber = StringUtils.defaultString(nihiiPharmacyNumber);
            for (ContinuedPharmaceuticalCareDossierEvent event : wrapper.getAllEventsOfType(ContinuedPharmaceuticalCareDossierEvent.class)) {
                String sGuid = Utils.generateGUID(String.valueOf(System.currentTimeMillis()) + nihiiPharmacyNumber);
                LOG.debug(module + "=-----= ----- " + procedure + " =$=" + info + "Generated SGUID: " + sGuid);
                event.setId(sGuid);
                String dGuid = Utils.generateGUID(String.valueOf(System.currentTimeMillis()) + nihiiPharmacyNumber);
                LOG.debug(module + "=-----= ----- " + procedure + " =$=" + info + "Generated DGUID: " + dGuid);
                event.setDguid(dGuid);
            }
        }

        setSingleMessageObject(singleMessageObject);
    }

    public SealedMessageRequestType setCommonData(EncryptionUtils encryptionUtils, String tipId, boolean requestPatientSignature, ThreeStateBoolean patientConsent) throws IntegrationModuleException, GFDDPPException, UnsupportedEncodingException, SingleMessageValidationException {
        SingleMessageWrapper wrapper = singleMessageWrapper;

        SealedMessageRequestType sealedMessageRequestType = new SealedMessageRequestType();
        sealedMessageRequestType.setRoutingParameters(createRoutingParameter(tipId));

        if (HasPharmaceuticalCareEventType) {
            MaxSetPatient patient = ((List<PharmaceuticalCareEventType>) wrapper.getAllEventsOfType(PharmaceuticalCareEventType.class)).get(0).getMaxPatient();
            PersonIdType personIdType = PersonIdType.valueOf(patient.getIdentification().getPersonId());
            String patientid = personIdType.getIdFrom(patient.getIdentification().getPersonId());
            sealedMessageRequestType.setAuthorizationParameters(RequestCreatorUtils.createAuthorizationParameters(requestPatientSignature, patientConsent, null, patientid, encryptionUtils));
        }

        sealedMessageRequestType.setSingleMessageSealed(getXmlBytes());
        return sealedMessageRequestType;
    }

    private RoutingParametersType createRoutingParameter(String tipId) {
        RoutingParametersType routingParametersType = new RoutingParametersType();
        routingParametersType.setTIPId(tipId);
        return routingParametersType;
    }

    private String getProductId(AbstractMedicinalProductIdType abstractMedicinalProductIdType, Product product) {
        String productKey = null;

        if (abstractMedicinalProductIdType instanceof CnkIdType) {
            CnkIdType cnkIdType = (CnkIdType) product.getDescription().getProductCode();
            productKey = cnkIdType.getCnk();
        } else if (abstractMedicinalProductIdType instanceof BeRegistrationIdType) {
            BeRegistrationIdType beRegistrationIdType = (BeRegistrationIdType) product.getDescription().getProductCode();
            productKey = String.valueOf(beRegistrationIdType.getCti());
        } else if (abstractMedicinalProductIdType instanceof AtcDddSystemIdType) {
            AtcDddSystemIdType atcDddSystemIdType = (AtcDddSystemIdType) product.getDescription().getProductCode();
            if (!StringUtils.isEmpty(atcDddSystemIdType.getAtc())) {
                productKey = atcDddSystemIdType.getAtc();
            } else if (!StringUtils.isEmpty(atcDddSystemIdType.getDdd())) {
                productKey = atcDddSystemIdType.getDdd();
            }
        }

        return productKey;
    }

    private String getProductMaxSetProductTypeId(AbstractMedicinalProductIdType abstractMedicinalProductIdType, MaxSetProductType product) {
        String productKey = null;

        if (abstractMedicinalProductIdType instanceof CnkIdType) {
            CnkIdType cnkIdType = (CnkIdType) product.getDescription().getProductCode();
            productKey = cnkIdType.getCnk();
        } else if (abstractMedicinalProductIdType instanceof BeRegistrationIdType) {
            BeRegistrationIdType beRegistrationIdType = (BeRegistrationIdType) product.getDescription().getProductCode();
            productKey = String.valueOf(beRegistrationIdType.getCti());
        } else if (abstractMedicinalProductIdType instanceof AtcDddSystemIdType) {
            AtcDddSystemIdType atcDddSystemIdType = (AtcDddSystemIdType) product.getDescription().getProductCode();
            if (!StringUtils.isEmpty(atcDddSystemIdType.getAtc())) {
                productKey = atcDddSystemIdType.getAtc();
            } else if (!StringUtils.isEmpty(atcDddSystemIdType.getDdd())) {
                productKey = atcDddSystemIdType.getDdd();
            }
        }
        return productKey;
    }

    /*
     * getDeliveryIds returns all the ids at this moment. In the future other id's will be possible.
     */
    public String getEvents() throws IntegrationModuleException, GFDDPPException {
        SingleMessageWrapper wrapper = singleMessageWrapper;

        Events events = new Events();

        if (HasBvacEventType) {
            be.apb.gfddpp.rtrn.registerdata.BvacEventType returnBvacEventType = new be.apb.gfddpp.rtrn.registerdata.BvacEventType();
            for (BvacEventType bvacEventType : wrapper.getAllEventsOfType(BvacEventType.class)) {
                returnBvacEventType.setSguid(bvacEventType.getId());

                if (bvacEventType.getPharmacyList() != null && bvacEventType.getPharmacyList().getPharmacy().size() > 0 && bvacEventType.getPharmacyList().getPharmacy().get(0).getBvacList() != null && bvacEventType.getPharmacyList().getPharmacy().get(0).getBvacList().getBvacDocument().size() > 0 && bvacEventType.getPharmacyList().getPharmacy().get(0).getBvacList().getBvacDocument().get(0).getIdentification() != null) {
                    BvacDocument bvacDocument = new BvacDocument();
                    bvacDocument.setDguid(bvacEventType.getPharmacyList().getPharmacy().get(0).getBvacList().getBvacDocument().get(0).getIdentification().getId());
                    returnBvacEventType.setBvacDocument(bvacDocument);
                }
            }
            events.setBvacEventType(returnBvacEventType);
        } else if (HasPharmaceuticalCareEventType) {
            be.apb.gfddpp.rtrn.registerdata.PharmaceuticalCareEventType pharmCareEventType = new be.apb.gfddpp.rtrn.registerdata.PharmaceuticalCareEventType();
            List<Dispensation> productsList = pharmCareEventType.getDispensation();

            for (PharmaceuticalCareEventType pharmaceuticalCareEventType : wrapper.getAllEventsOfType(PharmaceuticalCareEventType.class)) {
                for (DispensedForSamePrescription dispensedForSamePrescription : wrapper.getAllDispensedForSamePrescriptions()) {
                    for (Product product : dispensedForSamePrescription.getProduct()) {
                        Dispensation products = new Dispensation();
                        products.setDguid(product.getDispensationGUID());
                        Description description = product.getDescription();
                        if (description.getProductCode() != null) {
                            be.apb.gfddpp.rtrn.registerdata.Product prdct = new be.apb.gfddpp.rtrn.registerdata.Product();
                            prdct.setProductId(ProductFilterUtils.getId(description.getProductCode()));
                            products.setProduct(prdct);
                        } else if (description.getMagistralPreparation() != null) {
                            products.setMagistral(ProductFilterUtils.getId(description.getMagistralPreparation()));
                        }

                        if (StringUtils.isBlank(pharmCareEventType.getSguid())) {
                            pharmCareEventType.setSguid(pharmaceuticalCareEventType.getId());
                        }
                        if (StringUtils.isNotEmpty(product.getLocalReference())) {
                            products.setLocalReference(product.getLocalReference());
                        }
                        productsList.add(products);
                    }
                }

                for (DispensedWithoutPrescription dispensedWithoutPrescription : wrapper.getAllDispensedWithoutPrescriptions()) {
                    for (MaxSetProductType maxSetProductType : dispensedWithoutPrescription.getProduct()) {
                        Dispensation products = new Dispensation();
                        products.setDguid(maxSetProductType.getDispensationGUID());
                        Description description = maxSetProductType.getDescription();
                        if (description.getProductCode() != null) {
                            be.apb.gfddpp.rtrn.registerdata.Product prdct = new be.apb.gfddpp.rtrn.registerdata.Product();
                            prdct.setProductId(ProductFilterUtils.getId(description.getProductCode()));
                            products.setProduct(prdct);
                        } else if (description.getMagistralPreparation() != null) {
                            products.setMagistral(ProductFilterUtils.getId(description.getMagistralPreparation()));
                        }
                        if (StringUtils.isBlank(pharmCareEventType.getSguid())) {
                            pharmCareEventType.setSguid(pharmaceuticalCareEventType.getId());
                        }
                        if (StringUtils.isNotEmpty(maxSetProductType.getLocalReference())) {
                            products.setLocalReference(maxSetProductType.getLocalReference());
                        }
                        productsList.add(products);
                    }
                }
            }
            events.setPharmaceuticalCareEventType(pharmCareEventType);
        } else if (HasArchivePrescriptionEventType) {

            be.apb.gfddpp.rtrn.registerdata.ArchivePrescriptionEventType prescriptionEventType = new be.apb.gfddpp.rtrn.registerdata.ArchivePrescriptionEventType();
            for (ArchivePrescriptionEventType archivePrescriptionEventType : wrapper.getAllEventsOfType(ArchivePrescriptionEventType.class)) {
                prescriptionEventType.setSguid(archivePrescriptionEventType.getId());
                prescriptionEventType.setDguid(archivePrescriptionEventType.getDguid());
                prescriptionEventType.setRid(archivePrescriptionEventType.getRid());
            }
            events.setArchivePrescriptionEventType(prescriptionEventType);
        } else if (HasArchivePrescriptionCommentEventType) {

            be.apb.gfddpp.rtrn.registerdata.ArchivePrescriptionCommentEventType apcet = new be.apb.gfddpp.rtrn.registerdata.ArchivePrescriptionCommentEventType();
            for (ArchivePrescriptionCommentEventType archivePrescriptionCommentEventType : wrapper.getAllEventsOfType(ArchivePrescriptionCommentEventType.class)) {
                apcet.setSguid(archivePrescriptionCommentEventType.getId());
                apcet.setDguid(archivePrescriptionCommentEventType.getDguid());
                apcet.setRid(archivePrescriptionCommentEventType.getRid());
            }
            events.setArchivePrescriptionCommentEventType(apcet);
        } else if (HasContinuedPharmaceuticalCareDossierEvent) {
            be.apb.gfddpp.rtrn.registerdata.ContinuedPharmaceuticalCareDossierEvent apcet = new be.apb.gfddpp.rtrn.registerdata.ContinuedPharmaceuticalCareDossierEvent();
            for (ContinuedPharmaceuticalCareDossierEvent event : wrapper.getAllEventsOfType(ContinuedPharmaceuticalCareDossierEvent.class)) {
                apcet.setSguid(event.getId());
                apcet.setDguid(event.getDguid());
            }
            events.setContinuedPharmaceuticalCareDossierEvent(apcet);
        }
        
        
        return JaxContextCentralizer.getInstance().toXml(Events.class, events);
    }

    public String getLogInfoGuids() {
        String info = " ";
        SingleMessageWrapper wrapper = singleMessageWrapper;

        try {
            info += "Version:" + wrapper.getVersion();
        } catch (Exception e) {
        }

        try {
            info += " PharmacyId:" + wrapper.getHeaderPharmacyId();
        } catch (Exception e) {
        }

        if (HasPharmaceuticalCareEventType) {
            try {
                List<PharmaceuticalCareEventType> listEvent = wrapper.getAllEventsOfType(PharmaceuticalCareEventType.class);
                for (PharmaceuticalCareEventType event : listEvent) {
                    info += "EventType:PharmaceuticalCareEventType";
                    info += " sguid:" + event.getId();
                    if (event.getDispensedWithoutPrescription() != null) {
                        for (MaxSetProductType product : event.getDispensedWithoutPrescription().getProduct()) {
                            info += " dguid:" + product.getDispensationGUID();
                        }
                    }
                    for (DispensedForSamePrescription dispensedForSamePrescription : event.getDispensedForSamePrescription()) {
                        for (Product product : dispensedForSamePrescription.getProduct()) {
                            info += " dguid:" + product.getDispensationGUID();
                        }
                    }
                }
            } catch (Exception e) {
            }
        }

        if (HasBvacEventType) {
            try {
                List<BvacEventType> listEvent = wrapper.getAllEventsOfType(BvacEventType.class);
                for (BvacEventType event : listEvent) {
                    info += "EventType:BvacEventType";
                    for (be.apb.standards.smoa.schema.model.v1.BvacDocument bvacDocument : event.getPharmacyList().getPharmacy().get(0).getBvacList().getBvacDocument()) {
                        info += "dguid:" + bvacDocument.getIdentification().getId();
                    }
                }
            } catch (Exception e) {
            }
        }

        if (HasArchivePrescriptionEventType) {
            try {
                List<ArchivePrescriptionEventType> listEvent = wrapper.getAllEventsOfType(ArchivePrescriptionEventType.class);
                for (ArchivePrescriptionEventType event : listEvent) {
                    info += "EventType:ArchivePrescriptionEventType";
                    info += "sguid:" + event.getSguid();
                    info += " dguid:" + event.getDguid();
                }
            } catch (Exception e) {
            }
        }

        if (HasArchivePrescriptionCommentEventType) {
            try {
                List<ArchivePrescriptionCommentEventType> listEvent = wrapper.getAllEventsOfType(ArchivePrescriptionCommentEventType.class);
                for (ArchivePrescriptionCommentEventType event : listEvent) {
                    info += "EventType:ArchivePrescriptionCommentEventType";
                    info += "sguid:" + event.getSguid();
                    info += " dguid:" + event.getDguid();
                }
            } catch (Exception e) {
            }
        }

        if (HasMedicationHistoryEvent) {
            try {
                List<MedicationHistoryEvent> medicationHistoryEvents = wrapper.getAllEventsOfType(MedicationHistoryEvent.class);
                for (MedicationHistoryEvent event : medicationHistoryEvents) {
                    info += "EventType:MedicationHistoryEvent";
                    info += " sguid:" + event.getId();
                    for (MedicationHistoryType entity : event.getMedicationHistoryEntity()) {
                        info += " dguid:" + ((EntityIdType) entity.getEntityId()).getId();
                    }
                }
            } catch (Exception e) {
            }
        }
        return info;
    }

    public String getInfo() {
        return this.info;
    }

    public boolean getHasSyncEventType() {
        return this.HasSyncEventType;
    }

    public boolean getHasMedicationHistoryEvent() {
        return this.HasMedicationHistoryEvent;
    }

    public boolean getHasMedicationSchemeEntriesRequest() {
        return this.HasMedicationSchemeEntriesRequest;
    }

    public String getMessageID() {
        return singleMessageWrapper.getMessageID();
    }

    public String getEventType() {
        return eventtype;
    }

    public String getHeaderPharmacyId() {
        return singleMessageWrapper.getHeaderPharmacyId();
    }

    public SingleMessageWrapper getWrapper() {
        return singleMessageWrapper;
    }

    public byte[] getXmlBytes() throws GFDDPPException {
        String s = getJaxContextCentralizer().toXml(SingleMessage.class, singleMessageObject);
        byte[] ret = s.getBytes(Charsets.UTF_8);
        return ret;
    }

    public String getXmlString() throws GFDDPPException {
        return getJaxContextCentralizer().toXml(SingleMessage.class, singleMessageObject);
    }

}
