package be.business.connector.gfddpp.utils;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URL;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
//import org.apache.log4j.Logger;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import be.apb.gfddpp.validation.exception.SingleMessageValidationException;

import be.apb.gfddpp.helper.PropertyHandler;

public class SingleMessageValidationXsd {
	private final static Logger LOG = Logger.getLogger(SingleMessageValidationXsd.class);
	private final static String module = "SingleMessageValidationXsd";

	private static final String SINGLEMESSAGE_XSD = "singlemessage.xsd";
	private static final String JAXP_SCHEMA_SOURCE = "http://java.sun.com/xml/jaxp/properties/schemaSource";
	private static final String JAXP_SCHEMA_LANGUAGE = "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
	private static final String W3C_XML_SCHEMA_NS_URI = "http://www.w3.org/2001/XMLSchema";

	private URL xsdPath = null;
	private PropertyHandler propertyHandler;
	private String info = " ";
	
	public SingleMessageValidationXsd() {
		propertyHandler = new PropertyHandler("/smc.properties");
	}

	public SingleMessageValidationXsd(URL xsdPath) {
		this.xsdPath = xsdPath;
	}

	/**
	 * Assert valid kmehr prescription.
	 *
	 * @param xmlFile the xml file
	 * @param prescriptionType the prescription type
	 * @throws SingleMessageValidationException 
	 */
	public void assertValidSingleMessage(InputStream xmlFile) throws SingleMessageValidationException {
		String xmlString = getString(xmlFile);
		assertValidSingleMessage(xmlString);
	}

	public void assertValidSingleMessage(String xmlDocument) throws SingleMessageValidationException {
		xsdValidate(xmlDocument);
	}

	public void assertValidSingleMessage(byte[] xmlDocument) throws SingleMessageValidationException {
		String procedure = "assertValidSingleMessage";
		LOG.debug(module + "=-----= start " + procedure + " =$=" + info  );
		
		try {
			xsdValidate(new String(xmlDocument, "UTF-8"));
		} catch (UnsupportedEncodingException e) {
			LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + e.getMessage(),e );
		}
		
		LOG.debug(module + "=-----= einde " + procedure + " =$=" + info  );
	}

	public byte[] getBytes(InputStream inputStream) {
		String procedure = "getBytes";
		LOG.debug(module + "=-----= start " + procedure + " =$=" + info  );
		
		try {
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			byte[] buffer = new byte[1024];
			int len;
			while ((len = inputStream.read(buffer)) >= 0) {
				baos.write(buffer, 0, len);
			}
			LOG.debug(module + "=-----= einde " + procedure + " =$=" + info  );
			return baos.toByteArray();
		} catch (IOException e) {
			LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + e.getMessage(),e );
			throw new IllegalArgumentException(e);
		}
	}

	public String getString(InputStream inputStream) {
		String procedure = "getString";
		LOG.debug(module + "=-----= start " + procedure + " =$=" + info  );
		
		try {
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			byte[] buffer = new byte[1024];
			int len;
			while ((len = inputStream.read(buffer)) >= 0) {
				baos.write(buffer, 0, len);
			}
			LOG.debug(module + "=-----= einde " + procedure + " =$=" + info  );
			return baos.toString("UTF-8");
		} catch (IOException e) {
			LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + e.getMessage(),e );
			throw new IllegalArgumentException(e);
		}
		
		
	}

	/**
	 * Xsd validate.
	 *
	 * @param xmlDocument the xml document
	 * @param xsdPropertyName the xsd property name
	 * @throws SingleMessageValidationException 
	 * @throws ParserConfigurationException the parser configuration exception
	 * @throws SAXException the sAX exception
	 * @throws IOException Signals that an I/O exception has occurred.
	 */
	private void xsdValidate(String xmlDocument) throws SingleMessageValidationException {
		String procedure = "xsdValidate";
		LOG.debug(module + "=-----= start " + procedure + " =$=" + info  );
		
		try {

			if (xsdPath == null && propertyHandler != null) {
				xsdPath = getClass().getResource(propertyHandler.getProperty(SINGLEMESSAGE_XSD));
			}

			if (xsdPath == null) {
				//					|| ( url !=null && !new File(url.toURI()).exists()) 
				LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error Property: "+ SINGLEMESSAGE_XSD );
				throw new RuntimeException("Property is not correctly set");
			}

			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

			factory.setNamespaceAware(true);
			factory.setValidating(true);
			factory.setAttribute(JAXP_SCHEMA_LANGUAGE, W3C_XML_SCHEMA_NS_URI);
			//	factory.setAttribute(	JAXP_SCHEMA_SOURCE, 	 url);

			factory.setAttribute(JAXP_SCHEMA_SOURCE, new String[] { xsdPath.toString() });

			DocumentBuilder documentBuilder = factory.newDocumentBuilder();
			documentBuilder.setErrorHandler(new ErrorHandler() {
                                @Override
				public void warning(SAXParseException arg0) throws SAXException {
					String procedure = "xsdValidate";
					LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error warning: ",arg0  );
				}

                                @Override
				public void fatalError(SAXParseException arg0) throws SAXException {
					String procedure = "xsdValidate";
					LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error fatal: ",arg0  );
					throw arg0;
				}

                                @Override
				public void error(SAXParseException arg0) throws SAXException {
					String procedure = "xsdValidate";
					LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error error: ",arg0  );
					throw arg0;
				}
			});
			documentBuilder.parse(new ByteArrayInputStream(xmlDocument.getBytes("UTF-8")));
		} catch (Exception e) {
			LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error SingleMessageValidationException: ",e  );
			throw new SingleMessageValidationException(e);
		}
		LOG.debug(module + "=-----= einde " + procedure + " =$=" + info  );
		
	}

	public InputStream getResourceAsStream(String path) throws IOException {
		String procedure = "getResourceAsStream";
		File f = new File(path);
		if (f.exists()) {
			return new FileInputStream(f);
		} else {
			URL url = SingleMessageValidationXsd.class.getResource(path);
			InputStream stream = SingleMessageValidationXsd.class.getResourceAsStream(path);

			if (stream == null) {
				LOG.error(module + "=-----= ----- " + procedure + " =$=" + info + "error Invalid resource: " + path);
				throw new IOException("Invalid resource " + path);
			}
			return stream;
		}
	}
}
