package be.business.connector.mycarenet;

import be.apb.gfddpp.common.utils.JaxContextCentralizer;
import be.business.connector.common.ApplicationConfig;
import be.business.connector.common.module.AbstractIntegrationModule;
import be.business.connector.core.exceptions.IntegrationModuleException;
import be.business.connector.core.handlers.InsurabilityHandler;
import be.business.connector.core.utils.Exceptionutils;
import be.business.connector.mycarenet.domain.Insurability;
import be.business.connector.mycarenet.services.InsurabilityConsultationServiceImpl;
import be.fgov.ehealth.insurability.protocol.v1.GetInsurabilityForPharmacistResponse;
import org.apache.log4j.Logger;
import org.perf4j.aop.Profiled;

public class MyCareNetIntegrationModuleImpl extends AbstractIntegrationModule implements MyCareNetIntegrationModule {
	private final static Logger LOG = Logger.getLogger(MyCareNetIntegrationModuleImpl.class);

	public MyCareNetIntegrationModuleImpl() throws IntegrationModuleException {
		super();
                LOG.debug("MyCareNetIntegrationModule V1 initialized.");
	}

	@Deprecated
	@Profiled(logFailuresSeparately = true, tag = "0.MyCareNet#getInsurability")
	@Override
	public String getInsurability(String userName, String password, String pharmacyHolder, String pharmacySSIN, String pharmcayNihii, String date, String type, String careReceiverSSIN, String careReceiverMutuality, String careReceiverRegNrWithMut, String packageName, String reference, String userReference) throws IntegrationModuleException {
		ApplicationConfig.getInstance().assertValidSession();
		try {
			MyCareNetRequest myCareNet = new MyCareNetRequest();
			LOG.debug("******************** Call MyCarenet for insurability ******************** ");
			GetInsurabilityForPharmacistResponse ins = InsurabilityConsultationServiceImpl.getInstance().getInsurabilityForPharmacist(myCareNet.createRequest(userName, password, pharmacyHolder, pharmacySSIN, pharmcayNihii, date, type, careReceiverSSIN, careReceiverMutuality, careReceiverRegNrWithMut, packageName, reference, userReference));
			String insXml = null;

			if (ins != null) {
				insXml = JaxContextCentralizer.getInstance().toXml(GetInsurabilityForPharmacistResponse.class, ins);
				LOG.debug("******************** Call MyCarenet for insurability response: " + insXml + " ******************** ");
			}
			return insXml;
		} catch (Throwable t) {
			Exceptionutils.errorHandler(t);
		}
		return null;
	}

	@Profiled(logFailuresSeparately = true, tag = "0.MyCareNet#getInsurability")
	@Override
	public Insurability getInsurabilityForPharmacist(String userName, String password, String pharmacyHolder, String pharmacySSIN, String pharmcayNihii, String date, String type, String careReceiverSSIN, String careReceiverMutuality, String careReceiverRegNrWithMut, String packageName, String reference, String userReference) throws IntegrationModuleException {
		ApplicationConfig.getInstance().assertValidSession();
		try {
			MyCareNetRequest myCareNet = new MyCareNetRequest();
			InsurabilityHandler.setInsurability(null);
			InsurabilityHandler.setMessageId(null);
			LOG.debug("******************** Call MyCarenet for insurability ******************** ");
			GetInsurabilityForPharmacistResponse ins = InsurabilityConsultationServiceImpl.getInstance().getInsurabilityForPharmacist(myCareNet.createRequest(userName, password, pharmacyHolder, pharmacySSIN, pharmcayNihii, date, type, careReceiverSSIN, careReceiverMutuality, careReceiverRegNrWithMut, packageName, reference, userReference));
			String insXml = null;
			if (ins != null) {
				insXml = JaxContextCentralizer.getInstance().toXml(GetInsurabilityForPharmacistResponse.class, ins);
				LOG.debug("******************** Call MyCarenet for insurability response: " + insXml + " ******************** ");
			}
			return new Insurability(insXml, InsurabilityHandler.getMessageId());
		} catch (Throwable t) {
			Exceptionutils.errorHandler(t);
		}
		return null;
	}

}
