package be.business.connector.mycarenet;

import org.apache.log4j.Logger;
import org.perf4j.aop.Profiled;

import be.apb.gfddpp.common.utils.JaxContextCentralizer;
import be.business.connector.common.ApplicationConfig;
import be.business.connector.common.module.AbstractIntegrationModule;
import be.business.connector.core.exceptions.IntegrationModuleException;
import be.business.connector.core.handlers.InsurabilityHandler;
import be.business.connector.core.utils.Exceptionutils;
import be.business.connector.mycarenet.domain.Insurability;
import be.business.connector.mycarenet.services.InsurabilityConsultationServiceV2Impl;
import be.fgov.ehealth.insurability.protocol.v2.GetInsurabilityForPharmacistResponse;

public class MyCareNetIntegrationModuleV2Impl extends AbstractIntegrationModule implements MyCareNetIntegrationModule {
	private final static Logger LOG = Logger.getLogger(MyCareNetIntegrationModuleV2Impl.class);

	public MyCareNetIntegrationModuleV2Impl() throws IntegrationModuleException {
		super();
                LOG.debug("MyCareNetIntegrationModule V2 initialized.");
	}

	@Deprecated
	@Profiled(logFailuresSeparately = true, tag = "0.MyCareNet#getInsurability")
	@Override
	public String getInsurability(String userName, String password, String pharmacyHolder, String pharmacySSIN, String pharmcayNihii, String date, String type, String careReceiverSSIN, String careReceiverMutuality, String careReceiverRegNrWithMut, String packageName, String reference, String userReference) throws IntegrationModuleException {
		ApplicationConfig.getInstance().assertValidSession();
		try {
			MyCareNetRequestV2 myCareNet = new MyCareNetRequestV2();
			LOG.debug("******************** Call MyCarenet for insurability ******************** ");
			GetInsurabilityForPharmacistResponse ins = InsurabilityConsultationServiceV2Impl.getInstance().getInsurabilityForPharmacist(myCareNet.createRequest(userName, password, pharmacyHolder, pharmacySSIN, pharmcayNihii, date, type, careReceiverSSIN, careReceiverMutuality, careReceiverRegNrWithMut, packageName, reference, userReference));
			String insXml = null;
			if (ins != null) {
				insXml = JaxContextCentralizer.getInstance().toXml(GetInsurabilityForPharmacistResponse.class, ins);
				LOG.debug("******************** Call MyCarenet for insurability response: " + insXml + " ******************** ");
			}
			return insXml;
		} catch (Throwable t) {
			Exceptionutils.errorHandler(t);
		}
		return null;
	}

	@Profiled(logFailuresSeparately = true, tag = "0.MyCareNet#getInsurability")
	@Override
	public Insurability getInsurabilityForPharmacist(String userName, String password, String pharmacyHolder, String pharmacySSIN, String pharmcayNihii, String date, String type, String careReceiverSSIN, String careReceiverMutuality, String careReceiverRegNrWithMut, String packageName, String reference, String userReference) throws IntegrationModuleException {
		ApplicationConfig.getInstance().assertValidSession();
		try {
			MyCareNetRequestV2 myCareNet = new MyCareNetRequestV2();
			InsurabilityHandler.setInsurability(null);
			InsurabilityHandler.setMessageId(null);
			LOG.debug("******************** Call MyCarenet for insurability ******************** ");
			GetInsurabilityForPharmacistResponse ins = InsurabilityConsultationServiceV2Impl.getInstance().getInsurabilityForPharmacist(myCareNet.createRequest(userName, password, pharmacyHolder, pharmacySSIN, pharmcayNihii, date, type, careReceiverSSIN, careReceiverMutuality, careReceiverRegNrWithMut, packageName, reference, userReference));
			String insXml = null;
			if (ins != null) {
				insXml = JaxContextCentralizer.getInstance().toXml(GetInsurabilityForPharmacistResponse.class, ins);
				LOG.debug("******************** Call MyCarenet for insurability response: " + insXml + " ******************** ");
			}
			return new Insurability(insXml, InsurabilityHandler.getMessageId());
		} catch (Throwable t) {
			Exceptionutils.errorHandler(t);
		}
		return null;
	}

}
