package be.business.connector.recipe.executor;

import java.util.List;

import be.business.connector.core.exceptions.IntegrationModuleException;
import be.business.connector.recipe.executor.domain.GetPrescriptionForExecutorResult;
import be.recipe.services.executor.ListNotificationsItem;

/**
 * The Interface ExecutorIntegrationModule.
 */
public interface ExecutorIntegrationModule {

	/**
	 * Gets the prescription.
	 *
	 * @param rid
	 *            the rid
	 * @return the {@link GetPrescriptionForExecutorResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	GetPrescriptionForExecutorResult getPrescription(String rid) throws IntegrationModuleException;

	/**
	 * Mark as delivered.
	 *
	 * @param rid
	 *            the rid
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	void markAsDelivered(String rid) throws IntegrationModuleException;

	/**
	 * Mark as archived.
	 *
	 * @param rid
	 *            the rid
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	void markAsArchived(String rid) throws IntegrationModuleException;

	/**
	 * Mark as undelivered.
	 *
	 * @param rid
	 *            the rid
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	void markAsUndelivered(String rid) throws IntegrationModuleException;

	/**
	 * Revoke prescription.
	 *
	 * @param rid
	 *            the rid
	 * @param reason
	 *            the reason
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	void revokePrescription(String rid, String reason) throws IntegrationModuleException;

	/**
	 * List notifications.
	 *
	 * @param readFlag
	 *            the read flag
	 * @return the list of {@link ListNotificationsItem}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	List<ListNotificationsItem> listNotifications(boolean readFlag) throws IntegrationModuleException;

	/**
	 * Creates the feedback.
	 *
	 * @param prescriberId
	 *            the prescriber id
	 * @param rid
	 *            the rid
	 * @param feedbackText
	 *            the feedback text
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	void createFeedback(String prescriberId, String rid, byte[] feedbackText) throws IntegrationModuleException;

}