package be.business.connector.recipe.executor;

import be.business.connector.core.exceptions.IntegrationModuleException;
import be.recipe.services.executor.GetOpenPrescriptionForExecutor;
import be.recipe.services.executor.GetPrescriptionStatusParam;
import be.recipe.services.executor.GetPrescriptionStatusResult;
import be.recipe.services.executor.ListOpenPrescriptionsParam;
import be.recipe.services.executor.ListOpenPrescriptionsResult;
import be.recipe.services.executor.ListRelationsParam;
import be.recipe.services.executor.ListRelationsResult;
import be.recipe.services.executor.ListReservationsParam;
import be.recipe.services.executor.ListReservationsResult;
import be.recipe.services.executor.ListRidsHistoryParam;
import be.recipe.services.executor.ListRidsHistoryResult;
import be.recipe.services.executor.ListRidsInProcessParam;
import be.recipe.services.executor.ListRidsInProcessResult;
import be.recipe.services.executor.PutRidsInProcessParam;
import be.recipe.services.executor.PutRidsInProcessResult;

/**
 * The Interface ExecutorIntegrationModuleDevV4.
 */
public interface ExecutorIntegrationModuleDevV4 extends ExecutorIntegrationModuleV4 {

	/**
	 * Gets the {@link ListOpenPrescriptionsResult}.
	 *
	 * @param request
	 *            a {@link ListOpenPrescriptionsParam}
	 * @return the {@link ListOpenPrescriptionsResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	ListOpenPrescriptionsResult getData(ListOpenPrescriptionsParam request) throws IntegrationModuleException;

	/**
	 * Decrypt a {@link GetOpenPrescriptionForExecutor}.
	 *
	 * @param getOpenPrescriptionForExecutor
	 *            a the {@link GetOpenPrescriptionListParam}
	 * @return a {@link GetOpenPrescriptionForExecutor}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	GetOpenPrescriptionForExecutor decryptGetOpenPrescriptionForExecutor(
			final GetOpenPrescriptionForExecutor getOpenPrescriptionForExecutor) throws IntegrationModuleException;

	/**
	 * Gets the {@link GetPrescriptionStatusResult}.
	 *
	 * @param request
	 *            the {@link GetPrescriptionStatusParam}
	 * @return the {@link GetPrescriptionStatusResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	GetPrescriptionStatusResult getData(GetPrescriptionStatusParam request) throws IntegrationModuleException;

	/**
	 * Gets the {@link ListRidsHistoryResult}.
	 *
	 * @param request
	 *            the {@link ListRidsHistoryParam}
	 * @return the {@link ListPrescriptionHistoryParam}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	ListRidsHistoryResult getData(ListRidsHistoryParam request) throws IntegrationModuleException;

	/**
	 * Gets the {@link ListReservationsResult}.
	 *
	 * @param request
	 *            the {@link GetAllReservationsParam}
	 * @return the {@link ListReservationsResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	ListReservationsResult getData(ListReservationsParam request) throws IntegrationModuleException;

	/**
	 * Gets the {@link ListRidsInProcessResult}.
	 *
	 * @param request
	 *            the {@link ListRidsInProcessParam}
	 * @returnthe {@link ListRidsInProcessResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	ListRidsInProcessResult getData(ListRidsInProcessParam request) throws IntegrationModuleException;

	/**
	 * Puts the {@link PutRidsInProcessResult}.
	 *
	 * @param request
	 *            the {@link PutRidsInProcessParam}
	 * @returnthe {@link PutRidsInProcessResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	PutRidsInProcessResult putData(PutRidsInProcessParam request) throws IntegrationModuleException;

	/**
	 * Gets the {@link be.business.connector.recipe.executor.domain.GetPrescriptionForExecutorResult} and marks it as delivered.
	 *
	 * @param rid
	 *            the rid
	 * @return the {@link be.business.connector.recipe.executor.domain.GetPrescriptionForExecutorResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	be.business.connector.recipe.executor.domain.GetPrescriptionForExecutorResult getAndMarkAsDelivered(String rid)
			throws IntegrationModuleException;

	ListRelationsResult getData(ListRelationsParam patientRelationParam);

}