package be.business.connector.recipe.executor;

import static be.business.connector.recipe.utils.RidValidator.validateRid;

import java.io.IOException;
import java.math.BigInteger;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.apache.commons.collections.keyvalue.MultiKey;
import org.perf4j.aop.Profiled;

import com.sun.xml.ws.client.ClientTransportException;

import be.business.connector.common.ApplicationConfig;
import be.business.connector.common.StandaloneRequestorProvider;
import be.business.connector.core.domain.KgssIdentifierType;
import be.business.connector.core.exceptions.IntegrationModuleException;
import be.business.connector.core.handlers.InsurabilityHandler;
import be.business.connector.core.utils.Exceptionutils;
import be.business.connector.core.utils.I18nHelper;
import be.business.connector.core.utils.IOUtils;
import be.business.connector.core.utils.MarshallerHelper;
import be.business.connector.core.utils.PropertyHandler;
import be.business.connector.projects.common.utils.ValidationUtils;
import be.business.connector.recipe.executor.services.RecipeExecutorServiceDevV4Impl;
import be.business.connector.recipe.utils.RidValidator;
import be.ehealth.technicalconnector.service.kgss.domain.KeyResult;
import be.fgov.ehealth.etee.crypto.encrypt.EncryptionToken;
import be.recipe.services.core.Page;
import be.recipe.services.executor.CreateFeedback;
import be.recipe.services.executor.CreateFeedbackResponse;
import be.recipe.services.executor.CreateFeedbackResult;
import be.recipe.services.executor.GetOpenPrescriptionForExecutor;
import be.recipe.services.executor.GetPrescriptionForExecutor;
import be.recipe.services.executor.GetPrescriptionForExecutorResponse;
import be.recipe.services.executor.GetPrescriptionForExecutorResult;
import be.recipe.services.executor.GetPrescriptionForExecutorResultSealed;
import be.recipe.services.executor.GetPrescriptionStatus;
import be.recipe.services.executor.GetPrescriptionStatusParam;
import be.recipe.services.executor.GetPrescriptionStatusResponse;
import be.recipe.services.executor.GetPrescriptionStatusResult;
import be.recipe.services.executor.ListNotifications;
import be.recipe.services.executor.ListNotificationsItem;
import be.recipe.services.executor.ListNotificationsResponse;
import be.recipe.services.executor.ListNotificationsResult;
import be.recipe.services.executor.ListOpenPrescriptions;
import be.recipe.services.executor.ListOpenPrescriptionsParam;
import be.recipe.services.executor.ListOpenPrescriptionsResponse;
import be.recipe.services.executor.ListOpenPrescriptionsResult;
import be.recipe.services.executor.ListRelations;
import be.recipe.services.executor.ListRelationsParam;
import be.recipe.services.executor.ListRelationsResponse;
import be.recipe.services.executor.ListRelationsResult;
import be.recipe.services.executor.ListReservations;
import be.recipe.services.executor.ListReservationsParam;
import be.recipe.services.executor.ListReservationsResponse;
import be.recipe.services.executor.ListReservationsResult;
import be.recipe.services.executor.ListRidsHistory;
import be.recipe.services.executor.ListRidsHistoryParam;
import be.recipe.services.executor.ListRidsHistoryResponse;
import be.recipe.services.executor.ListRidsHistoryResult;
import be.recipe.services.executor.ListRidsInProcess;
import be.recipe.services.executor.ListRidsInProcessParam;
import be.recipe.services.executor.ListRidsInProcessResponse;
import be.recipe.services.executor.ListRidsInProcessResult;
import be.recipe.services.executor.MarkAsArchived;
import be.recipe.services.executor.MarkAsArchivedResponse;
import be.recipe.services.executor.MarkAsArchivedResult;
import be.recipe.services.executor.MarkAsDelivered;
import be.recipe.services.executor.MarkAsDeliveredResponse;
import be.recipe.services.executor.MarkAsDeliveredResult;
import be.recipe.services.executor.MarkAsUnDelivered;
import be.recipe.services.executor.MarkAsUnDeliveredResponse;
import be.recipe.services.executor.MarkAsUndeliveredResult;
import be.recipe.services.executor.PutRidsInProcess;
import be.recipe.services.executor.PutRidsInProcessParam;
import be.recipe.services.executor.PutRidsInProcessResponse;
import be.recipe.services.executor.PutRidsInProcessResult;
import be.recipe.services.executor.RevokePrescription;
import be.recipe.services.executor.RevokePrescriptionResponse;
import be.recipe.services.executor.RevokePrescriptionResult;
/**
 * The Class ExecutorIntegrationModuleDevV4Impl.
 */
public class ExecutorIntegrationModuleDevV4Impl extends ExecutorIntegrationModuleImpl implements ExecutorIntegrationModuleDevV4 {

	
	private static Map<MultiKey, byte[]> sessionMap = new HashMap<MultiKey, byte[]>();
	
	/**
	 * Instantiates a new executor integration module V 4 impl.
	 *
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	public ExecutorIntegrationModuleDevV4Impl() throws IntegrationModuleException {
		super();
	}

	/**
	 * {@inheritDoc}
	 */
	@Profiled(logFailuresSeparately = true, tag = "ExecutorIntegrationModuleV4#getData(GetOpenPrescriptionListParam)")
	@Override
	public ListOpenPrescriptionsResult getData(final ListOpenPrescriptionsParam param) throws IntegrationModuleException {
		ApplicationConfig.getInstance().assertValidPharmacySession();
		ValidationUtils.validatePatientId(param.getPatientId());
		ValidationUtils.validateMandateHolderId(param.getMandateHolderId(), true);
		final ListOpenPrescriptions getOpenPrescriptionList = getOpenPrescriptionList(param);
		try {
			try {
				final ListOpenPrescriptionsResponse response = RecipeExecutorServiceDevV4Impl.getInstance().listOpenPrescriptions(getOpenPrescriptionList);
				final ListOpenPrescriptionsResult result = unsealGetDataResponse(response);
				checkStatus(result);
				sessionMap.put(new MultiKey(StandaloneRequestorProvider.getRequestorIdInformation(), param.getPatientId()), result.getSession());
				return result;
			} catch (final ClientTransportException cte) {
				throw new IntegrationModuleException(I18nHelper.getLabel("error.connection.executor"), cte);
			}
		} catch (final Throwable t) {
			Exceptionutils.errorHandler(t, getOpenPrescriptionList.getMguid());
		}
		return null;
	}
	
	/**
	 * Gets the gets the open prescription list request.
	 *
	 * @param listOpenPrescriptionsParam
	 *            the data
	 * @return the gets the open prescription list request
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	private ListOpenPrescriptions getOpenPrescriptionList(final ListOpenPrescriptionsParam listOpenPrescriptionsParam)
			throws IntegrationModuleException {
		listOpenPrescriptionsParam.setSession(sessionMap.get(new MultiKey( StandaloneRequestorProvider.getRequestorIdInformation(), listOpenPrescriptionsParam.getPatientId())));
		final ListOpenPrescriptions request = new ListOpenPrescriptions();
		request.setListOpenPrescriptionsParamSealed(getSealedData(listOpenPrescriptionsParam));
		request.setProgramIdentification(PropertyHandler.getInstance().getProperty("programIdentification"));
		request.setMguid(UUID.randomUUID().toString());
		return request;
	}

	/**
	 * Gets the sealed data.
	 *
	 * @param listOpenPrescriptionsParam
	 *            the request
	 * @return the sealed data
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	private byte[] getSealedData(final ListOpenPrescriptionsParam listOpenPrescriptionsParam) throws IntegrationModuleException {
		listOpenPrescriptionsParam.setSymmKey(getSymmKey().getEncoded());
		return sealForRecipe(listOpenPrescriptionsParam, ListOpenPrescriptionsParam.class);
	}
	

	/**
	 * Unseal get data response.
	 *
	 * @param response
	 *            the get data response
	 * @return the gets the open prescription list response
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	private ListOpenPrescriptionsResult unsealGetDataResponse(final ListOpenPrescriptionsResponse response) throws IntegrationModuleException {
		final MarshallerHelper<ListOpenPrescriptionsResult, Object> marshaller = new MarshallerHelper<>(
				ListOpenPrescriptionsResult.class, Object.class);
		return marshaller.unsealWithSymmKey(response.getListOpenPrescriptionsResultSealed(), getSymmKey());
	}

	/**
	 * {@inheritDoc}
	 */
	@Profiled(logFailuresSeparately = true, tag = "0.ExecutorIntegrationModuleV4#getData(GetPrescriptionStatusParam)")
	@Override
	public GetPrescriptionStatusResult getData(final GetPrescriptionStatusParam param) throws IntegrationModuleException {
		RidValidator.validateRid(param.getRid());
		ApplicationConfig.getInstance().assertValidPharmacySession();
		final GetPrescriptionStatus getPrescriptionStatus = getPrescriptionStatus(param);
		try {
			try {
				final GetPrescriptionStatusResponse response = RecipeExecutorServiceDevV4Impl.getInstance().getPrescriptionStatus(getPrescriptionStatus);
				final GetPrescriptionStatusResult result = unsealGetPrescriptionStatusResponse(response);
				checkStatus(result);
				return result;
			} catch (final ClientTransportException cte) {
				throw new IntegrationModuleException(I18nHelper.getLabel("error.connection.executor"), cte);
			}
		} catch (final Throwable t) {
			Exceptionutils.errorHandler(t, getPrescriptionStatus.getMguid());
		}
		return null;
	}
	
	/**
	 * Gets the gets the prescription status request.
	 *
	 * @param getPrescriptionStatusParam
	 *            the data
	 * @return the gets the prescription status request
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	private GetPrescriptionStatus getPrescriptionStatus(final GetPrescriptionStatusParam getPrescriptionStatusParam)
			throws IntegrationModuleException {
		final GetPrescriptionStatus getPrescriptionStatus = new GetPrescriptionStatus();
		getPrescriptionStatus.setGetPrescriptionStatusParamSealed(getSealedData(getPrescriptionStatusParam));
		getPrescriptionStatus.setProgramIdentification(PropertyHandler.getInstance().getProperty("programIdentification"));
		getPrescriptionStatus.setMguid(UUID.randomUUID().toString());
		return getPrescriptionStatus;
	}

	/**
	 * Unseal get prescription status response.
	 *
	 * @param response
	 *            the get data response
	 * @return the gets the prescription status response
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	private GetPrescriptionStatusResult unsealGetPrescriptionStatusResponse(final GetPrescriptionStatusResponse response)
			throws IntegrationModuleException {
		final MarshallerHelper<GetPrescriptionStatusResult, Object> marshaller = new MarshallerHelper<>(GetPrescriptionStatusResult.class,
				Object.class);
		return marshaller.unsealWithSymmKey(response.getGetPrescriptionStatusResultSealed(), getSymmKey());
	}

	/**
	 * {@inheritDoc}
	 */
	@Profiled(logFailuresSeparately = true, tag = "0.ExecutorIntegrationModuleV4#getData(ListRidsHistoryParam)")
	@Override
	public ListRidsHistoryResult getData(final ListRidsHistoryParam param) throws IntegrationModuleException {
		ApplicationConfig.getInstance().assertValidPharmacySession();
		ValidationUtils.validatePatientId(param.getPatientId());
		final ListRidsHistory listPrescriptionHistory = getListRidsHistory(param);
		try {
			try {
				final ListRidsHistoryResponse response = RecipeExecutorServiceDevV4Impl.getInstance().listRidsHistory(listPrescriptionHistory);
				final ListRidsHistoryResult result = unsealListRidsHistoryResponse(response);
				checkStatus(result);
				sessionMap.put(new MultiKey(param.getExecutorId(), param.getPatientId()), result.getSession());
				return result;
			} catch (final ClientTransportException cte) {
				throw new IntegrationModuleException(I18nHelper.getLabel("error.connection.executor"), cte);
			}
		} catch (final Throwable t) {
			Exceptionutils.errorHandler(t, listPrescriptionHistory.getMguid());
		}
		return null;
	}

	private ListRidsHistory getListRidsHistory(final ListRidsHistoryParam listRidsHistoryParam)
			throws IntegrationModuleException {
		listRidsHistoryParam.setSession(sessionMap.get(new MultiKey( listRidsHistoryParam.getExecutorId(), listRidsHistoryParam.getPatientId())));
		final ListRidsHistory listPrescriptionHistory = new ListRidsHistory();
		listPrescriptionHistory.setListRidsHistoryParamSealed(getSealedData(listRidsHistoryParam));
		listPrescriptionHistory.setProgramIdentification(PropertyHandler.getInstance().getProperty("programIdentification"));
		listPrescriptionHistory.setMguid(UUID.randomUUID().toString());
		return listPrescriptionHistory;
	}
	
	/**
	 * Gets the sealed data.
	 *
	 * @param listRidsHistoryParam
	 *            the request
	 * @return the sealed data
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	private byte[] getSealedData(final ListRidsHistoryParam listRidsHistoryParam) throws IntegrationModuleException {
		listRidsHistoryParam.setSymmKey(getSymmKey().getEncoded());
		return sealForRecipe(listRidsHistoryParam, ListRidsHistoryParam.class);
	}
	
	/**
	 * Unseal list prescription history response.
	 *
	 * @param response
	 *            the get data response
	 * @return the list prescription history response
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	private ListRidsHistoryResult unsealListRidsHistoryResponse(final ListRidsHistoryResponse response) {
		final MarshallerHelper<ListRidsHistoryResult, Object> marshaller = new MarshallerHelper<>(
				ListRidsHistoryResult.class, Object.class);
		return marshaller.unsealWithSymmKey(response.getListRidsHistoryResultSealed(), getSymmKey());
	}

	/**
	 * {@inheritDoc}
	 */
	@Profiled(logFailuresSeparately = true, tag = "ExecutorIntegrationModuleV4#getPrescription")
	@Override
	public be.business.connector.recipe.executor.domain.GetPrescriptionForExecutorResult getPrescription(final String rid)
			throws IntegrationModuleException {
		RidValidator.validateRid(rid);
		ApplicationConfig.getInstance().assertValidPharmacySession();
		InsurabilityHandler.setInsurability(null);
		InsurabilityHandler.setMessageId(null);

		final String guid = UUID.randomUUID().toString();
		try {
			// create get prescription request
			final byte[] sealedGetPrescriptionForExecutorParam = getSealedGetPrescriptionForExecutorParam(rid);

			final GetPrescriptionForExecutor request = new GetPrescriptionForExecutor();
			request.setDisablePatientInsurabilityCheckParam(
					Boolean.parseBoolean(getPropertyHandler().getProperty("patient.insurability.disable")));
			request.setGetPrescriptionForExecutorParamSealed(sealedGetPrescriptionForExecutorParam);
			request.setProgramIdentification(PropertyHandler.getInstance().getProperty("programIdentification"));
			request.setMguid(guid);
			GetPrescriptionForExecutorResponse response = null;
			try {
				response = RecipeExecutorServiceDevV4Impl.getInstance().getPrescriptionForExecutor(request);
			} catch (final ClientTransportException cte) {
				throw new IntegrationModuleException(I18nHelper.getLabel("error.connection.executor"), cte);
			}

			final GetPrescriptionForExecutorResult executorResult = response.getGetPrescriptionForExecutorResult();
			final be.business.connector.recipe.executor.domain.GetPrescriptionForExecutorResult finalResult = createGetPrescriptionForExecutorResult(
					executorResult.getGetPrescriptionForExecutorResultSealed());
			checkStatus(finalResult);
			getPrescriptionCache().put(rid, finalResult);
			return finalResult;
		} catch (final Throwable t) {
			Exceptionutils.errorHandler(t, guid);
		}
		return null;
	}
	
	protected be.business.connector.recipe.executor.domain.GetPrescriptionForExecutorResult createGetPrescriptionForExecutorResult(
			final byte[] securedContent) throws IntegrationModuleException {
		final MarshallerHelper<GetPrescriptionForExecutorResultSealed, Object> marshaller = new MarshallerHelper<>(
				GetPrescriptionForExecutorResultSealed.class, Object.class);
		final String requestorIdInformation = StandaloneRequestorProvider.getRequestorIdInformation();
		final String requestorTypeInformation = StandaloneRequestorProvider.getRequestorTypeInformation();
		final GetPrescriptionForExecutorResultSealed sealedResult = marshaller.unsealWithSymmKey(securedContent, getSymmKey());
		
							
		
		final be.business.connector.recipe.executor.domain.GetPrescriptionForExecutorResult finalResult = new be.business.connector.recipe.executor.domain.GetPrescriptionForExecutorResult(
				sealedResult);
		finalResult.setStatus(sealedResult.getStatus());
		//Only do checkStatus in dev
		checkStatus(finalResult);
		finalResult.setSealedContent(sealedResult.getPrescription());
		
		final KeyResult key = getKeyFromKgss(sealedResult.getEncryptionKeyId(),
				getEtkHelper().getEtks(KgssIdentifierType.NIHII_PHARMACY, requestorIdInformation).get(0).getEncoded());
		final byte[] unsealedPrescription = unsealWithSymKey(sealedResult, key, requestorIdInformation, requestorTypeInformation);
		
		finalResult.setPrescription(unsealedPrescription);
		finalResult.setEncryptionKey(key.getSecretKey().getEncoded());
		finalResult.setInsurabilityResponse(InsurabilityHandler.getInsurability());
		finalResult.setMessageId(InsurabilityHandler.getMessageId());
		return finalResult;
	}

	/**
	 * {@inheritDoc}
	 */
	@Profiled(logFailuresSeparately = true, tag = "0.ExecutorIntegrationModuleV4#decryptGetOpenPrescriptionForExecutor")
	@Override
	public GetOpenPrescriptionForExecutor decryptGetOpenPrescriptionForExecutor(
			final GetOpenPrescriptionForExecutor gopfe) throws IntegrationModuleException {
		try {
			final String requestorIdInformation = StandaloneRequestorProvider.getRequestorIdInformation();
			final KeyResult key = getKeyFromKgss(gopfe.getEncryptionKeyId(),
					getEtkHelper().getEtks(KgssIdentifierType.NIHII_PHARMACY, requestorIdInformation).get(0).getEncoded());
			final byte[] unsealedPrescription = unsealPrescriptionForUnknown(key, gopfe.getPrescription());
			final GetOpenPrescriptionForExecutor finalResult = new GetOpenPrescriptionForExecutor();
			finalResult.setCreationDate(gopfe.getCreationDate());
			finalResult.setEncryptionKeyId(gopfe.getEncryptionKeyId());
			finalResult.setFeedbackAllowed(gopfe.isFeedbackAllowed());
			finalResult.setId(gopfe.getId());
			finalResult.setPatientId(gopfe.getPatientId());
			finalResult.setPrescriberId(gopfe.getPrescriberId());
			finalResult.setPrescription(IOUtils.decompress(unsealedPrescription));
			finalResult.setPrescriptionType(gopfe.getPrescriptionType());
			finalResult.setRid(gopfe.getRid());
			finalResult.setStatus(gopfe.getStatus());
			finalResult.setExpirationDate(gopfe.getExpirationDate());
			return finalResult;
		} catch (final IOException ex) {
			Exceptionutils.errorHandler(ex, "error.data.uncompression");
		}
		return null;
	}

	/**
	 * {@inheritDoc}
	 */
	@Profiled(logFailuresSeparately = true, tag = "0.ExecutorIntegrationModuleV4#markAsArchived")
	@Override
	public void markAsArchived(final String rid) throws IntegrationModuleException {
		RidValidator.validateRid(rid);
		ApplicationConfig.getInstance().assertValidPharmacySession();
		final MarkAsArchived request = new MarkAsArchived();
		request.setMguid(UUID.randomUUID().toString());
		try {

			// create param
			final byte[] sealedMarkAsArchivedParam = getSealedMarkAsArchivedParam(rid);
			// create request
			request.setMarkAsArchivedParamSealed(sealedMarkAsArchivedParam);
			request.setProgramIdentification(PropertyHandler.getInstance().getProperty("programIdentification"));

			// call WS
			MarkAsArchivedResponse response;
			try {
				response = RecipeExecutorServiceDevV4Impl.getInstance().markAsArchived(request);
				final MarshallerHelper<MarkAsArchivedResult, Object> marshaller = new MarshallerHelper<>(MarkAsArchivedResult.class,
						Object.class);
				final MarkAsArchivedResult result = (MarkAsArchivedResult) marshaller
						.unsealWithSymmKey(response.getMarkAsArchivedResultSealed(), getSymmKey());
				checkStatus(result);
			} catch (final ClientTransportException cte) {
				throw new IntegrationModuleException(I18nHelper.getLabel("error.connection.executor"), cte);
			}
		} catch (final Throwable t) {
			Exceptionutils.errorHandler(t, request.getMguid());
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Profiled(logFailuresSeparately = true, tag = "0.ExecutorIntegrationModuleV4#markAsDelivered")
	@Override
	public void markAsDelivered(final String rid) throws IntegrationModuleException {
		RidValidator.validateRid(rid);
		ApplicationConfig.getInstance().assertValidPharmacySession();
		final MarkAsDelivered request = new MarkAsDelivered();
		request.setMguid(UUID.randomUUID().toString());
		try {
			final byte[] sealedMarkAsDeliveredParam = getSealedMarkAsDeliveredParam(rid);

			request.setMarkAsDeliveredParamSealed(sealedMarkAsDeliveredParam);
			request.setProgramIdentification(PropertyHandler.getInstance().getProperty("programIdentification"));
			MarkAsDeliveredResponse response = null;
			try {
				response = RecipeExecutorServiceDevV4Impl.getInstance().markAsDelivered(request);
				final MarshallerHelper<MarkAsDeliveredResult, Object> marshaller = new MarshallerHelper<>(MarkAsDeliveredResult.class,
						Object.class);
				final MarkAsDeliveredResult result = (MarkAsDeliveredResult) marshaller
						.unsealWithSymmKey(response.getMarkAsDeliveredResultSealed(), getSymmKey());
				checkStatus(result);
			} catch (final ClientTransportException cte) {
				throw new IntegrationModuleException(I18nHelper.getLabel("error.connection.executor"), cte);
			}
		} catch (final Throwable t) {
			Exceptionutils.errorHandler(t, request.getMguid());
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Profiled(logFailuresSeparately = true, tag = "0.ExecutorIntegrationModuleV4#markAsUnDelivered")
	@Override
	public void markAsUndelivered(final String rid) throws IntegrationModuleException {
		RidValidator.validateRid(rid);
		ApplicationConfig.getInstance().assertValidPharmacySession();
		final MarkAsUnDelivered request = new MarkAsUnDelivered();
		request.setProgramIdentification(PropertyHandler.getInstance().getProperty("programIdentification"));
		request.setMguid(UUID.randomUUID().toString());
		try {
			final byte[] sealedMarkAsUnDeliveredParam = getSealedMarkAsUnDeliveredParam(rid);
			request.setMarkAsUndeliveredParamSealed(sealedMarkAsUnDeliveredParam);


			MarkAsUnDeliveredResponse response = null;
			try {
				response = RecipeExecutorServiceDevV4Impl.getInstance().markAsUnDelivered(request);
				final MarshallerHelper<MarkAsUndeliveredResult, Object> marshaller = new MarshallerHelper<>(MarkAsUndeliveredResult.class,
						Object.class);
				final MarkAsUndeliveredResult result = (MarkAsUndeliveredResult) marshaller
						.unsealWithSymmKey(response.getMarkAsUnDeliveredResultSealed(), getSymmKey());
				checkStatus(result);
			} catch (final ClientTransportException cte) {
				throw new IntegrationModuleException(I18nHelper.getLabel("error.connection.executor"), cte);
			}
		} catch (final Throwable t) {
			Exceptionutils.errorHandler(t, request.getMguid());
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Profiled(logFailuresSeparately = true, tag = "0.ExecutorIntegrationModuleV4#revokePrescription")
	@Override
	public void revokePrescription(final String rid, final String reason) throws IntegrationModuleException {
		RidValidator.validateRid(rid);
		ApplicationConfig.getInstance().assertValidPharmacySession();
		final RevokePrescription request = new RevokePrescription();
		try {
			final byte[] sealedRevokePrescriptionParam = getSealedRevokePrescriptionParam(rid, reason);

			request.setRevokePrescriptionParamSealed(sealedRevokePrescriptionParam);
			request.setProgramIdentification(PropertyHandler.getInstance().getProperty("programIdentification"));
			request.setMguid(UUID.randomUUID().toString());

			RevokePrescriptionResponse response = null;
			try {
				response = RecipeExecutorServiceDevV4Impl.getInstance().revokePrescriptionForExecutor(request);
				final MarshallerHelper<RevokePrescriptionResult, Object> marshaller = new MarshallerHelper<>(RevokePrescriptionResult.class,
						Object.class);
				final RevokePrescriptionResult result = (RevokePrescriptionResult) marshaller
						.unsealWithSymmKey(response.getRevokePrescriptionResultSealed(), getSymmKey());
				checkStatus(result);
			} catch (final ClientTransportException cte) {
				throw new IntegrationModuleException(I18nHelper.getLabel("error.connection.executor"), cte);
			}

		} catch (final Throwable t) {
			Exceptionutils.errorHandler(t, request.getMguid());
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Profiled(logFailuresSeparately = true, tag = "0.ExecutorIntegrationModuleV4#listNotifications")
	@Override
	public List<ListNotificationsItem> listNotifications(final boolean readFlag) throws IntegrationModuleException {
		ApplicationConfig.getInstance().assertValidPharmacySession();
		final ListNotifications request = new ListNotifications();
		request.setMguid(UUID.randomUUID().toString());

		try {
			final byte[] sealedListNotificationsParam = getSealedListNotificationsParam(readFlag);

			request.setListNotificationsParamSealed(sealedListNotificationsParam);
			request.setProgramIdentification(PropertyHandler.getInstance().getProperty("programIdentification"));

			ListNotificationsResponse response = null;
			try {
				response = RecipeExecutorServiceDevV4Impl.getInstance().listNotifications(request);
				final MarshallerHelper<ListNotificationsResult, Object> marshaller = new MarshallerHelper<>(ListNotificationsResult.class,
						Object.class);
				final ListNotificationsResult result = (ListNotificationsResult) marshaller
						.unsealWithSymmKey(response.getListNotificationsResultSealed(), getSymmKey());
				checkStatus(result);
			} catch (final ClientTransportException cte) {
				throw new IntegrationModuleException(I18nHelper.getLabel("error.connection.executor"), cte);
			}

			final byte[] securedContent = response.getListNotificationsResultSealed();
			return createListNotificationItems(securedContent);
		} catch (final Throwable t) {
			Exceptionutils.errorHandler(t, request.getMguid());
		}
		return null;
	}

	/**
	 * {@inheritDoc}
	 */
	@Profiled(logFailuresSeparately = true, tag = "0.ExecutorIntegrationModuleV4#createFeedback")
	@Override
	public void createFeedback(final String prescriberId, final String rid, final byte[] feedbackText) throws IntegrationModuleException {
		validateRid(rid);
		ApplicationConfig.getInstance().assertValidPharmacySession();
		final CreateFeedback request = new CreateFeedback();
		request.setMguid(UUID.randomUUID().toString());
		try {
			getKmehrHelper().assertValidFeedback(feedbackText);
			final List<EncryptionToken> etkRecipients = getEtkHelper().getEtks(KgssIdentifierType.NIHII, prescriberId);

			for (int i = 0; i < etkRecipients.size(); i++) {
				final EncryptionToken etkRecipient = etkRecipients.get(i);
				final byte[] sealedCreateFeedbackParam = getSealedCreateFeedbackParam(feedbackText, etkRecipient, rid, prescriberId);

				request.setCreateFeedbackParamSealed(sealedCreateFeedbackParam);
				request.setProgramIdentification(PropertyHandler.getInstance().getProperty("programIdentification"));

				CreateFeedbackResponse response;
				try {
					response = RecipeExecutorServiceDevV4Impl.getInstance().createFeedback(request);

					final MarshallerHelper<CreateFeedbackResult, Object> marshaller1 = new MarshallerHelper<>(CreateFeedbackResult.class,
							Object.class);
					final CreateFeedbackResult result = marshaller1.unsealWithSymmKey(response.getCreateFeedbackResultSealed(),
							getSymmKey());
					checkStatus(result);
				} catch (final ClientTransportException cte) {
					throw new IntegrationModuleException(I18nHelper.getLabel("error.connection.executor"), cte);
				}

			}
		} catch (final Throwable t) {
			Exceptionutils.errorHandler(t, request.getMguid());
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Profiled(logFailuresSeparately = true, tag = "0.ExecutorIntegrationModuleV4#getData(ListReservationsParam)")
	@Override
	public ListReservationsResult getData(final ListReservationsParam param) throws IntegrationModuleException {
		ApplicationConfig.getInstance().assertValidPharmacySession();
		final Calendar lastSyncDate = Calendar.getInstance();
		try {
			try {
				final ListReservationsResult completeResult = new ListReservationsResult();
				ListReservationsResult result;
				do {
					BigInteger currentPage;
					if (param.getPage() == null) {
						param.setPage(new Page());
					}
					if (param.getPage() != null && param.getPage().getPageNumber() == null) {
						currentPage = BigInteger.valueOf(0);
					}else {
						currentPage = param.getPage().getPageNumber();
					}
					param.getPage().setPageNumber(currentPage);
					final ListReservations request = getListReservations(param);
					final ListReservationsResponse response = RecipeExecutorServiceDevV4Impl.getInstance().listReservations(request);
					result = unsealListReservationsResponse(response);
					checkStatus(result);
					completeResult.getItems().addAll(result.getItems());
					param.getPage().setPageNumber(BigInteger.valueOf(currentPage.intValue() + 1));
				} while (result != null && result.isHasMoreResults() != null && result.isHasMoreResults().booleanValue());
				
				writeReservationsOnDisk(param, completeResult, lastSyncDate);
				return completeResult;
				
			} catch (final ClientTransportException cte) {
				throw new IntegrationModuleException(I18nHelper.getLabel("error.connection.executor"), cte);
			}
		} catch (final Throwable t) {
			Exceptionutils.errorHandler(t);
		}
		return null;
	}
	
	/**
	 * Gets the all reservations request.
	 *
	 * @param param
	 *            the data
	 * @return the all reservations request
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	private ListReservations getListReservations(final ListReservationsParam param) throws IntegrationModuleException {
		param.setSymmKey(getSymmKey().getEncoded());
		param.setStartDate(param.getStartDate() == null ?readLastDateToDisk(): param.getStartDate());
		final ListReservations listReservations = new ListReservations();
		listReservations.setListReservationsParamSealed(getSealedData(param));
		listReservations.setProgramIdentification(PropertyHandler.getInstance().getProperty("programIdentification"));
		listReservations.setMguid(UUID.randomUUID().toString());
		
		return listReservations;
	}
	
	/**
	 * Gets the sealed data.
	 *
	 * @param listReservationsParam
	 *            the request
	 * @return the sealed data
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	private byte[] getSealedData(final ListReservationsParam listReservationsParam) throws IntegrationModuleException {
		return sealForRecipe(listReservationsParam, ListReservationsParam.class);
	}

	/**
	 * Unseal get all reservations response.
	 * 
	 * @param response
	 *            the get data response
	 * @return the gets the all reservations response
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	private ListReservationsResult unsealListReservationsResponse(final ListReservationsResponse response) {
		final MarshallerHelper<ListReservationsResult, Object> marshaller = new MarshallerHelper<>(ListReservationsResult.class,
				Object.class);
		final ListReservationsResult result = marshaller.unsealWithSymmKey(response.getListReservationsResultSealed(), getSymmKey());
		return result;
	}

	/**
	 * {@inheritDoc}
	 */
	@Profiled(logFailuresSeparately = true, tag = "0.ExecutorIntegrationModuleV4#getData(ListRidsInProcessParam)")
	@Override
	public ListRidsInProcessResult getData(final ListRidsInProcessParam param) throws IntegrationModuleException {
		ApplicationConfig.getInstance().assertValidPharmacySession();
		final ListRidsInProcess request = getListRidsInProcess(param);
		try {
			try {
				final ListRidsInProcessResponse response = RecipeExecutorServiceDevV4Impl.getInstance().listRidsInProcess(request);
				final ListRidsInProcessResult result = unsealListRidsInProcessResponse(response);
				checkStatus(result);
				return result;
			} catch (final ClientTransportException cte) {
				throw new IntegrationModuleException(I18nHelper.getLabel("error.connection.executor"), cte);
			}
		} catch (final Throwable t) {
			Exceptionutils.errorHandler(t, request.getMguid());
		}
		return null;
	}
	
	/**
	 * Gets the all ridsi N process request.
	 *
	 * @param listRidsInProcessParam
	 *            the data
	 * @return the all ridsi N process request
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	private ListRidsInProcess getListRidsInProcess(final ListRidsInProcessParam listRidsInProcessParam) throws IntegrationModuleException {
		//Update param with symmKey
		listRidsInProcessParam.setSymmKey(getSymmKey().getEncoded());
		
		//Create request object
		final ListRidsInProcess getAllRidInProcess = new ListRidsInProcess();
		getAllRidInProcess.setListRidsInProcessParamSealed(getSealedData(listRidsInProcessParam));
		getAllRidInProcess.setProgramIdentification(PropertyHandler.getInstance().getProperty("programIdentification"));
		getAllRidInProcess.setMguid(UUID.randomUUID().toString());
		
		return getAllRidInProcess;
	}


	/**
	 * {@inheritDoc}
	 */
	@Profiled(logFailuresSeparately = true, tag = "0.ExecutorIntegrationModuleV4#putData(PutAllRidInProcessParam)")
	@Override
	public PutRidsInProcessResult putData(final PutRidsInProcessParam param) throws IntegrationModuleException {
		ApplicationConfig.getInstance().assertValidPharmacySession();
		final PutRidsInProcess request = getPutAllRidInProcess(param);
		try {
			try {
				final PutRidsInProcessResponse response = RecipeExecutorServiceDevV4Impl.getInstance().putRidsInProcess(request);
				final PutRidsInProcessResult result = unsealPutRidsInProcessResponse(response);
				checkStatus(result);
				return result;
			} catch (final ClientTransportException cte) {
				throw new IntegrationModuleException(I18nHelper.getLabel("error.connection.executor"), cte);
			}
		} catch (final Throwable t) {
			Exceptionutils.errorHandler(t, request.getMguid());
		}
		return null;
	}
	
	/**
	 * Put all ridsi N process request.
	 *
	 * @param putRidsInProcessParam
	 *            the data
	 * @return the put data
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	private PutRidsInProcess getPutAllRidInProcess(final PutRidsInProcessParam putRidsInProcessParam) throws IntegrationModuleException {
		//Update param with symmKey
		putRidsInProcessParam.setSymmKey(getSymmKey().getEncoded());

		//Create request object
		final PutRidsInProcess putAllRidInProcess = new PutRidsInProcess();
		putAllRidInProcess.setPutRidsInProcessParamSealed(getSealedData(putRidsInProcessParam));
		putAllRidInProcess.setProgramIdentification(PropertyHandler.getInstance().getProperty("programIdentification"));
		putAllRidInProcess.setMguid(UUID.randomUUID().toString());
		
		return putAllRidInProcess;
	}
	
	/**
	 * Gets the sealed data.
	 *
	 * @param putRidsInProcessParam
	 *            the request
	 * @return the sealed data
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	private byte[] getSealedData(final PutRidsInProcessParam putRidsInProcessParam) throws IntegrationModuleException {
		return sealForRecipe(putRidsInProcessParam, PutRidsInProcessParam.class);
	}
	
	/**
	 * Lists the sealed data.
	 *
	 * @param litRidsInProcessParam
	 *            the request
	 * @return the sealed data
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	private byte[] getSealedData(final ListRidsInProcessParam litRidsInProcessParam) throws IntegrationModuleException {
		return sealForRecipe(litRidsInProcessParam, ListRidsInProcessParam.class);
	}

	/**
	 * Unseal put all rid in process response.
	 *
	 * @param response
	 *            the get data response
	 * @return the put all rid in process response
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	private PutRidsInProcessResult unsealPutRidsInProcessResponse(final PutRidsInProcessResponse response) {
		final MarshallerHelper<PutRidsInProcessResult, Object> marshaller = new MarshallerHelper<>(PutRidsInProcessResult.class,
				Object.class);
		final PutRidsInProcessResult result = marshaller.unsealWithSymmKey(response.getPutRidsInProcessResultSealed(), getSymmKey());
		return result;
	}

	/**
	 * Unseal get all rid in process response.
	 *
	 * @param getDataResponse
	 *            the get data response
	 * @return the gets the all rid in process result
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	private ListRidsInProcessResult unsealListRidsInProcessResponse(final ListRidsInProcessResponse response) {
		final MarshallerHelper<ListRidsInProcessResult, Object> marshaller = new MarshallerHelper<>(ListRidsInProcessResult.class,
				Object.class);
		final ListRidsInProcessResult result = marshaller.unsealWithSymmKey(response.getListRidsInProcessResultSealed(), getSymmKey());
		return result;
	}

	/**
	 * {@inheritDoc}
	 * @throws IntegrationModuleException 
	 */
	@Profiled(logFailuresSeparately = true, tag = "0.ExecutorIntegrationModuleV4#getAndMarkAsDelivered")
	@Override
	public be.business.connector.recipe.executor.domain.GetPrescriptionForExecutorResult getAndMarkAsDelivered(final String rid) throws IntegrationModuleException {
		try {
			final be.business.connector.recipe.executor.domain.GetPrescriptionForExecutorResult getPrescriptionForExecutorResult;
			if (getPrescriptionCache().containsKey(rid)) {
				getPrescriptionForExecutorResult = getPrescriptionCache().get(rid);
			} else {
				getPrescriptionForExecutorResult = getPrescription(rid);
			}
			markAsDelivered(rid);
			markAsArchived(rid);
			return getPrescriptionForExecutorResult;
		} catch (final Exception t) {
			Exceptionutils.errorHandler(t);
		} return null;
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public ListRelationsResult getData(final ListRelationsParam patientRelationParam) throws IntegrationModuleException {
		ApplicationConfig.getInstance().assertValidPharmacySession();
		final ListRelations request = getListPatientRelation(patientRelationParam);
		try {
			try {
				final ListRelationsResponse dataResponse = RecipeExecutorServiceDevV4Impl.getInstance().listRelations(request);
				final ListRelationsResult unsealedResponse = unsealListRelationsResponse(dataResponse);
				checkStatus(unsealedResponse);
				sessionMap.put(new MultiKey(patientRelationParam.getExecutorId(), patientRelationParam.getPatientId()), unsealedResponse.getSession());
				return unsealedResponse;
			} catch (final ClientTransportException cte) {
				throw new IntegrationModuleException(I18nHelper.getLabel("error.connection.executor"), cte);
			}
		} catch (final Throwable t) {
			Exceptionutils.errorHandler(t, request.getMguid());
		}
		return null;
	}
	
	/**
	 * Creates a {@link ListRelationsParam} request.
	 *
	 * @param data            the data
	 * @return the patient relation
	 * @throws IntegrationModuleException             the integration module exception
	 */
	private ListRelations getListPatientRelation(final ListRelationsParam data) throws IntegrationModuleException {
		data.setSymmKey(getSymmKey().getEncoded());
		final ListRelations patientRelation = new ListRelations();
		patientRelation.setListRelationsParamSealed(getSealedData(data));
		patientRelation.setProgramIdentification(PropertyHandler.getInstance().getProperty("programIdentification"));
		patientRelation.setMguid(UUID.randomUUID().toString());
		return patientRelation;
	}
	
	/**
	 * Gets the sealed data.
	 *
	 * @param data
	 *            the data
	 * @return the sealed data
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	private byte[] getSealedData(final ListRelationsParam data) throws IntegrationModuleException {
		data.setSymmKey(getSymmKey().getEncoded());
		return sealForRecipe(data, ListRelationsParam.class);
	}
	
	/**
	 * Unseal list relations response.
	 *
	 * @param response the response
	 * @return the list relations result
	 */
	private ListRelationsResult unsealListRelationsResponse(final ListRelationsResponse response) {
		final MarshallerHelper<ListRelationsResult, Object> marshaller = new MarshallerHelper<>(ListRelationsResult.class,
				Object.class);
		final ListRelationsResult result = marshaller.unsealWithSymmKey(response.getListRelationsResultSealed(), getSymmKey());
		return result;
	}

	/**
	 * Mark as archived async.
	 *
	 * @param rid
	 *            the rid
	 */
	private void markAsDeliveredAsync(final String rid) {
		new Thread(new Runnable() {
			public void run() {
				try {
					markAsDelivered(rid);
				} catch (final IntegrationModuleException e) {
					throw new RuntimeException(e);
				}
			}
		}).start();
	}
	
	/**
	 * Mark as archived async.
	 *
	 * @param rid
	 *            the rid
	 */
	private void markAsArchivedAsync(final String rid) {
		new Thread(new Runnable() {
			public void run() {
				try {
					markAsArchived(rid);
				} catch (final IntegrationModuleException e) {
					throw new RuntimeException(e);
				}
			}
		}).start();
	}
}