package be.business.connector.recipe.executor;

import be.business.connector.core.exceptions.IntegrationModuleException;
import be.business.connector.recipe.executor.domain.GetPrescriptionForExecutorResult;
import be.recipe.services.executor.GetOpenPrescriptionForExecutor;
import be.recipe.services.executor.GetPrescriptionStatusParam;
import be.recipe.services.executor.GetPrescriptionStatusResult;
import be.recipe.services.executor.ListOpenPrescriptionsParam;
import be.recipe.services.executor.ListOpenPrescriptionsResult;
import be.recipe.services.executor.ListRelationsParam;
import be.recipe.services.executor.ListRelationsResult;
import be.recipe.services.executor.ListReservationsParam;
import be.recipe.services.executor.ListReservationsResult;
import be.recipe.services.executor.ListRidsHistoryParam;
import be.recipe.services.executor.ListRidsHistoryResult;
import be.recipe.services.executor.ListRidsInProcessParam;
import be.recipe.services.executor.ListRidsInProcessResult;
import be.recipe.services.executor.PutRidsInProcessParam;
import be.recipe.services.executor.PutRidsInProcessResult;

/**
 * The Interface ExecutorIntegrationModuleV4.
 */
public interface ExecutorIntegrationModuleV4 extends ExecutorIntegrationModule {

	/**
	 * Gets the {@link ListOpenPrescriptionsResult}.
	 *
	 * @param request
	 *            a {@link ListOpenPrescriptionsParam}
	 * @return the {@link ListOpenPrescriptionsResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	ListOpenPrescriptionsResult getData(ListOpenPrescriptionsParam request) throws IntegrationModuleException;
	
	/**
	 * Gets the {@link GetPrescriptionStatusResult}.
	 *
	 * @param request
	 *            the {@link GetPrescriptionStatusParam}
	 * @return the {@link GetPrescriptionStatusResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	GetPrescriptionStatusResult getData(GetPrescriptionStatusParam gpsp);

	/**
	 * Gets the {@link ListRidsHistoryResult}.
	 *
	 * @param request
	 *            the {@link ListRidsHistoryParam}
	 * @return the {@link ListPrescriptionHistoryParam}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	ListRidsHistoryResult getData(ListRidsHistoryParam request);

	/**
	 * Gets the {@link ListReservationsResult}.
	 *
	 * @param request
	 *            the {@link GetAllReservationsParam}
	 * @return the {@link ListReservationsResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	ListReservationsResult getData(ListReservationsParam request);

	/**
	 * Gets the {@link ListRidsInProcessResult}.
	 *
	 * @param request
	 *            the {@link ListRidsInProcessParam}
	 * @returnthe {@link ListRidsInProcessResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	ListRidsInProcessResult getData(ListRidsInProcessParam request);

	/**
	 * Puts the {@link PutRidsInProcessResult}.
	 *
	 * @param request
	 *            the {@link PutRidsInProcessParam}
	 * @return the {@link PutRidsInProcessResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	PutRidsInProcessResult putData(PutRidsInProcessParam request);

	/**
	 * Gets the {@link be.business.connector.recipe.executor.domain.GetPrescriptionForExecutorResult} and marks it as delivered.
	 *
	 * @param rid
	 *            the rid
	 * @return the {@link be.business.connector.recipe.executor.domain.GetPrescriptionForExecutorResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	GetPrescriptionForExecutorResult getAndMarkAsDelivered(String rid);

	/**
	 * Gets the {@link ListRelationsResult}.
	 *
	 * @param request
	 *            the {@link ListRelationsParam}
	 * @return the {@link ListRelationsResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	ListRelationsResult getData(ListRelationsParam patientRelationParam);

	/**
	 * Decrypt a {@link GetOpenPrescriptionForExecutor}.
	 *
	 * @param getOpenPrescriptionForExecutor
	 *            a the {@link GetOpenPrescriptionListParam}
	 * @return a {@link GetOpenPrescriptionForExecutor}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	GetOpenPrescriptionForExecutor decryptGetOpenPrescriptionForExecutor(
			final GetOpenPrescriptionForExecutor getOpenPrescriptionForExecutor) throws IntegrationModuleException;

}