package be.business.connector.recipe.patient;

import be.business.connector.core.exceptions.IntegrationModuleException;
import be.recipe.services.patient.GetPrescriptionForPatientResult;
import be.recipe.services.patient.ListPatientPrescriptionsResult;

/**
 * The Interface PatientIntegrationModule.
 */
public interface PatientIntegrationModule {

	/**
	 * Gets the prescription.
	 *
	 * @param rid
	 *            the rid
	 * @return the prescription
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	GetPrescriptionForPatientResult getPrescription(String rid) throws IntegrationModuleException;


	/**
	 * Revoke prescription.
	 *
	 * @param rid
	 *            the rid
	 * @param reason
	 *            the reason
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	void revokePrescription(String rid, String reason) throws IntegrationModuleException;

	/**
	 * List open prescription.
	 *
	 * @return the {@link ListOpenPrescriptionsResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	ListPatientPrescriptionsResult listOpenPrescriptions() throws IntegrationModuleException;

	/**
	 * Update feedback flag.
	 *
	 * @param rid
	 *            the rid
	 * @param feedbackAllowed
	 *            the feedback allowed
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	void updateFeedbackFlag(String rid, boolean feedbackAllowed) throws IntegrationModuleException;

}
