package be.business.connector.recipe.patient;

import be.business.connector.core.exceptions.IntegrationModuleException;
import be.recipe.services.executor.ListOpenPrescriptionsParam;
import be.recipe.services.patient.CreateRelationParam;
import be.recipe.services.patient.CreateRelationResult;
import be.recipe.services.patient.CreateReservationParam;
import be.recipe.services.patient.CreateReservationResult;
import be.recipe.services.patient.GetPrescriptionStatusParam;
import be.recipe.services.patient.GetPrescriptionStatusResult;
import be.recipe.services.patient.GetVisionParam;
import be.recipe.services.patient.GetVisionResult;
import be.recipe.services.patient.ListOpenRidsParam;
import be.recipe.services.patient.ListOpenRidsResult;
import be.recipe.services.patient.ListPatientPrescriptionsResult;
import be.recipe.services.patient.ListRelationsParam;
import be.recipe.services.patient.ListRelationsResult;
import be.recipe.services.patient.ListRidsHistoryParam;
import be.recipe.services.patient.ListRidsHistoryResult;
import be.recipe.services.patient.PutVisionParam;
import be.recipe.services.patient.PutVisionResult;
import be.recipe.services.patient.RevokeRelationParam;
import be.recipe.services.patient.RevokeRelationResult;

/**
 * The Interface PatientIntegrationModuleV4.
 * 
 * @author <a href="mailto:bruno.casneuf@healthconnect.be">Bruno Casneuf</a>
 */
public interface PatientIntegrationModuleV4 extends PatientIntegrationModule {

	/**
	 * Gets the vision.
	 *
	 * @param request
	 *            the {@link GetVisionParam}
	 * @return  the {@link GetVisionResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	GetVisionResult getData(GetVisionParam request) throws IntegrationModuleException;

	/**
	 * Put the vision .
	 *
	 * @param request
	 *            the {@link PutVisionParam}
	 * @return  the {@link PutVisionResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	PutVisionResult putData(PutVisionParam request) throws IntegrationModuleException;

	/**
	 * Put the reservation data.
	 *
	 * @param data
	 *            the {@link CreateReservationParam}
	 * @return  the {@link CreateReservationResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	CreateReservationResult putData(CreateReservationParam data) throws IntegrationModuleException;

	/**
	 * Update feedback flag.
	 *
	 * @param rid
	 *            the rid
	 * @param feedbackAllowed
	 *            the feedback allowed
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	void updateFeedbackFlag(String rid, boolean feedbackAllowed) throws IntegrationModuleException;

	/**
	 * Gets the prescription status.
	 *
	 * @param data
	 *             the {@link GetPrescriptionStatusParam}
	 * @return  the {@link GetPrescriptionStatusResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	GetPrescriptionStatusResult getData(GetPrescriptionStatusParam data) throws IntegrationModuleException;

	/**
	 * Gets the prescription history.
	 *
	 * @param request
	 *             the {@link ListRidsHistoryParam}
	 * @return  the {@link ListRidsHistoryResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	ListRidsHistoryResult getData(ListRidsHistoryParam request) throws IntegrationModuleException;

	/**
	 * Gets the open prescription list.
	 *
	 * @param request
	 *             the {@link ListOpenPrescriptionsParam}
	 * @return  the {@link ListOpenPrescriptionsResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	ListOpenRidsResult getData(ListOpenRidsParam request) throws IntegrationModuleException;

	/**
	 * Put patient relation data.
	 *
	 * @param patientRelationParam
	 *             the {@link PatientRelationParam}
	 * @return  the {@link PatientRelationResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	CreateRelationResult putData(CreateRelationParam patientRelationParam) throws IntegrationModuleException;

	/**
	 * Gets the patient relation data.
	 *
	 * @param patientRelationParam
	 *            the {@link PatientRelationParam}
	 * @return  the {@link PatientRelationResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	ListRelationsResult getData(ListRelationsParam patientRelationParam) throws IntegrationModuleException;

	/**
	 * Revokes the patient relation data.
	 *
	 * @param patientRelationParam
	 *             the {@link PatientRelationParam}
	 * @return the {@link PatientRelationResult}
	 * @throws IntegrationModuleException
	 */
	RevokeRelationResult revokeData(RevokeRelationParam patientRelationParam) throws IntegrationModuleException;

}
