package be.business.connector.recipe.patient.services;

import be.business.connector.core.exceptions.IntegrationModuleException;
import be.fgov.ehealth.recipe.protocol.v1.GetPrescriptionForPatientRequest;
import be.fgov.ehealth.recipe.protocol.v1.GetPrescriptionForPatientResponse;
import be.fgov.ehealth.recipe.protocol.v1.ListPatientPrescriptionsRequest;
import be.fgov.ehealth.recipe.protocol.v1.ListPatientPrescriptionsResponse;
import be.fgov.ehealth.recipe.protocol.v1.RevokePatientPrescriptionRequest;
import be.fgov.ehealth.recipe.protocol.v1.RevokePatientPrescriptionResponse;
import be.fgov.ehealth.recipe.protocol.v1.UpdatePatientFeedbackFlagRequest;
import be.fgov.ehealth.recipe.protocol.v1.UpdatePatientFeedbackFlagResponse;

/**
 * The Interface RecipePatientService.
 */
public interface RecipePatientService {

	/**
	 * Gets the prescription for patient.
	 *
	 * @param request
	 *            the request
	 * @return the prescription for patient
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	GetPrescriptionForPatientResponse getPrescriptionForPatient(GetPrescriptionForPatientRequest request)
			throws IntegrationModuleException;

	/**
	 * Revoke prescription.
	 *
	 * @param revokePrescriptionRequest
	 *            the revoke prescription request
	 * @return the revoke patient prescription response
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	RevokePatientPrescriptionResponse revokePrescription(RevokePatientPrescriptionRequest revokePrescriptionRequest)
			throws IntegrationModuleException;

	/**
	 * List open prescriptions.
	 *
	 * @param listOpenPrescriptionsRequest
	 *            the list open prescriptions request
	 * @return the list patient prescriptions response
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	ListPatientPrescriptionsResponse listOpenPrescriptions(
			ListPatientPrescriptionsRequest listOpenPrescriptionsRequest) throws IntegrationModuleException;

	/**
	 * Update feedback flag.
	 *
	 * @param updateFeedbackFlagRequest
	 *            the update feedback flag request
	 * @return the update patient feedback flag response
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	UpdatePatientFeedbackFlagResponse updateFeedbackFlag(UpdatePatientFeedbackFlagRequest updateFeedbackFlagRequest)
			throws IntegrationModuleException;

}