package be.business.connector.recipe.patient.services;

import be.business.connector.core.exceptions.IntegrationModuleException;
import be.fgov.ehealth.recipe.protocol.v4.CreateRelationRequest;
import be.fgov.ehealth.recipe.protocol.v4.CreateRelationResponse;
import be.fgov.ehealth.recipe.protocol.v4.CreateReservationRequest;
import be.fgov.ehealth.recipe.protocol.v4.CreateReservationResponse;
import be.fgov.ehealth.recipe.protocol.v4.GetPrescriptionRequest;
import be.fgov.ehealth.recipe.protocol.v4.GetPrescriptionResponse;
import be.fgov.ehealth.recipe.protocol.v4.GetPrescriptionStatusRequest;
import be.fgov.ehealth.recipe.protocol.v4.GetPrescriptionStatusResponse;
import be.fgov.ehealth.recipe.protocol.v4.GetVisionRequest;
import be.fgov.ehealth.recipe.protocol.v4.GetVisionResponse;
import be.fgov.ehealth.recipe.protocol.v4.ListOpenPrescriptionsRequest;
import be.fgov.ehealth.recipe.protocol.v4.ListOpenPrescriptionsResponse;
import be.fgov.ehealth.recipe.protocol.v4.ListOpenRidsRequest;
import be.fgov.ehealth.recipe.protocol.v4.ListOpenRidsResponse;
import be.fgov.ehealth.recipe.protocol.v4.ListRelationsRequest;
import be.fgov.ehealth.recipe.protocol.v4.ListRelationsResponse;
import be.fgov.ehealth.recipe.protocol.v4.ListRidsHistoryRequest;
import be.fgov.ehealth.recipe.protocol.v4.ListRidsHistoryResponse;
import be.fgov.ehealth.recipe.protocol.v4.PutFeedbackFlagRequest;
import be.fgov.ehealth.recipe.protocol.v4.PutFeedbackFlagResponse;
import be.fgov.ehealth.recipe.protocol.v4.PutVisionForPatientRequest;
import be.fgov.ehealth.recipe.protocol.v4.PutVisionForPatientResponse;
import be.fgov.ehealth.recipe.protocol.v4.RevokePrescriptionRequest;
import be.fgov.ehealth.recipe.protocol.v4.RevokePrescriptionResponse;
import be.fgov.ehealth.recipe.protocol.v4.RevokeRelationRequest;
import be.fgov.ehealth.recipe.protocol.v4.RevokeRelationResponse;

// TODO: Auto-generated Javadoc
/**
 * The Interface RecipePatientServiceV4.
 */
public interface RecipePatientServiceV4 {

	/**
	 * Gets the prescription for the  patient.
	 *
	 * @param getPrescriptionForPrescriberRequest the get prescription for prescriber request
	 * @return the prescription for patient
	 * @throws IntegrationModuleException the integration module exception
	 */
	GetPrescriptionResponse getPrescriptionForPatient(GetPrescriptionRequest getPrescriptionForPrescriberRequest)
			throws IntegrationModuleException;

	/**
	 * Revokes a prescription.
	 *
	 * @param revokePrescriptionRequest the revoke prescription request
	 * @return the revoke prescription response
	 * @throws IntegrationModuleException the integration module exception
	 */
	RevokePrescriptionResponse revokePrescription(RevokePrescriptionRequest revokePrescriptionRequest) throws IntegrationModuleException;

	/**
	 * Puts a feedback flag.
	 *
	 * @param updateFeedbackFlagRequest the update feedback flag request
	 * @return the put feedback flag response
	 * @throws IntegrationModuleException the integration module exception
	 */
	PutFeedbackFlagResponse putFeedbackFlag(PutFeedbackFlagRequest updateFeedbackFlagRequest) throws IntegrationModuleException;

	/**
	 * Gets the vision.
	 *
	 * @param request the request
	 * @return the vision
	 * @throws IntegrationModuleException the integration module exception
	 */
	GetVisionResponse getVision(GetVisionRequest request) throws IntegrationModuleException;

	/**
	 * Puts a  vision.
	 *
	 * @param request the request
	 * @return the put vision for patient response
	 * @throws IntegrationModuleException the integration module exception
	 */
	PutVisionForPatientResponse putVision(PutVisionForPatientRequest request) throws IntegrationModuleException;

	/**
	 * List relations.
	 *
	 * @param request the request
	 * @return the list relations response
	 * @throws IntegrationModuleException the integration module exception
	 */
	ListRelationsResponse listRelations(ListRelationsRequest request) throws IntegrationModuleException;

	/**
	 * Creates the relation.
	 *
	 * @param request the request
	 * @return the creates the relation response
	 * @throws IntegrationModuleException the integration module exception
	 */
	CreateRelationResponse createRelation(CreateRelationRequest request) throws IntegrationModuleException;

	/**
	 * Revoke relation.
	 *
	 * @param request the request
	 * @return the revoke relation response
	 * @throws IntegrationModuleException the integration module exception
	 */
	RevokeRelationResponse revokeRelation(RevokeRelationRequest request) throws IntegrationModuleException;

	/**
	 * Creates the reservation.
	 *
	 * @param request the request
	 * @return the creates the reservation response
	 * @throws IntegrationModuleException the integration module exception
	 */
	CreateReservationResponse createReservation(CreateReservationRequest request) throws IntegrationModuleException;

	/**
	 * Gets the prescription status.
	 *
	 * @param request the request
	 * @return the prescription status
	 * @throws IntegrationModuleException the integration module exception
	 */
	GetPrescriptionStatusResponse getPrescriptionStatus(GetPrescriptionStatusRequest request) throws IntegrationModuleException;

	/**
	 * List rids history.
	 *
	 * @param request the request
	 * @return the list rids history response
	 * @throws IntegrationModuleException the integration module exception
	 */
	ListRidsHistoryResponse listRidsHistory(ListRidsHistoryRequest request) throws IntegrationModuleException;

	/**
	 * List open rids.
	 *
	 * @param listOpenPrescriptionsRequest the list open prescriptions request
	 * @return the list open rids response
	 * @throws IntegrationModuleException the integration module exception
	 */
	ListOpenRidsResponse listOpenRids(ListOpenRidsRequest listOpenPrescriptionsRequest) throws IntegrationModuleException;

	/**
	 * List open prescriptions.
	 *
	 * @param request the request
	 * @return the list open prescriptions response
	 * @throws IntegrationModuleException the integration module exception
	 */
	ListOpenPrescriptionsResponse listOpenPrescriptions(ListOpenPrescriptionsRequest request) throws IntegrationModuleException;

}