package be.business.connector.recipe.prescriber;

import java.util.List;

import be.business.connector.core.exceptions.IntegrationModuleException;
import be.recipe.services.prescriber.GetPrescriptionForPrescriberResult;
import be.recipe.services.prescriber.ListFeedbackItem;

/**
 * The Interface PrescriberIntegrationModule.
 *
 * @author <a href="mailto:bruno.casneuf@healthconnect.be">Bruno Casneuf</a>
 */
public interface PrescriberIntegrationModule {

	/**
	 * Creates the prescription.
	 *
	 * @param feedbackRequested
	 *            the feedback requested
	 * @param patientId
	 *            the patient id
	 * @param prescription
	 *            the prescription
	 * @param prescriptionType
	 *            the prescription type
	 * @return the string
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	String createPrescription(boolean feedbackRequested, String patientId, byte[] prescription, String prescriptionType)
			throws IntegrationModuleException;

	/**
	 * Revoke prescription.
	 *
	 * @param rid
	 *            the rid
	 * @param reason
	 *            the reason
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	void revokePrescription(String rid, String reason) throws IntegrationModuleException;

	/**
	 * Gets the prescription.
	 *
	 * @param rid
	 *            the rid
	 * @return the prescription
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	GetPrescriptionForPrescriberResult getPrescription(String rid) throws IntegrationModuleException;

	/**
	 * Lists the open prescriptions.
	 *
	 * @param patientId
	 *            the patient id
	 * @return the list
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	List<String> listOpenPrescription(String patientId) throws IntegrationModuleException;

	/**
	 * Lists the open prescription.
	 *
	 * @return the list
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	List<String> listOpenPrescription() throws IntegrationModuleException;

	/**
	 * Send notification.
	 *
	 * @param notificationText
	 *            the notification text
	 * @param patientId
	 *            the patient id
	 * @param executorId
	 *            the executor id
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	void sendNotification(byte[] notificationText, String patientId, String executorId) throws IntegrationModuleException;

	/**
	 * Update feedback flag.
	 *
	 * @param rid
	 *            the rid
	 * @param feedbackAllowed
	 *            the feedback allowed
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	void updateFeedbackFlag(String rid, boolean feedbackAllowed) throws IntegrationModuleException;

	/**
	 * List the {@link ListFeedbackItem}}.
	 *
	 * @param readFlag
	 *            the read flag
	 * @return the list
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	List<ListFeedbackItem> listFeedback(boolean readFlag) throws IntegrationModuleException;

	/**
	 * Ping.
	 *
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	void ping() throws IntegrationModuleException;

	/**
	 * Sets the personal password.
	 *
	 * @param personalPassword
	 *            the new personal password
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	void setPersonalPassword(String personalPassword) throws IntegrationModuleException;

	/**
	 * Prepare create prescription.
	 *
	 * @param patientId
	 *            the patient id
	 * @param prescriptionType
	 *            the prescription type
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	void prepareCreatePrescription(String patientId, String prescriptionType) throws IntegrationModuleException;

}