package be.business.connector.recipe.prescriber;

import be.business.connector.core.exceptions.IntegrationModuleException;
import be.business.connector.recipe.prescriber.domain.ListPrescriptionHistoryParam;
import be.business.connector.recipe.prescriber.domain.ListPrescriptionHistoryResult;
import be.recipe.services.prescriber.GetPrescriptionStatusParam;
import be.recipe.services.prescriber.GetPrescriptionStatusResult;
import be.recipe.services.prescriber.ListOpenRidsParam;
import be.recipe.services.prescriber.ListOpenRidsResult;
import be.recipe.services.prescriber.ListRidsHistoryParam;
import be.recipe.services.prescriber.ListRidsHistoryResult;
import be.recipe.services.prescriber.PutVisionParam;
import be.recipe.services.prescriber.PutVisionResult;
import be.recipe.services.prescriber.ValidationPropertiesParam;
import be.recipe.services.prescriber.ValidationPropertiesResult;

/**
 * The Interface PrescriberIntegrationModuleDevV4.
 */
public interface PrescriberIntegrationModuleDevV4 extends PrescriberIntegrationModuleV4 {

	/**
	 * Creates the prescription.
	 *
	 * @param feedbackRequested
	 *            the feedback requested
	 * @param patientId
	 *            the patient id
	 * @param prescription
	 *            the prescription
	 * @param prescriptionType
	 *            the prescription type
	 * @param visibility
	 *            the visibility
	 * @param expirationDate
	 *            the expiration date
	 * @return the rid
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	String createPrescription(boolean feedbackRequested, String patientId, byte[] prescription, String prescriptionType, String visibility,
			String expirationDate) throws IntegrationModuleException;

	/**
	 * Gets the prescription status.
	 *
	 * @param data
	 *            the {@link GetPrescriptionStatusParam}
	 * @return the {@link GetPrescriptionStatusResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	GetPrescriptionStatusResult getData(GetPrescriptionStatusParam data) throws IntegrationModuleException;

	/**
	 * Lists the prescription history.
	 *
	 * @param request
	 *            the {@link ListPrescriptionHistoryParam}
	 * @return the {@link ListPrescriptionHistoryResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	ListRidsHistoryResult getData(ListRidsHistoryParam request) throws IntegrationModuleException;

	/**
	 * Put the vision.
	 *
	 * @param request
	 *            the {@link PutVisionForPrescriberParam}
	 * @return the {@link PutVisionForPrescriberResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	PutVisionResult putData(PutVisionParam request) throws IntegrationModuleException;

	/**
	 * Gets the open prescription list.
	 *
	 * @param data
	 *            the {@link ListOpenRidsParam}
	 * @return the {@link ListOpenRidsResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	ListOpenRidsResult getData(ListOpenRidsParam data) throws IntegrationModuleException;

	/**
	 * Gets the validation properties.
	 *
	 * @param data
	 *            the {@link ValidationPropertiesParam}
	 * @return the {@link ValidationPropertiesResult}
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	ValidationPropertiesResult getData(ValidationPropertiesParam data) throws IntegrationModuleException;

}