package be.business.connector.recipe.prescriber.services;

import be.business.connector.core.exceptions.IntegrationModuleException;
import be.recipe.services.prescriber.CreatePrescription;
import be.recipe.services.prescriber.CreatePrescriptionResponse;
import  be.recipe.services.prescriber.GetPrescriptionForPrescriber;
import  be.recipe.services.prescriber.GetPrescriptionForPrescriberResponse;
import be.recipe.services.prescriber.GetPrescriptionStatus;
import be.recipe.services.prescriber.GetPrescriptionStatusResponse;
import  be.recipe.services.prescriber.ListFeedbacks;
import  be.recipe.services.prescriber.ListFeedbacksResponse;
import be.recipe.services.prescriber.ListOpenRids;
import be.recipe.services.prescriber.ListRidsHistory;
import be.recipe.services.prescriber.ListRidsHistoryResponse;
import be.recipe.services.prescriber.PutVision;
import be.recipe.services.prescriber.PutVisionResponse;
import  be.recipe.services.prescriber.RevokePrescription;
import  be.recipe.services.prescriber.RevokePrescriptionResponse;
import  be.recipe.services.prescriber.SendNotification;
import  be.recipe.services.prescriber.SendNotificationResponse;
import be.recipe.services.prescriber.UpdateFeedbackFlag;
import be.recipe.services.prescriber.UpdateFeedbackFlagResponse;
import be.recipe.services.prescriber.ValidationProperties;
import be.recipe.services.prescriber.ValidationPropertiesResponse;

/**
 * The Interface RecipePrescriberServiceV4.
 */
public interface RecipePrescriberServiceDevV4 {

	/**
	 * Creates the prescription.
	 *
	 * @param request
	 *            the request
	 * @return the creates the prescription response
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	CreatePrescriptionResponse createPrescription(CreatePrescription request) throws IntegrationModuleException;

	/**
	 * Revoke prescription.
	 *
	 * @param revokePrescriptionRequest
	 *            the revoke prescription request
	 * @return the revoke prescription response
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	RevokePrescriptionResponse revokePrescription(RevokePrescription revokePrescriptionRequest) throws IntegrationModuleException;

	/**
	 * Gets the prescription for.
	 *
	 * @param getPrescriptionForPrescriberRequest            the get prescription for prescriber request
	 * @return the prescription for prescriber
	 * @throws IntegrationModuleException             the integration module exception
	 */
	GetPrescriptionForPrescriberResponse getPrescriptionForPrescriber(GetPrescriptionForPrescriber getPrescriptionForPrescriberRequest)
			throws IntegrationModuleException;

	/**
	 * List open prescriptions.
	 *
	 * @param listOpenPrescriptionsRequest
	 *            the list open prescriptions request
	 * @return the list open prescription response
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	be.recipe.services.prescriber.ListOpenRidsResponse listOpenRids(ListOpenRids listOpenPrescriptionsRequest)
			throws IntegrationModuleException;


	/**
	 * Send notification.
	 *
	 * @param sendNotificationRequest
	 *            the send notification request
	 * @return the send notification response
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	SendNotificationResponse sendNotification(SendNotification sendNotificationRequest) throws IntegrationModuleException;

	/**
	 * Update feedback flag.
	 *
	 * @param updateFeedbackFlagRequest
	 *            the update feedback flag request
	 * @return the update feedback flag response
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	UpdateFeedbackFlagResponse putFeedbackFlag(UpdateFeedbackFlag updateFeedbackFlagRequest) throws IntegrationModuleException;

	/**
	 * List feedbacks.
	 *
	 * @param listFeedbacksRequest
	 *            the list feedbacks request
	 * @return the list feedbacks response
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	ListFeedbacksResponse listFeedbacks(ListFeedbacks listFeedbacksRequest) throws IntegrationModuleException;

//	/**
//	 * Alive check.
//	 *
//	 * @param paramAliveCheckRequest
//	 *            the param alive check request
//	 * @return the ping response
//	 * @throws IntegrationModuleException
//	 *             the integration module exception
//	 */
//	PingResponse aliveCheck(Ping paramAliveCheckRequest) throws IntegrationModuleException;

	/**
	 * Gets the validation properties.
	 *
	 * @param request the request
	 * @return the validation properties
	 * @throws IntegrationModuleException the integration module exception
	 */
	ValidationPropertiesResponse getValidationProperties(ValidationProperties request) throws IntegrationModuleException;

	/**
	 * Gets the prescription status.
	 *
	 * @param request the request
	 * @return the prescription status
	 * @throws IntegrationModuleException the integration module exception
	 */
	GetPrescriptionStatusResponse getPrescriptionStatus(GetPrescriptionStatus request) throws IntegrationModuleException;

//	/**
//	 * List open prescriptions.
//	 *
//	 * @param request the request
//	 * @return the list open rids response
//	 * @throws IntegrationModuleException the integration module exception
//	 */
//	ListOpenPrescriptionResponse listOpenPrescription(ListOpenPrescription request) throws IntegrationModuleException;

	/**
	 * Put vision.
	 *
	 * @param request the request
	 * @return the put vision response
	 * @throws IntegrationModuleException the integration module exception
	 */
	PutVisionResponse putVisionForPrescriber(PutVision request) throws IntegrationModuleException;

	/**
	 * List prescription history.
	 *
	 * @param request the request
	 * @return the list rid history response
	 * @throws IntegrationModuleException the integration module exception
	 */
	ListRidsHistoryResponse listRidsHistory(ListRidsHistory request) throws IntegrationModuleException;

}
