package be.business.connector.recipe.prescriber.services;

import be.business.connector.core.exceptions.IntegrationModuleException;
import be.fgov.ehealth.recipe.protocol.v4.CreatePrescriptionRequest;
import be.fgov.ehealth.recipe.protocol.v4.CreatePrescriptionResponse;
import be.fgov.ehealth.recipe.protocol.v4.GetPrescriptionRequest;
import be.fgov.ehealth.recipe.protocol.v4.GetPrescriptionResponse;
import be.fgov.ehealth.recipe.protocol.v4.GetPrescriptionStatusRequest;
import be.fgov.ehealth.recipe.protocol.v4.GetPrescriptionStatusResponse;
import be.fgov.ehealth.recipe.protocol.v4.GetValidationPropertiesRequest;
import be.fgov.ehealth.recipe.protocol.v4.GetValidationPropertiesResponse;
import be.fgov.ehealth.recipe.protocol.v4.ListFeedbacksRequest;
import be.fgov.ehealth.recipe.protocol.v4.ListFeedbacksResponse;
import be.fgov.ehealth.recipe.protocol.v4.ListOpenRidsRequest;
import be.fgov.ehealth.recipe.protocol.v4.ListOpenRidsResponse;
import be.fgov.ehealth.recipe.protocol.v4.ListRidsHistoryRequest;
import be.fgov.ehealth.recipe.protocol.v4.ListRidsHistoryResponse;
import be.fgov.ehealth.recipe.protocol.v4.PutFeedbackFlagRequest;
import be.fgov.ehealth.recipe.protocol.v4.PutFeedbackFlagResponse;
import be.fgov.ehealth.recipe.protocol.v4.PutVisionForPrescriberRequest;
import be.fgov.ehealth.recipe.protocol.v4.PutVisionForPrescriberResponse;
import be.fgov.ehealth.recipe.protocol.v4.RevokePrescriptionRequest;
import be.fgov.ehealth.recipe.protocol.v4.RevokePrescriptionResponse;
import be.fgov.ehealth.recipe.protocol.v4.SendNotificationRequest;
import be.fgov.ehealth.recipe.protocol.v4.SendNotificationResponse;

/**
 * The Interface RecipePrescriberServiceV4.
 */
public interface RecipePrescriberServiceV4 {

	/**
	 * Creates the prescription.
	 *
	 * @param request
	 *            the request
	 * @return the creates the prescription response
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	CreatePrescriptionResponse createPrescription(CreatePrescriptionRequest request) throws IntegrationModuleException;

	/**
	 * Revoke prescription.
	 *
	 * @param revokePrescriptionRequest
	 *            the revoke prescription request
	 * @return the revoke prescription response
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	RevokePrescriptionResponse revokePrescription(RevokePrescriptionRequest revokePrescriptionRequest) throws IntegrationModuleException;

	/**
	 * Gets the prescription for.
	 *
	 * @param getPrescriptionForPrescriberRequest            the get prescription for prescriber request
	 * @return the prescription for prescriber
	 * @throws IntegrationModuleException             the integration module exception
	 */
	GetPrescriptionResponse getPrescriptionForPrescriber(GetPrescriptionRequest getPrescriptionForPrescriberRequest)
			throws IntegrationModuleException;

	/**
	 * List open prescriptions.
	 *
	 * @param listOpenPrescriptionsRequest
	 *            the list open prescriptions request
	 * @return the list open prescription response
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	ListOpenRidsResponse listOpenRids(ListOpenRidsRequest listOpenPrescriptionsRequest)
			throws IntegrationModuleException;


	/**
	 * Send notification.
	 *
	 * @param sendNotificationRequest
	 *            the send notification request
	 * @return the send notification response
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	SendNotificationResponse sendNotification(SendNotificationRequest sendNotificationRequest) throws IntegrationModuleException;

	/**
	 * Update feedback flag.
	 *
	 * @param updateFeedbackFlagRequest
	 *            the update feedback flag request
	 * @return the update feedback flag response
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	PutFeedbackFlagResponse putFeedbackFlag(PutFeedbackFlagRequest updateFeedbackFlagRequest) throws IntegrationModuleException;

	/**
	 * List feedbacks.
	 *
	 * @param listFeedbacksRequest
	 *            the list feedbacks request
	 * @return the list feedbacks response
	 * @throws IntegrationModuleException
	 *             the integration module exception
	 */
	ListFeedbacksResponse listFeedbacks(ListFeedbacksRequest listFeedbacksRequest) throws IntegrationModuleException;


	/**
	 * Gets the validation properties.
	 *
	 * @param request the request
	 * @return the validation properties
	 * @throws IntegrationModuleException the integration module exception
	 */
	GetValidationPropertiesResponse getValidationProperties(GetValidationPropertiesRequest request) throws IntegrationModuleException;

	/**
	 * Gets the prescription status.
	 *
	 * @param request the request
	 * @return the prescription status
	 * @throws IntegrationModuleException the integration module exception
	 */
	GetPrescriptionStatusResponse getPrescriptionStatus(GetPrescriptionStatusRequest request) throws IntegrationModuleException;

	/**
	 * Put vision.
	 *
	 * @param request the request
	 * @return the put vision response
	 * @throws IntegrationModuleException the integration module exception
	 */
	PutVisionForPrescriberResponse putVisionForPrescriber(PutVisionForPrescriberRequest request) throws IntegrationModuleException;

	/**
	 * List prescription history.
	 *
	 * @param request the request
	 * @return the list rid history response
	 * @throws IntegrationModuleException the integration module exception
	 */
	ListRidsHistoryResponse listRidsHistory(ListRidsHistoryRequest request) throws IntegrationModuleException;

}
